<?php

declare(strict_types=1);

/** webEdition CMS
 *
 * $Rev: 66d79e25ceb9 $
 * $Author: Marc Kraemer <marc.kraemer@webedition.org> $
 * $Date: 2021-01-25 22:14 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_base_sessionHandler implements SessionHandlerInterface, SessionUpdateTimestampHandlerInterface{

	public const CRYPT_NONE = 0;
	public const CRYPT_TRANSPARENT = 1;
	public const CRYPT_COOKIE = 2;

//prevent crashed or killed sessions to stay

	/** @var int */
	private $execTime;

	/** @var string */
	private $sessionName;

	/** @var we_database_base */
	private $DB;

	/** @var string */
	private $id = '';

	/** @var string */
	private $crypt = '';

	/** @var string */
	private $hash = '';

	/** @var array<string,mixed> */
	public static $releaseError = [];

	/** @var float */
	public static $acquireLock = 0;

//	private static $debug = false;

	public function __construct(){
		if(session_status() === PHP_SESSION_ACTIVE){
			return;
		}
		/* if(!empty($_REQUEST['debug'])){
		  self::$debug = true;
		  } */
		if($this->id){
			session_start();
			return;
		}
		session_set_save_handler($this, true);
		$this->DB = new DB_WE();
		$this->execTime = max(min(60, ((int) get_cfg_var('max_execution_time'))), 5); //time might be wrong (1&1); make exectime at least 5 seconds which is quite small
		$this->id = md5(uniqid('', true));
		if(defined('SYSTEM_WE_SESSION_CRYPT') && SYSTEM_WE_SESSION_CRYPT){
			// due to IE we can't use HTTP_ACCEPT_LANGUAGE, HTTP_ACCEPT_ENCODING - they change the string on each request
			$key = $_SERVER['WE_ORG_ROOT'] . (!empty($_SERVER['HTTP_USER_AGENT']) ? $_SERVER['HTTP_USER_AGENT'] : 'HTTP_USER_AGENT');
			if(SYSTEM_WE_SESSION_CRYPT === self::CRYPT_COOKIE){
				if(!isset($_COOKIE['secure'])){
					$cookie = session_get_cookie_params();
					$_COOKIE['secure'] = we_users_user::getHashIV(30);
					if(version_compare(PHP_VERSION, '7.3.0') >= 0){
						setcookie('secure', (string) $_COOKIE['secure'], ['expires' => 0, 'path' => '/', 'secure' => we_http_util::isHttps(), 'httponly' => true, 'samesite' => 'strict']);
					} else {//FIXME: remove in cardada
						setcookie('secure', (string) $_COOKIE['secure'], 0, '/', $cookie['domain'], we_http_util::isHttps(), true);
					}
				}
				$key .= $_COOKIE['secure'];
			}
			$this->crypt = hash('haval224,4', $key);
			//double key size is needed
			$this->crypt .= $this->crypt;
		}
		session_start();
	}

	public function __destruct(){
		if(session_status() === PHP_SESSION_ACTIVE){
			session_write_close();
		}
	}

	public function open(/* string */$savePath, /* string */ $sessName)/* :string */{
		$this->sessionName = $sessName;
		return true;
	}

	public function close(): bool{//FIX for php >5.5, where write is only called, if sth. in session changed
		/* if(self::$debug){
		  t_e(session_id());
		  } */
		return true;
	}

	public function read(/* string */$sessID)/* : string */{
		if(!$this->validateId($sessID)){
			$sessID = $this->session_create_id();
		}
		//$sessID = $this->DB->escape(self::getSessionID($sessID));
		$lock = microtime(true);
		if(fB('SELECT 1 FROM ' . SESSION_TABLE . ' WHERE touch>=NOW()-INTERVAL ' . SYSTEM_WE_SESSION_TIME . ' second AND session_id=x\'' . $sessID . '\' AND sessionName="' . $this->sessionName . '"')){//session exists
			$max = $this->execTime * 6; //just wait 80% of the time to clear the lock, if not cleared, break the lock

			while(!($this->DB->query('UPDATE ' . SESSION_TABLE . ' SET
	lock_id=x\'' . $this->id . '\',
	lockTime=NOW()
WHERE
	session_id=x\'' . $sessID . '\' AND
	sessionName="' . $this->sessionName . '"' . ( --$max ? ' AND
	(lock_id IS NULL OR lock_id=x\'' . $this->id . '\' OR lockTime+INTERVAL ' . $this->execTime . ' second<NOW())' : '')
			) &&
			$this->DB->affected_rows())){
				self::$releaseError = self::$releaseError ?: getHash('SELECT sessionName,HEX(session_id) AS session_id,lockTime,HEX(lock_id) AS lock_id,touch,NOW()'/* ,debug */ . ' FROM ' . SESSION_TABLE . ' WHERE session_id=x\'' . $sessID . '\' AND sessionName="' . $this->sessionName . '"');
				usleep(250000);
			}
			$hash = 'sess_' . $this->sessionName . $sessID;
			//now we have a session lock, read data
			try{
				$data = we_cache_we::load($hash);
			}catch(we_exception_cache $ex){
				$data = fS('SELECT session_data FROM ' . SESSION_TABLE . ' WHERE	session_id=x\'' . $sessID . '\' AND	sessionName="' . $this->sessionName . '"');
				we_cache_we::save($hash, $data, (int) SYSTEM_WE_SESSION_TIME);
			}

			self::$acquireLock = microtime(true) - $lock;
			if(!$max){
				self::$releaseError['myLock'] = $this->id;
			} else {
				self::$releaseError = [];
			}

			if($data){
				$data = ($data[0] === '$' && $this->crypt ? we_customer_customer::decryptData($data, $this->crypt) : $data);
				if($data && $data[0] === 'x'){
					//valid gzip
					$data = (string) (gzuncompress($data) ?: '');
					$this->hash = md5($sessID . $data, true);
					return $data;
				}//else we need a new sessionid; if decrypt failed we might else destroy an existing valid session
			}
		}
		self::$acquireLock = microtime(true) - $lock;
		//if we don't find valid data, generate a new ID because of session stealing
		$this->write($sessID, '', true);
		return '';
	}

	public function write($sessID, $sessData, $lock = false): bool{
		if(!$sessData && !$lock){
			return $this->destroy($sessID);
		}
		$hash = 'sess_' . $this->sessionName . $sessID;

		if(md5($sessID . $sessData, true) == $this->hash){//if nothing changed,we don't have to bother the db
			$this->DB->query('UPDATE ' . SESSION_TABLE . ' SET ' .
					we_database_base::arraySetter([
						'lock_id' => sql_function($lock ? 'x\'' . $this->id . '\'' : 'NULL'),
						'lockTime' => sql_function($lock ? 'NOW()' : 'NULL'),
					]) . ' WHERE session_id=x\'' . $sessID . '\' AND sessionName="' . $this->sessionName . '"');
			we_cache_we::touch($hash, (int) SYSTEM_WE_SESSION_TIME);
			if($this->DB->affected_rows()){//make sure we had an successfull update
				return true;
			}
		}

		$sessData = SYSTEM_WE_SESSION_CRYPT && $this->crypt ? we_customer_customer::cryptData(gzcompress($sessData, 4) ?: '', $this->crypt, true) : (gzcompress($sessData, 4) ?: '');

		$this->DB->query('REPLACE INTO ' . SESSION_TABLE . ' SET ' . we_database_base::arraySetter([
					'sessionName' => $this->sessionName,
					'session_id' => sql_function('x\'' . $sessID . '\''),
					'session_data' => sql_function('x\'' . bin2hex($sessData) . '\''),
					'lock_id' => sql_function($lock ? 'x\'' . $this->id . '\'' : 'NULL'),
					'lockTime' => sql_function($lock ? 'NOW()' : 'NULL'),
						/* 'debug' => print_r([
						  'URI' => $_SERVER['REQUEST_URI'],
						  'oldLock'=>bin2hex($this->id),
						  'Cookie' => $_COOKIE,
						  'Req' => $_REQUEST,
						  ], true) */
		]));
		we_cache_we::save($hash, $sessData, (int) SYSTEM_WE_SESSION_TIME);
		return true;
	}

	public function destroy($sessID): bool{
		unset($_SESSION);
		self::delSess($sessID, $this->sessionName);
		return true;
	}

	private static function delSess(string $sessID, string $sessName): void{
		DB_WE::$inst->query('DELETE FROM ' . SESSION_TABLE . ' WHERE session_id=x\'' . $sessID . '\' AND sessionName="' . $sessName . '"');
		we_cache_we::clean('sess_' . $sessName . $sessID);
		self::cleanSessions();
	}

	public function gc($sessMaxLifeTime): bool{
		self::cleanSessions();
		return true;
	}

	public static function cleanSessions(): void{
		DB_WE::$inst->query('DELETE FROM ' . SESSION_TABLE . ' WHERE touch<NOW()-INTERVAL ' . SYSTEM_WE_SESSION_TIME . ' second');
		DB_WE::$inst->query('OPTIMIZE TABLE ' . SESSION_TABLE);
	}

	private static function session_create_id(): string{
		session_regenerate_id(false); // @phan-suppress-current-line PhanPluginUseReturnValueInternalKnown
		return session_id();
	}

	static function makeNewID(bool $destroy = false): void{
		if($destroy){
			session_destroy();
			session_start();
			return;
		}
		//we need a new lock on the generated id, since partial data is sent to the browser, subsequent calls with the new sessionid might happen
		self::delSess(session_id(), session_name());
		self::session_create_id();
		session_write_close();
		session_start();
	}

	public static function getCurrentHex(): string{
		$sessID = session_id();
		return preg_match('|^([a-f0-9]){32,40}$|', $sessID) ? $sessID : bin2hex($sessID);
	}

	public function updateTimestamp($sessID, $val){
		$this->DB->query('UPDATE ' . SESSION_TABLE . ' SET lock_id=NULL,lockTime=NULL WHERE session_id=x\'' . $sessID . '\' AND sessionName="' . $this->sessionName . '" AND lock_id=x\'' . $this->id . '\'');
		we_cache_we::touch('sess_' . $this->sessionName . $sessID, (int) SYSTEM_WE_SESSION_TIME);
		return true;
	}

	public function validateId($sessID){
		return preg_match('|^([a-f0-9]){32,40}$|', $sessID) === 1;
	}

}
