<?php

declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 345f4a15685a $
 * $Author: Marc Kraemer <marc.kraemer@webedition.org> $
 * $Date: 2021-01-25 18:20 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
abstract class we_cache_db implements we_cache_base{

	private const PREFIX = 'WEdb_';
	private const DBSET = 'DB_SET';
	private const DBSET_EXP = 3600;

	protected static $dbSet = 0;

	/** @var we_cache_backend */
	protected static $cache;

	/** @var array<string,array> */
	private static $queryCache = [];

	public static function setup(): void{
		if(self::$cache || !defined('MEMCACHE_DATABASE') || !MEMCACHE_DATABASE){
			return;
		}
		$id = 'db_' . dechex(crc32(DB_DATABASE . DB_USER . TBL_PREFIX));
		self::$cache = we_cache_backendNone::getBackend(MEMCACHE_BACKEND, $id);
		try{
			self::$dbSet = self::$cache->get(self::DBSET);
		}catch(we_exception_cache $e){
			self::$dbSet = 1;
			self::$cache->set(self::DBSET, 1, self::DBSET_EXP); /* after 1 hour the number can be reset */
		}
	}

	public static function reset(array $servers): void{
		if(!MEMCACHE_DATABASE){
			return;
		}
		self::$cache->quit();
		self::$cache->setServers($servers);
		return;
	}

	/**
	 *
	 * @param string $entry
	 * @return array
	 * @throws we_exception_cache
	 */
	public static function load(string $entry){
		$entry = base64_encode(md5($entry, true));
		if(isset(self::$queryCache[$entry])){
			return self::$queryCache[$entry];
		}
		if(!defined('MEMCACHE_DATABASE') || !MEMCACHE_DATABASE){
			throw new we_exception_cache();
		}
		return (array) self::$cache->get(self::$dbSet . '_' . $entry);
	}

	public static function save(string $entry, $data, int $expiry = 300, $keepLocal = false): bool{
		if(!$entry || str_contains($entry, SESSION_TABLE) || defined('DISABLE_DB_CACHE')){//don't store session info
			return false;
		}
		$entry = base64_encode(md5($entry, true));
		self::$queryCache[$entry] = $data;
		if($keepLocal || !defined('MEMCACHE_DATABASE') || !MEMCACHE_DATABASE){
			return true;
		}
		return self::$cache->set(self::$dbSet . '_' . $entry, $data, $expiry);
	}

	public static function clean(string $pattern = ''): bool{
		if(str_contains($pattern, SESSION_TABLE)){//don't store session info
			return true;
		}
		self::$queryCache = [];
		if(!defined('MEMCACHE_DATABASE') || !MEMCACHE_DATABASE){
			return true;
		}
		self::$dbSet = self::$cache->increment(self::DBSET); /* after 1 hour the number can be reset */
		return true;
	}

	public static function touch(string $entry, int $expiry = 300): bool{
		if(!MEMCACHE_DATABASE){
			return true;
		}
		self::$cache->touch(self::DBSET, self::DBSET_EXP);
		self::$cache->touch(self::$dbSet . '_' . base64_encode(md5($entry, true)), $expiry);
		return true;
	}

}
