<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 1777123537e7 $
 * $Author: Lukas Imhof $
 * $Date: 2021-01-19 11:01 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_collection_item{
	public const CLASS_YES = 'we-state-green';
	public const CLASS_NO = 'we-state-red';
	public const CLASS_NONE = 'we-state-none';

	/** @var int */
	public $id = -1;

	/** @var int */
	public $tableid = -1;

	/** @var string */
	public $path = '';

	/** @var string */
	public $ct = '';

	/** @var string */
	public $name = '';

	/** @var string */
	public $ext = '';

	/** @var array */
	public $elements = [
		'attrib_title' => [
			'Dat' => '',
			'state' => self::CLASS_NONE,
			'write' => ''
		],
		'attrib_alt' => [
			'Dat' => '',
			'state' => self::CLASS_NONE,
			'write' => ''
		],
		'meta_title' => [
			'Dat' => '',
			'state' => self::CLASS_NONE,
			'write' => ''
		],
		'meta_description' => [
			'Dat' => '',
			'state' => self::CLASS_NONE,
			'write' => ''
		],
		'custom' => [
			'type' => '',
			'Dat' => '',
			'BDID' => 0
		]
	];

	/** @var array */
	public $icon = [
		'imageView' => '',
		'imageViewPopup' => '',
		'sizeX' => 0,
		'sizeY' => 0,
	];

	public function __construct(int $id = -1, string $path = '', string $ct = '', int $tableid = 0, string $name = '', string $ext = ''){
		$this->id = $id;
		$this->path = $path;
		$this->ct = $ct;
		$this->tableid = $tableid;
		$this->name = $name;
		$this->ext = $ext;
	}

	public function updateElements(array $hasMeta): void{
		$c = 0;
		foreach(['attrib_title', 'attrib_alt', 'meta_title', 'meta_description'] as $name){
			$cur = &$this->elements[$name];
			'@phan-var array{Dat:string,state:string} $cur';
			$cur['write'] = $cur['Dat'] && in_array($this->ct, $hasMeta, true) && $c++ < 2 ? $cur['Dat'] : '';
			switch($name){
				case 'attrib_title':
					$cur['state'] = $this->ct !== 'image/*' ? self::CLASS_NONE : ($cur['Dat'] ? self::CLASS_YES : self::CLASS_NO);
					$cur['Dat'] = $cur['Dat'] ? g_l('weClass', '[collection][attr_title]') . ': ' . $cur['Dat'] : ($cur['state'] === self::CLASS_NONE ? '' : g_l('weClass', '[collection][attr_title]') . ': ' . g_l('weClass', '[collection][notSet]'));
					break;
				case 'attrib_alt':
					$cur['state'] = $this->ct !== 'image/*' ? self::CLASS_NONE : ($cur['Dat'] ? self::CLASS_YES : self::CLASS_NO);
					$cur['Dat'] = $cur['Dat'] ? g_l('weClass', '[collection][attr_alt]') . ': ' . $cur['Dat'] : ($cur['state'] === self::CLASS_NONE ? '' : g_l('weClass', '[collection][attr_alt]') . ': ' . g_l('weClass', '[collection][notSet]'));
					break;
				case 'meta_title':
					$cur['state'] = !in_array($this->ct, $hasMeta, true) ? self::CLASS_NONE : ($cur['Dat'] ? self::CLASS_YES : self::CLASS_NO);
					$cur['Dat'] = $cur['Dat'] && in_array($this->ct, $hasMeta, true) ? g_l('weClass', '[Title]') . ': ' . $cur['Dat'] : ($cur['state'] === self::CLASS_NONE ? '' : g_l('weClass', '[Title]') . ': ' . g_l('weClass', '[collection][notSet]'));
					break;
				case 'meta_description':
					$cur['state'] = !in_array($this->ct, $hasMeta, true) ? self::CLASS_NONE : ($cur['Dat'] ? self::CLASS_YES : self::CLASS_NO);
					$cur['Dat'] = $cur['Dat'] && in_array($this->ct, $hasMeta, true) ? g_l('weClass', '[Description]') . ': ' . $cur['Dat'] : ($cur['state'] === self::CLASS_NONE ? '' : g_l('weClass', '[Description]') . ': ' . g_l('weClass', '[collection][notSet]')); //
					break;
				default:
			}
		}
	}

}
