<?php

declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 9c89d948646f $
 * $Author: Marc Kraemer <marc.kraemer@webedition.org> $
 * $Date: 2020-12-23 12:27 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
//@phan-file-suppress PhanUndeclaredClassMethod,PhanUndeclaredMethod,PhanUndeclaredExtendedClass
use Sabre\DAV;

abstract class we_davfs_dbFile extends DAV\File{

	/** @phan-var string */
	protected $path;

	/** @phan-var int */
	protected $ID = 0;

	/** @phan-var string */
	protected $Table;

	function __construct(string $path, string $table, int $id){
		$this->path = $path;
		$this->Table = $table;
		$this->ID = $id ?: path_to_id($this->path, $this->Table);
	}

	/**
	 *
	 * @return string
	 */
	function getName(){
		return basename($this->path);
	}

	/**
	 *
	 * @return int
	 */
	function getSize(){
		return fI('SELECT LENGTH(c.Dat) FROM ' . CONTENT_TABLE . ' c WHERE c.DocumentTable="' . stripTblPrefix($this->Table) . '" AND c.nHash=x\'' . md5('data') . '\' AND c.DID=' . $this->ID);
	}

	/**
	 * @return string
	 */
	function getETag(){
		return '"' . fS('SELECT CONCAT_WS("-",c.DID,c.DocumentTable,MD5(c.Dat)) FROM ' . CONTENT_TABLE . ' c WHERE c.DocumentTable="' . stripTblPrefix($this->Table) . '" AND c.nHash=x\'' . md5('data') . '\' AND c.DID=' . $this->ID) . '"';
	}

	/**
	 * @return string
	 */
	public function getContentType(){
		return 'text/plain';
	}

	/**
	 * @return int
	 */
	public function getLastModified(){
		return fI('SELECT UNIX_TIMESTAMP(Mod_Date) FROM ' . $this->Table . ' WHERE ID=' . $this->ID);
	}

	protected function doPatch(string $content, string $newContent, int $rangeType, int $offset): string{
		switch($rangeType){
			case DAV\PartialUpdate\Plugin::RANGE_APPEND://append
				return $content . $newContent;
			case DAV\PartialUpdate\Plugin::RANGE_START:
				return mb_substr($content, 0, $offset) . $newContent . mb_substr($content, $offset + strlen($newContent));
			case DAV\PartialUpdate\Plugin::RANGE_END:
				return mb_substr($content, 0, ($offset < 0 ? $offset : -1 * $offset)) . $newContent;
		}
		return '';
	}

	protected function getLockedDocument(we_document_files $we_doc): string{
		$dat = getHash('SELECT username,CONCAT(First," ",Second,"(",username,")") AS Name FROM ' . USER_TABLE . ' WHERE ID=' . $we_doc->isLockedByUser());
		return sprintf(g_l('alert', '[file_locked_footer]'), $dat['Name']);
	}

}
