<?php

declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 4bec8f6a4c70 $
 * $Author: Marc Kraemer <marc.kraemer@webedition.org> $
 * $Date: 2020-12-15 12:10 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
abstract class we_dialog_formmailBlockLog{

	public static function getDialog(): void{
		if(we_base_request::bool('clearlog')){
			DB_WE::$inst->query('TRUNCATE ' . FORMMAIL_BLOCK_TABLE);
		} else if(($id = we_base_request::int('clearEntry'))){
			DB_WE::$inst->query('DELETE FROM ' . FORMMAIL_BLOCK_TABLE . ' WHERE id=' . $id);
		}

		function getContents(): we_html_multiIconBoxContent{
			$headline = [
				we_html_element::b(g_l('prefs', '[ip_address]')),
				we_html_element::b(g_l('prefs', '[blocked_until]')),
				'',
			];

			$content = [];

			$count = 15;
			$start = max(we_base_request::int('start', 0), 0);

			$num_all = fI('SELECT COUNT(1) FROM ' . FORMMAIL_BLOCK_TABLE);

			$all = DB_WE::$inst->getAllq('SELECT * FROM ' . FORMMAIL_BLOCK_TABLE . ' ORDER BY blockedUntil DESC LIMIT ' . $start . ',' . $count);
			if($all){
				foreach($all as $item){
					$content[] = [
						$item['ip'],
						($item['blockedUntil'] == -1 ?
						htmlspecialchars(g_l('prefs', '[forever]')) :
						date(g_l('weEditorInfo', '[date_format]'), (int) $item['blockedUntil'])),
						we_html_element::a(['href' => "javascript:clearEntry(" . $item['id'] . ",'" . $item['ip'] . "')"], g_l('prefs', '[unblock]'))
					];
				}

				$next = $start + $count;

				$nextprev = '<table style="margin-top: 10px;" class="default"><tr><td style="padding-right:20px;">' .
						($start > 0 ?
						we_html_button::create_button(we_html_button::BACK, WEBEDITION_DIR . 'we_cmd.php?we_cmd[0]=we_gui_showFormmailBlockLog&start=' . ($start - $count)) : //bt_back
						we_html_button::create_button(we_html_button::BACK, "", '', "", true)) .
						"</td><td style='text-align:center' class='defaultfont' width='120'><b>" . ($start + 1) . "&nbsp;-&nbsp;" .
						min($num_all, $start + $count) .
						"&nbsp;" . g_l('global', '[from]') . " " . ($num_all) . '</b></td><td style="padding-right:20px;">' .
						($next < $num_all ?
						we_html_button::create_button(we_html_button::NEXT, WEBEDITION_DIR . "we_cmd.php?we_cmd[0]=we_gui_showFormmailBlockLog&start=" . $next) : //bt_next
						we_html_button::create_button(we_html_button::NEXT, "", "", "", true)) .
						'</td></tr></table>';

				return (new we_html_multiIconBoxContent())
								->add((new we_html_multiIconBoxContentLine('', we_html_tools::dialogBorder(730, $content, $headline) . $nextprev))->noline());
			}
			return (new we_html_multiIconBoxContent())
							->add((new we_html_multiIconBoxContentLine('', we_html_element::span(['class' => 'middlefont lowContrast'], g_l('prefs', '[log_is_empty]')) .
											we_html_element::br(2)))
									->noline()
			);
		}

		echo we_html_tools::getHtmlTop(g_l('prefs', '[formmail_log]'), we_html_element::jsScript(JS_DIR . 'formmaillog.js'), we_html_element::body(['class' => 'weDialogBody',
					'onload' => "self.focus();"],
						(new we_html_multiIconBox(getContents(), 'show_log_data'))
								->marginLeft()
								->buttons(we_html_button::formatButtons(we_html_button::create_button(we_html_button::REFRESH, "javascript:location.reload();") . we_html_button::create_button(we_html_button::CLOSE, "javascript:WE().util.dialogTop(window).doClose();") . we_html_button::create_button(we_html_button::LOG_CLEAR, "javascript:clearBlockLog()")))
								->headline(g_l('prefs', '[formmail_log]'))
								->get()
		));
	}

}
