<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 2338614f04fa $
 * $Author: Marc Kraemer <marc.kraemer@webedition.org> $
 * $Date: 2020-12-21 13:19 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
/* the parent class for documents */

abstract class we_document_files extends we_document_base{
	//Extension of the document
	/** @var string[] Array of possible filename extensions for the document */
	var $Extensions = [];

	/** @var string */
	var $Extension = '';

	/** @var string */
	var $Filehash = '';

	/** @var string database table in which the object is stored */
	public $Table = FILE_TABLE;

	/** initializes all data not static in class and not saved by persistent slots/elements.
	 * This is called from constructor and __wakeup on serialized documents
	 */
	protected function initStatic(): void{
		parent::initStatic();
		$this->persistent_slots = array_merge($this->persistent_slots, [
			'Extension' => we_base_request::STRING,
		]);

		$this->setExtensions();
	}

	public function getText(int $renumber = 0): string{
		return $this->Filename . ($renumber ? '_' . $renumber : '') . $this->Extension;
	}

	protected function formExtension(): string{
		//$doctype = $this->DocType ?? '';
		$doctype = $this instanceof we_document_textContent ? $this->DocType : '';

		if(!$this->ID && we_base_request::string('we_cmd', '', 0) === 'load_editor' && $doctype == ''){ //	Neues Dokument oder Dokument ohne DocType
			switch($this->ContentType){
				case we_base_ContentTypes::HTML: //	is HTML-File
					$selected = DEFAULT_HTML_EXT;
					break;
				case we_base_ContentTypes::WEDOCUMENT: //	webEdition Document
					$selected = ($this->IsDynamic == 1 ? DEFAULT_DYNAMIC_EXT : DEFAULT_STATIC_EXT);
					break;
				default: //	no webEdition Document
					$selected = $this->Extension;
					break;
			}
		} else { //	bestehendes Dokument oder Dokument mit DocType
			$selected = $this->Extension;
		}
		return we_html_element::div([], $this->Extensions ?
				we_html_tools::formElementTable(g_l('weClass', '[extension]'), we_html_tools::getExtensionPopup('we_' . $this->Name . '_Extension', $selected, $this->Extensions, 100, [
						'onselect' => "_EditorFrame.setEditorIsHot(true);"], we_base_permission::hasPerm([we_base_perm::WORK_DOCEXTENSION_EDIT]))) :
				we_html_element::hidden('we_' . $this->Name . '_Extension', $selected)
		);
	}

	public function we_new(): void{
		parent::we_new();
		$this->setExtensions();
		if(is_array($this->Extensions) && $this->Extensions){
			$this->Extension = $this->Extensions[0];
		}
		switch($this->Table){
			case FILE_TABLE:
			case TEMPLATES_TABLE:
				if(!isset($GLOBALS['WE_IS_DYN'])){
					if(!$this->ParentID && ($ws = we_users_util::get_ws($this->Table))){
						$this->setParentID(intval(reset($ws)));
					}
				}
		}
	}

	protected function we_load(int $from = self::LOAD_MAIN_DB): bool{
		$ret = parent::we_load($from);
		$this->setExtensions();
		return $ret;
	}

	private function setExtensions(): void{
		if(!isWE()){
			return;
		}
		if($this->ContentType){
			$exts = we_base_ContentTypes::getExtension($this->ContentType);
			$this->Extensions = $exts ?: [];
			$this->Extension = $this->Extension ?: (count($exts) === 1 ? $exts[0] : '');
		}
	}

	protected function isFileExtensionNotValid(): bool{
		return !(preg_match('/^\.?[a-zA-Z0-9]+$/iD', $this->Extension) || !$this->Extension);
	}

	protected function isFilenameDouble(): bool{
		return fB('SELECT 1 FROM ' . $this->DB_WE->escape($this->Table) . ' WHERE ParentID=' . intval($this->ParentID) . ' AND Filename="' . $this->DB_WE->escape($this->Filename) . '" AND Extension="' . $this->DB_WE->escape($this->Extension) . '" AND ID!=' . intval($this->ID) . ' LIMIT 1');
	}

	public function uploadImportFile(array $fileVars, array $docVars): void{
		// TODO: there are more bad combinations to consider
		//$this->setElement('type', $this->ContentType, "attrib");
		//$this->Published = time(); // nok for parked existing docs

		$fileVars['weFileSize'] = $fileVars['weFileSize'] < 1 ? 1 : $fileVars['weFileSize'];
		$this->setElementI('filesize', $fileVars['weFileSize'], 'attrib');

		if(we_base_permission::hasPerm([we_base_perm::WORK_CATEGORY_EDIT])){
			$this->Category = isset($docVars['categories']) && $docVars['categories'] ? $docVars['categories'] : $this->Category;
		}

		$tmpFile = WE_DOCUMENT_ROOT . $fileVars['fileTemp'];
		if($docVars['importMetadata'] && $this->Extension === '.pdf'){
			assert($this instanceof we_document_pdf);
			$this->setMetaDataFromFile($tmpFile);
		}
		$this->DocChanged = true;

		if($docVars['transaction']){
			//$_SESSION['weS']['we_data']['tmpName'] = $tmpFile; // what's this?
			$this->saveInSession($docVars['transaction']);
		} else {
			if(!$this->we_save()){
				throw new we_exception_file(g_l('importFiles', '[save_error]'));
			}
			if(!$this->we_publish()){
				throw new we_exception_file("publish_error");
			}
		}
	}

	public function uploadSetData(array $fileVars): bool{
		if(($fh = @fopen(WE_DOCUMENT_ROOT . $fileVars['fileTemp'], 'rb'))){
			//$mime = explode('/', $fileVars['weFileCt']);//, $mime[0]
			$this->setElementS('data', (fread($fh, ($fileVars['weFileSize'] < 1 ? 1 : $fileVars['weFileSize'])) ?: ''));
			fclose($fh);

			return true;
		}

		return false;
	}

	protected function update_filehash(): void{
		switch($this->Table){
			default:
				return;
			case TEMPLATES_TABLE:
			case FILE_TABLE:
		}
		$this->wasUpdate = $this->ID > 0;
		$usepath = ($this->Table === TEMPLATES_TABLE ?
			(!str_contains($this->Path, '.tmpl') ? TEMPLATES_PATH . $this->Path : TEMPLATES_PATH . substr_replace($this->Path, '.php', -5)) :
			WE_DOCUMENT_ROOT . $this->Path
			);

		$this->Filehash = (is_readable($usepath) ? md5_file($usepath) : null);
		$this->savePersistentSlotsToDB(['Filehash']);
	}

	protected function writeDocument(): bool{
		$update = $this->isMoved();
		try{
			$doc = $this->getDocumentToSave();
		} catch (we_exception_file $e){
			return false;
		}
		if(!$this->writeSiteDir($doc) || !$this->writeMainDir($doc)){
			return false;
		}

		if($update){
			$this->rewriteNavigation();
		}
		$this->update_filehash();
		return true;
	}

}
