<?php

declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: b54077f1393e $
 * $Author: Marc Kraemer <marc.kraemer@webedition.org> $
 * $Date: 2021-01-06 17:13 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_editor_imageTransform extends we_editor_base{

	public function show(): string{
		if(!($this->we_doc instanceof we_document_image)){
			exit('ERROR: Couldn\'t initialize we_document_image object');
		}

		switch(we_base_request::string('we_cmd', '', 0)){
			case 'image_resize':
				$json = $this->we_getImageResizeDialogJS();
				$dialog = $this->we_getImageResizeDialog();
				break;
			case 'image_convertJPEG':
				$json = self::we_getImageConvertDialogJS();
				$dialog = self::we_getImageConvertDialog();
				break;
			case 'image_rotate':
				$json = $this->we_getImageRotateDialogJS();
				$dialog = $this->we_getImageRotateDialog();
				break;
			default:
				$dialog = $json = '';
		}
		return $this->getPage($dialog, we_html_element::jsScript(JS_DIR . 'image_edit.js', '', ['id' => 'loadImage_edit', 'data-imgEdit' => setDynamicVar($json)]));
	}

	private function we_getImageResizeDialogJS(): array{
		assert($this->we_doc instanceof we_document_image);
		[$width, $height] = $this->we_doc->getOrigSize();

		return [
			'okCmd' => 'ResizeDialog',
			'width' => $width,
			'height' => $height,
			'ratio_wh' => (float) ($height ? ($width / $height) : 0),
			'ratio_hw' => (float) ($width ? ($height / $width) : 0),
		];
	}

	private static function we_getImageConvertDialogJS(): array{
		return [
			'okCmd' => 'ImageConvert'
		];
	}

	private function we_getImageRotateDialogJS(): array{
		assert($this->we_doc instanceof we_document_image);
		$imageSize = $this->we_doc->getOrigSize();
		return [
			'okCmd' => 'ImageRotate',
			'width' => $imageSize[0],
			'height' => $imageSize[1]
		];
	}

	private function we_getImageResizeDialog(): string{
		assert($this->we_doc instanceof we_document_image);
		[$width, $height] = $this->we_doc->getOrigSize();

		$buttons = we_html_button::position_yes_no_cancel(we_html_button::create_button(we_html_button::OK, "javascript:doOK();"), '', we_html_button::create_button(we_html_button::CANCEL, "javascript:WE().util.dialogTop(window).doClose();"));

		$widthInput = we_html_tools::textInput("width", (string) $width, 0, ['onkeyup' => "we_keep_ratio(this,this.form.widthSelect);"], 'number', '60px');
		$heightInput = we_html_tools::textInput("height", (string) $height, 0, ['onkeyup' => "we_keep_ratio(this,this.form.heightSelect);"], 'number', '60px');

		$widthSelect = '<select class="weSelect" name="widthSelect" onchange="we_switchPixelPercent(this.form.width,this);"><option value="pixel">' . g_l('weClass', '[unit][pixel]') . '</option><option value="percent">' . g_l('weClass', '[unit][percent]') . '</option></select>';
		$heightSelect = '<select class="weSelect" name="heightSelect" onchange="we_switchPixelPercent(this.form.height,this);"><option value="pixel">' . g_l('weClass', '[unit][pixel]') . '</option><option value="percent">' . g_l('weClass', '[unit][percent]') . '</option></select>';

		$ratio_checkbox = we_html_forms::checkbox('1', true, "ratio", g_l('thumbnails', '[ratio]'), false, "defaultfont", "if(this.checked){we_keep_ratio(this.form.width,this.form.widthSelect);}");

		$table = '<table class=" defaultfont">
	<tr><td>' . g_l('weClass', '[width]') . ':</td><td>' . $widthInput . '</td><td>' . $widthSelect . '</td></tr>
	<tr><td>' . g_l('weClass', '[height]') . ':</td><td>' . $heightInput . '</td><td>' . $heightSelect . '</td></tr>
	<tr><td colspan="3">' . $ratio_checkbox . '</td></tr>
</table>' .
				(($this->we_doc->getGDType() === "jpg") ?
				'<br/><div class="defaultfont">' . g_l('weClass', '[quality]') . '</div>' . we_base_imageEdit::qualitySelect("quality") :
				'');
		$content = (new we_html_multiIconBoxContent())->add(new we_html_multiIconBoxContentLine('', $table));
		return (new we_html_multiIconBox($content))->marginLeft()->buttons($buttons)->headline(g_l('weClass', '[resize]'))->get();
	}

	private static function we_getImageConvertDialog(): string{
		$buttons = we_html_button::position_yes_no_cancel(we_html_button::create_button(we_html_button::OK, "javascript:doOK();"), '', we_html_button::create_button(we_html_button::CANCEL, "javascript:WE().util.dialogTop(window).doClose();"));
		$dialog = '<div class="defaultfont">' . g_l('weClass', '[quality]') . '</div>' . we_base_imageEdit::qualitySelect("quality");
		$content = (new we_html_multiIconBoxContent())
				->add(new we_html_multiIconBoxContentLine('', $dialog));

		return (new we_html_multiIconBox($content))->marginLeft()->buttons($buttons)->headline(g_l('weClass', '[convert]'))->get();
	}

	private function we_getImageRotateDialog(): string{
		assert($this->we_doc instanceof we_document_image);
		$buttons = we_html_button::position_yes_no_cancel(we_html_button::create_button(we_html_button::OK, "javascript:doOK();"), '', we_html_button::create_button(we_html_button::CANCEL, "javascript:WE().util.dialogTop(window).doClose();"));

		$dialog = we_html_forms::radiobutton('180', true, 'degrees', g_l('weClass', '[rotate180]')) .
				we_html_forms::radiobutton('90', false, 'degrees', g_l('weClass', '[rotate90l]')) .
				we_html_forms::radiobutton('270', false, 'degrees', g_l('weClass', '[rotate90r]')) .
				(($this->we_doc->getGDType() === 'jpg') ?
				'<br/><div class="defaultfont">' . g_l('weClass', '[quality]') . '</div>' . we_base_imageEdit::qualitySelect('quality') :
				'');

		$content = (new we_html_multiIconBoxContent())
				->add(new we_html_multiIconBoxContentLine('', $dialog));


		return (new we_html_multiIconBox($content))->marginLeft()->buttons($buttons)->headline(g_l('weClass', '[rotate]'))->get();
	}

}
