<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 4bec8f6a4c70 $
 * $Author: Marc Kraemer <marc.kraemer@webedition.org> $
 * $Date: 2020-12-15 12:10 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_editor_objectTextarea extends we_editor_base{
	/** @var string */
	private $name = '';

	/**
	 * @var null|we_contents_object
	 */
	protected $we_doc = null;

	public function __construct(we_base_mainmodel $we_doc, we_base_jsCmd $jsCmd = null, string $we_transaction = ''){
		parent::__construct($we_doc, $jsCmd);
		$nr = we_base_request::string('we_cmd', '', 1);
		$this->name = we_base_request::string('we_cmd', '', 2);

		if(we_base_request::bool('ok')){
			$this->we_doc->setElementB($this->name . we_contents_objectBase::NAME_DIVIDER . 'wysiwyg', we_base_request::bool('wysiwyg'));
			$this->we_doc->setElementB($this->name . we_contents_objectBase::NAME_DIVIDER . 'inlineedit', we_base_request::bool('inlineedit'));
			$this->we_doc->setElementB($this->name . we_contents_objectBase::NAME_DIVIDER . 'removefirstparagraph', we_base_request::bool('removefirstparagraph'));
			$this->we_doc->setElementS($this->name . we_contents_objectBase::NAME_DIVIDER . 'commands', we_base_request::string('commands'));
			//$this->we_doc->setElementS($this->name . we_contents_objectBase::NAME_DIVIDER . 'contextmenu', we_base_request::string('contextmenu'));
			$this->we_doc->setElementS($this->name . we_contents_objectBase::NAME_DIVIDER . 'bgcolor', we_base_request::string('bgcolor', ''));
			$this->we_doc->setElementS($this->name . we_contents_objectBase::NAME_DIVIDER . 'class', we_base_request::string('class', ''));
			$this->we_doc->setElementS($this->name . we_contents_objectBase::NAME_DIVIDER . 'cssClasses', implode(',', we_base_request::stringList('cssClasses')));
			$this->we_doc->setElementS($this->name . we_contents_objectBase::NAME_DIVIDER . 'fontsizes', implode(',', we_base_request::stringList('fontsizes')));
			$this->we_doc->setElementS($this->name . we_contents_objectBase::NAME_DIVIDER . 'formats', implode(',', we_base_request::stringList('formats')));
			$this->we_doc->setElementS($this->name . we_contents_objectBase::NAME_DIVIDER . 'tinyparams', we_base_request::_(we_base_request::RAW_CHECKED, 'tinyparams', ''));
			$this->we_doc->setElementS($this->name . we_contents_objectBase::NAME_DIVIDER . 'templates', implode(',', we_base_request::intList('templates', [])));
			$this->we_doc->setElementS($this->name . we_contents_objectBase::NAME_DIVIDER . 'gallerytemplates', implode(',', we_base_request::intList('gallerytemplates', [
			])));
			$this->we_doc->saveInSession($we_transaction);
			//$this->jsCmd->addCmd('object_TextArea_apply');

			$this->jsCmd->addCmd('object_changeTextareaParams_at_class', $we_transaction, $nr); //we_object_class_changeEntry
			$this->jsCmd->addCmd('close_dialog');
		}
	}

	public function show(): string{
		$onOffVals = ['0' => 'false', '1' => 'true'];

		$parts = (new we_html_multiIconBoxContent())
			->add((new we_html_multiIconBoxContentLine('wysiwyg', we_html_tools::select('wysiwyg', $onOffVals, 1, (string) (int) $this->we_doc->getElementB($this->name . we_contents_objectBase::NAME_DIVIDER . 'wysiwyg'), false, [
						], true, 60) . we_html_element::hidden('wysiwyg_orig', $this->we_doc->getElementB($this->name . we_contents_objectBase::NAME_DIVIDER . 'wysiwyg'))))->space(we_html_multiIconBox::SPACE_MED2)
			)
			->add((new we_html_multiIconBoxContentLine('removefirstparagraph', we_html_tools::select('removefirstparagraph', $onOffVals, 1, (string) (int) $this->we_doc->getElementB($this->name . we_contents_objectBase::NAME_DIVIDER . 'removefirstparagraph'), false, [
						], true, 60)))->space(we_html_multiIconBox::SPACE_MED2))
			->add((new we_html_multiIconBoxContentLine('inlineedit', we_html_tools::select('inlineedit', $onOffVals, 1, (string) (int) $this->we_doc->getElementB($this->name . we_contents_objectBase::NAME_DIVIDER . 'inlineedit'), false, [
						], true, 60) . we_html_element::hidden('inlineedit_orig', $this->we_doc->getElementB($this->name . we_contents_objectBase::NAME_DIVIDER . 'inlineedit'))))
				->space(we_html_multiIconBox::SPACE_MED2)
			)
			->add((new we_html_multiIconBoxContentLine('class', we_html_tools::textInput('class', $this->we_doc->getElementS($this->name . we_contents_objectBase::NAME_DIVIDER . 'class'), 255, [
						], 'text', '400px')))
				->space(we_html_multiIconBox::SPACE_MED2)
			)
			->add((new we_html_multiIconBoxContentLine('bgcolor', we_html_tools::textInput('bgcolor', $this->we_doc->getElementS($this->name . we_contents_objectBase::NAME_DIVIDER . 'bgcolor'), 20, [
						], 'text', '60px')))
				->space(we_html_multiIconBox::SPACE_MED2)
			)
			->add((new we_html_multiIconBoxContentLine('commands', we_html_tools::select('tmp_commands', we_wysiwyg_editor::getEditorCommands(false), 1, '', false, [
						'onchange' => "var elem=document.getElementById('commands'); var txt = this.options[this.selectedIndex].text; if(elem.value.split(',').indexOf(txt)==-1){elem.value=(elem.value) ? (elem.value + ',' + txt) : txt;}this.selectedIndex=-1"]) . we_html_element::br() . we_contents_base::htmlTextArea('commands', 1, 30, htmlspecialchars($this->we_doc->getElementS($this->name . we_contents_objectBase::NAME_DIVIDER . 'commands')), [
						'id' => 'commands', 'style' => 'width:400px;height:4ex;'])))
				//->noline()
				->space(we_html_multiIconBox::SPACE_MED2)
			)
			/*
			  ->add((new we_html_multiIconBoxContentLine('contextmenu', we_html_tools::select('tmp_contextmenu', we_wysiwyg_editor::getEditorCommands(false), 1, '', false, [
			  'onchange' => "var elem=document.getElementById('contextmenu'); var txt = this.options[this.selectedIndex].text; if(elem.value.split(',').indexOf(txt)==-1){elem.value=(elem.value) ? (elem.value + ',' + txt) : txt;}this.selectedIndex=-1"]) . we_html_element::br() . we_contents_base::htmlTextArea('contextmenu', 1, 30, htmlspecialchars($this->we_doc->getElementS($this->name . we_contents_objectBase::NAME_DIVIDER . 'contextmenu')), [
			  'id' => 'contextmenu', 'style' => 'width:400px;height:4ex;'])))
			  ->space(we_html_multiIconBox::SPACE_MED2)
			  )
			 */
			->add((new we_html_multiIconBoxContentLine('fontsizes', we_html_tools::select('tmp_fontsizes', we_wysiwyg_editor::getAttributeOptions('fontsizes'), 1, '', false, [
						'onchange' => "var elem=document.we_form.fontsizes; var txt = this.options[this.selectedIndex].text; if(elem.value.split(',').indexOf(txt)==-1){elem.value=(elem.value) ? (elem.value + ',' + txt) : txt;}this.selectedIndex=-1"]) . we_html_element::br() . we_html_tools::textInput('fontsizes', $this->we_doc->getElementS($this->name . we_contents_objectBase::NAME_DIVIDER . 'fontsizes'), 1024, [
						], 'text', '400px')))
				->space(we_html_multiIconBox::SPACE_MED2)
			)
			->add((new we_html_multiIconBoxContentLine('formats', we_html_tools::select('tmp_formats', we_wysiwyg_editor::getAttributeOptions('formats'), 1, '', false, [
						'onchange' => "var elem=document.we_form.formats; var txt = this.options[this.selectedIndex].text; if(elem.value.split(',').indexOf(txt)==-1){elem.value=(elem.value) ? (elem.value + ',' + txt) : txt;}this.selectedIndex=-1"]) . we_html_element::br() . we_html_tools::textInput('formats', $this->we_doc->getElementS($this->name . we_contents_objectBase::NAME_DIVIDER . 'formats'), 1024, [
						], 'text', '400px')))
				->space(we_html_multiIconBox::SPACE_MED2)
			)
			->add((new we_html_multiIconBoxContentLine('classes', we_html_tools::textInput('cssClasses', htmlspecialchars($this->we_doc->getElementS($this->name . we_contents_objectBase::NAME_DIVIDER . 'cssClasses')), 1024, [
						], 'text', '400px')))
				->space(we_html_multiIconBox::SPACE_MED2)
			)
			->add((new we_html_multiIconBoxContentLine('tinyparams', we_html_tools::textInput('tinyparams', $this->we_doc->getElementS($this->name . we_contents_objectBase::NAME_DIVIDER . 'tinyparams'), 1024, [
						], 'text', '400px')))
				->space(we_html_multiIconBox::SPACE_MED2)
			)
			->add((new we_html_multiIconBoxContentLine('templates', we_html_tools::textInput('templates', $this->we_doc->getElementS($this->name . we_contents_objectBase::NAME_DIVIDER . 'templates'), 1024, [
						], 'text', '400px')))
				->space(we_html_multiIconBox::SPACE_MED2)
			)
			->add((new we_html_multiIconBoxContentLine('gallerytemplates', we_html_tools::textInput('gallerytemplates', $this->we_doc->getElementS($this->name . we_contents_objectBase::NAME_DIVIDER . 'gallerytemplates'), 1024, [
					], 'text', '400px')))
			->space(we_html_multiIconBox::SPACE_MED2)
		);

		$out = we_html_element::hidden('ok', 1);

		foreach($_REQUEST['we_cmd'] as $k => $v){
			$out .= we_html_element::hidden('we_cmd[' . $k . ']', $v);
		}

		return $this->getPage($out .
					(new we_html_multiIconBox($parts))
					->marginLeft()
					->buttons(we_html_button::position_yes_no_cancel(we_html_button::create_button(we_html_button::OK, "javascript:document.forms[0].submit();"), '', we_html_button::create_button(we_html_button::CANCEL, "javascript:WE().util.dialogTop(window).doClose()")))
					->headline(g_l('modules_object', '[field][textarea]') . ' "' . $this->we_doc->getElementS($this->name . we_contents_objectBase::NAME_DIVIDER . 'name') . '" - ' . g_l('modules_object', '[attributes]'))
					->get(), '', [
				'onload' => "top.focus();",
				'class' => 'weDialogBody'], [
				'action' => we_http_util::getScriptName()
		]);
	}

}
