<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: b0ef97cc1636 $
 * $Author: Lukas Imhof $
 * $Date: 2021-01-07 17:21 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_fileupload_ui_base extends we_fileupload{
	protected $responseClass = 'we_fileupload_resp_base';
	protected $doImport = false;
	protected $genericFilename = '';
	protected $type = 'base';
	protected $isMultiUpload = false;

	/** @var array<string,string> */
	protected $cmds = [
		'cmdFileselectOnclick' => '',
		'cmdUploadFinished' => '',
		'cmdCancel' => '',
		'cmdReset' => ''
	];
	protected $isPreset = false;
	protected $fileNameTemp = '';
	protected $transaction;
	protected $extension;
	//array is used in settings as well

	/** @var array<string,string> */
	public static $scaleProps = ['' => '', '320' => '320', '640' => '640', '960' => '960', '1280' => '1280', '1440' => '1440', '1600' => '1600', '1920' => '1920', '2560' => '2560'];
	protected $isMultiselect = false;

	/** @var array<string,int> */
	protected $controls = [
		'showControls' => self::CONTROLS_SHOW_TRUE,
		'showControlsInitialState' => self::CONTROLS_SHOW_TRUE,
		'showBtnsUploadCancel' => self::CONTROLS_SHOW_TRUE,
		'showEditOptions' => self::CONTROLS_SHOW_FALSE,
		'showImportMeta' => self::CONTROLS_SHOW_FALSE
	];

	/** @var array<string,int> */
	protected $itemControlls = [
		'showDelete' => self::CONTROLS_SHOW_FALSE
	];

	/** @var array<string,mixed> */
	protected $fileinfo = ['isempty' => true];
	protected $showFilenamesFull = false;

	const CONTROLS_SHOW_FALSE = 0;
	const CONTROLS_SHOW_TRUE = 1;
	const CONTROLS_SHOW_AUTO = 2;

	/**
	 *
	 * @param string $name
	 * @param string[] $acceptedCT
	 * @param string[] $acceptedExt
	 * @param string[] $forbiddenCT
	 * @param string[] $forbiddenExt
	 */
	public function __construct(string $name, array $acceptedCT = [], array $acceptedExt = [], array $forbiddenCT = [], array $forbiddenExt = []){
		parent::__construct($name);
		$this->setGenericFileName();
		$this->doCommitFile = false;
		$this->responseClass = 'we_fileupload_resp_base';

		for($i = 0; $i < count($acceptedCT); $i++){
			if(($excluded = we_base_ContentTypes::getExcluded($acceptedCT[$i]))){
				$forbiddenExt = array_merge($forbiddenExt, $excluded);
			}
			if(empty(we_base_ContentTypes::getExtension($acceptedCT[$i]))){
				unset($acceptedCT[$i]);
			}
		}
		for($i = 0; $i < count($forbiddenCT); $i++){
			if(($excluded = we_base_ContentTypes::getExcluded($forbiddenCT[$i]))){
				$acceptedExt = array_merge($acceptedExt, $excluded);
			}
			if(empty(we_base_ContentTypes::getExtension($forbiddenCT[$i]))){
				unset($forbiddenCT[$i]);
			}
		}
		$this->setTypeCondition('accepted', $acceptedCT, $acceptedExt);
		$this->setTypeCondition('forbidden', $forbiddenCT, $forbiddenExt);
		$this->setUploadPerms();
	}

	public function setShowFilenamesFull(bool $showFilenamesFull = false): void{
		$this->showFilenamesFull = $showFilenamesFull;
	}

	public function setCmd(string $name, string $cmd = ''): void{
		if(isset($this->cmds[$name])){
			$this->cmds[$name] = $cmd;
		}
	}

	public function setIsPreset(bool $isPreset = false): void{
		$this->isPreset = $isPreset;
	}

	public function setGenericFileName(string $genericFileName = ''): void{
		$this->genericFilename = $genericFileName ?: TEMP_DIR . self::REPLACE_BY_UNIQUEID;
	}

	public static function getExternalDropZone(string $name = 'we_File', string $content = '', string $style = '', bool $dragFromTree = true, bool $dragFromExt = true, string $jsCmdTree = '', string $jsCmdExt = '', array $contentTypes = [
		], string $table = ''): string{
		if(!self::isDragAndDrop()){
			return $content;
		}

		$table = $table ?: FILE_TABLE;
		$cts = $contentTypes ? ',' . implode(',', $contentTypes) . ',' : '';

		return we_html_element::cssLink(CSS_DIR . 'we_fileupload.css') . we_html_element::jsScript(JS_DIR . 'fileupload/we_fileupload_externalDropzone.js') . // insert this in every top
			we_html_element::div(['class' => 'external_dropzone', 'id' => 'div_externalDropbox_' . $name], we_html_element::div(['id' => 'div_' . $name . '_fileDrag',
					'class' => 'we_file_drag',
					'ondrop' => "handleDrop(event,'" . $name . "', " . ($dragFromTree ? 'true' : 'false') . ", " . ($dragFromExt ? 'true' : 'false') . ", '" . $jsCmdTree . "', '" . $jsCmdExt . "', '" . $cts . "', '" . $table . "');",
					'ondragover' => "handleDragOver(event, '" . $name . "');",
					'ondragleave' => "handleDragLeave(event, '" . $name . "');",
					'style' => (self::isDragAndDrop() ? '' : 'display:none;') . $style
					], $content));
	}

	public function getButtonWrapped(string $type, bool $disabled = false, string $prefix = '_btn'): string{ // FIXME: this stuff is obsolete too...
		switch($type){
			case 'browse':
				$fileInput = we_html_element::input(array_filter(['class' => 'fileInput fileInputHidden',
						'style' => 'width: 170px;',
						'type' => 'file',
						'name' => $this->name,
						'multiple' => $this->isMultiselect ? 'multiple' : '',
						'id' => $this->name,
						'accept' => trim($this->typeCondition['accepted']['all'], ','),
				]));
				$btn = we_html_button::create_button('fat:browse_harddisk,fa-lg fa-hdd-o', "javascript:void(0);", '', '', $disabled, false, $prefix, false, '', 'weBtn noMarginLeft');

				return we_html_element::div(['id' => 'div_' . $this->name . '_fileInputWrapper', 'class' => 'we_fileInputWrapper', 'style' => 'vertical-align:top;display:inline-block;'], $fileInput . $btn
				);
			case 'reset':
				$btn = we_html_button::create_button(we_html_button::RESET, "javascript:weFileUpload_instance.reset();", '', '', $disabled, false, $prefix, true, '', 'weBtn noMarginLeft');
				return we_html_element::div(['id' => 'div_fileupload_btnReset', 'style' => 'height:30px;'], $btn);

			case 'upload':
				$js = 'weFileUpload_instance.startUpload();';
				$btn = we_html_button::create_button(we_html_button::UPLOAD, "javascript:" . $js, '', '', $disabled, false, $prefix, true, '', 'weBtn noMarginLeft');
				return we_html_element::div(['id' => 'div_fileupload_btnUpload'], $btn);

			case 'cancel':
				$js = 'weFileUpload_instance.cancelUpload();';
				$btn = we_html_button::create_button(we_html_button::CANCEL, "javascript:" . $js, '', '', $disabled, false, $prefix, true, '', 'weBtn noMarginLeft');
				return we_html_element::div(['id' => 'div_fileupload_btnCancel'], $btn);
			default:
				return '';
		}
	}

	public function getDivBtnUploadCancel(): string{
		return we_html_element::div(['class' => 'fileupload_divBtnUploadCancel'], $this->getButtonWrapped('upload', true) . $this->getButtonWrapped('cancel', true));
	}

	protected function getDivBtnInputReset(): string{
		return we_html_element::div(['class' => 'fileupload_divBtnBrowseReset'], $this->getButtonWrapped('browse', false) . $this->getButtonWrapped('reset', true));
	}

	public function getHTML(): string{
		return we_html_element::div(['class' => 'fileuploadBase'], $this->getHiddens() . $this->getFormUploader($this->getDropZoneContent()));
	}

	protected function getHiddens(): string{
		return we_html_element::hiddens([
				'weFileNameTemp' => '',
				'weFileName' => '',
				'weFileCt' => '',
		]);
	}

	public function getHTMLExternalProgress(): string{
		return '';
	}

	public function getHtmlAlertBoxes(): string{
		$text = we_fileupload::getMaxUploadSizeMB() ? sprintf(g_l('import', '[upload][size_limit_set_to]'), we_fileupload::getMaxUploadSizeMB()) : g_l('import', '[upload][no_size_limit]');
		$box = we_html_tools::alertAttentionBox($text, we_html_tools::TYPE_INFO, 542);

		return we_html_element::div(['id' => 'div_alert'], $box);
	}

	protected function getDropZoneContent(): string{
		if(!$this->fileinfo || !empty($this->fileinfo['isempty'])){
			$contentType = empty($this->fileinfo['ct']) ? '' : $this->fileinfo['ct'];

			return we_html_element::div(['class' => 'dropzone_divPreview', 'style' => 'width:112px; height:112px;position:absolute;left: 52px;'], '<div class="empty" style="display:table-cell; vertical-align:middle; text-align:center;height:116px;width:116px;">' . '<span class="docIcon filedrag_set_icon" data-contenttype="' . $contentType . '" data-extension=""></span>') . '</div>' .
				we_html_element::div(['style' => 'position:absolute;left: 180px;font-size:2em; color:#ddd;margin-top:24px;'], g_l('import', '[upload][drop_text][ok]'));
		}

		$divFileInfo = we_html_element::div(['style' => 'color: black; text-align: left; display:block;display:table-cell;left:130px;position:absolute;'], we_html_element::div([
					'style' => 'display:table-cell;vertical-align:middle;height:116px;'], $this->fileinfo['size'] . '<br/>' . $this->fileinfo['typetext'] . '<br/>' . $this->fileinfo['metatext']));

		return we_html_element::div(['class' => 'dropzone_divPreview', 'style' => 'width:112px; height:112px;position:absolute;left: 12px;'], '<div class="filled" style="display:table-cell; vertical-align:middle; text-align:center;height:116px;width:116px;">' . ($this->fileinfo['thumbnail'] ?: '<span class="docIcon filedrag_set_icon" data-contenttype="' . ($this->fileinfo['ct'] ?? '') . '" data-extension="' . ($this->fileinfo['ext'] ?? '') . '"></span>')) . '</div>' . $divFileInfo .
			we_html_element::div(['style' => 'position:absolute;right: 20px;font-size:2em; color:#ddd;'], we_html_element::div(['style' => 'display:table-cell;vertical-align:middle;height:116px;'], g_l('import', '[upload][drop_text][ok]')));
	}

	protected function getFormUploader(string $content = '', bool $incFilesContainer = true): string{// TODO: add ICON to default-content; rename: getHTMLUploader
		$divButtons = we_html_element::div(['class' => 'container_BtnUploadCancel' . ($this->controls['showBtnsUploadCancel'] === self::CONTROLS_SHOW_FALSE ? ' hidden' : '')], $this->getDivBtnUploadCancel()
		);

		$divEditOptions = we_html_element::div(['id' => 'fileupload_editoptions', 'class' => ($this->controls['showEditOptions'] === self::CONTROLS_SHOW_TRUE ? '' : 'hidden')], self::getFormImageEditClientside($this->type, false));
		$divImportMeta = we_html_element::div(['id' => 'fileupload_importmeta', 'class' => 'containerImportMeta' . ($this->controls['showImportMeta'] === self::CONTROLS_SHOW_TRUE ? '' : ' hidden')], $this->getCheckImportMeta());

		$divControlls = we_html_element::div(['id' => 'fileupload_controlls', 'class' => 'container_controlls' . ($this->controls['showControlsInitialState'] === self::CONTROLS_SHOW_TRUE ? '' : ' hidden')], we_html_element::div([
					'id' => 'fileupload_controlls_left', 'class' => 'container_controllsLeft'], $divButtons) .
				we_html_element::div(['id' => 'fileupload_controlls_right', 'class' => 'container_controllsRight'], $divEditOptions . $divImportMeta)
		);

		return we_html_element::div(['class' => 'container_BtnBrowseReset'], $this->getDivBtnInputReset()) .
			we_html_element::div(['id' => 'div_' . $this->name . '_fileDrag', 'class' => $this->isDragAndDrop ? 'we_file_drag' : 'we_file_drag no_dragdrop'], ($incFilesContainer ? self::getHtmlLoup() : '') .
				we_html_element::div(['class' => 'emptyDropzone', 'id' => 'div_empty_dropzone'], $content ?: ($this->isDragAndDrop ? g_l('importFiles', '[dragdrop_text]') : g_l('importFiles', '[no_dragdrop_text]'))) .
				($incFilesContainer ? we_html_element::div(['id' => 'div_upload_files', 'class' => 'container_uploadItems', 'style' => 'display:none;'], '') : '')
			) . $divControlls;
	}

	public static function getFormImageEditClientside(string $type = '', bool $hide = false): string{
		$maxImgDim = FILE_UPLOAD_IMG_MAX_SIZE && !we_base_permission::hasPerm([we_base_perm::UPLOAD_DISABLE_DIM_LIMIT]) ? FILE_UPLOAD_IMG_MAX_SIZE : '';
		$attribs = ['name' => 'fuOpts_scale', 'type' => 'text', 'class' => 'wetextinput optsScaleInput', 'autocomplete' => 'off',
			'value' => $maxImgDim];
		$scaleValue = we_html_element::input($attribs);
		$scaleProps = !$maxImgDim ? self::$scaleProps : array_filter(self::$scaleProps, function($val) use($maxImgDim){
				return $val <= $maxImgDim;
			});
		$scalePropositions = we_html_tools::select('fuOpts_scaleProps', $scaleProps, 1, 0, false, [], false, 0, 'weSelect optsScalePropositions');
		$scaleWhatSelect = we_html_tools::select('fuOpts_scaleWhat', [
				'pixel_l' => g_l('importFiles', '[edit_pixel_longest]'),
				'pixel_w' => g_l('importFiles', '[edit_pixel_width]'),
				'pixel_h' => g_l('importFiles', '[edit_pixel_height]')
				], 1, 0, false, [], false, 155, 'weSelect optsUnitSelect');

		$quality = '<i class="fa fa-image qualityIconLeft"></i>' . we_html_element::input(['type' => 'range', 'value' => '90', 'min' => '10', 'max' => '90', 'step' => '5',
				'name' => 'fuOpts_quality', 'class' => 'optsQuality']) . '<i class="fa fa-image qualityIconRight"></i>';
		$btnProcess = we_html_button::create_button(we_html_button::MAKE_PREVIEW, "javascript:", '', '', true, false, '_weFileupload', false, g_l('importFiles', '[image][makePreview]'), 'btnImgEditRefresh_global');

		return we_html_element::div([], we_html_element::hidden('fuOpts_doEdit', 1)) .
			we_html_element::div(['class' => 'imgEditOpts' . ($hide ? ' hidden' : ''), 'id' => 'editImage'], we_html_element::div([
					'class' => 'scaleDiv'], we_html_element::div(['class' => 'labelContainer'], g_l('importFiles', '[scale_label]') . ':') .
					we_html_element::div(['class' => 'inputContainer'], $scaleWhatSelect . ' ' . $scalePropositions . $scaleValue) /* . $scaleHelp */
				) .
				we_html_element::div(['class' => 'qualityDiv'], we_html_element::div(['class' => 'labelContainer'], g_l('weClass', '[quality]') . ':') .
					we_html_element::div(['class' => 'inputContainer'], $quality) .
					we_html_element::div(['class' => 'btnContainer'], $btnProcess)
				)
		);
	}

	public function getCheckImportMeta(): string{
		return we_html_element::div(['id' => 'div_importMeta', 'class' => 'importMeta'], we_html_forms::checkboxWithHidden(($this->imageEditProps['importMetadata'] ? true : false), 'fu_doc_importMetadata_ctrl', g_l('importFiles', '[import_metadata]'), false, 'defaultfont', ''
				)
		);
	}

	public static function getHtmlLoup(): string{
		$divLoupe = we_html_element::div(['id' => 'we_fileUpload_loupe', 'class' => 'editorLoupe'], we_html_element::div(['id' => 'we_fileUpload_loupeInner', 'class' => 'editorLoupeInner']) .
				we_html_element::div(['id' => 'we_fileUpload_loupeInfo', 'class' => 'editorLoupeInfo']) .
				we_html_element::div(['id' => 'we_fileUpload_focusPoint', 'class' => 'editorFocusPoint'])
		);
		$divLoupeFallback = we_html_element::div(['id' => 'we_fileUpload_loupeFallback', 'class' => 'editorLoupeFallback']); //editorLoupeFallback
		$divLoupeCrosshairH = we_html_element::div(['class' => 'editorCrosshairH']);
		$divLoupeCrosshairV = we_html_element::div(['class' => 'editorCrosshairV']);
		$divFixedFocusPoint = we_html_element::div(['id' => 'editorFocuspointFixed', 'class' => 'editorFocusPoint focusPointOnSet']);
		$divLoupeSpinner = we_html_element::div(['id' => 'we_fileUpload_spinner', 'class' => 'editorLoupeSpinner'], we_html_element::span([], '<i class="fa fa-2x fa-spinner fa-pulse"></i>'));

		return we_html_element::div(['class' => 'containerEditorLoupe'], $divLoupe . $divLoupeFallback . $divLoupeCrosshairH . $divLoupeCrosshairV . $divFixedFocusPoint . $divLoupeSpinner);
	}

	protected function getHtmlUploadItem(): string{
		$btnTrash = we_html_button::create_button(we_html_button::TRASH, "javascript:weFileUpload_instance.removeItem(##INDEX##, this);");
		$btnEditFilename = we_html_button::create_button(we_html_button::EDIT, "javascript:void(0);", '', '', false, true, '', false, '', 'btnEditFilename');

		$progress = new we_gui_progressBar(0, 300, '_' . $this->name . '_##INDEX##');
		$uploadError = we_html_element::div(['class' => 'elemUploadError hidden'], we_html_element::span(['class' => 'fa-stack alertIcon'], '<i class="fa fa-exclamation-circle" ></i>') .
				we_html_element::div(['class' => 'textUploadError'], 'beat'));
		$progressbar = we_html_element::div(['class' => 'elemProgressLeft'], $progress->getHTML()) . we_html_element::div(['class' => 'elemProgressRight'], $uploadError);

		$scaleNotice = we_html_element::div(['data-index' => '##INDEX##', 'class' => 'elemScaleNotice'], we_html_element::span(['class' => 'fa-stack alertIcon'], '<i class="fa fa-question-circle" ></i>') .
				we_html_element::div(['class' => 'textScaleNotice']));
		$forceLongestNotice = we_html_element::div(['data-index' => '##INDEX##', 'class' => 'elemForceLongestNotice'], we_html_element::span(['class' => 'fa-stack alertIcon'], '<i class="fa fa-question-circle" ></i>') .
				we_html_element::div(['class' => 'textForceLongestNotice'], sprintf(g_l('importFiles', '[force_max_length]'), FILE_UPLOAD_IMG_MAX_SIZE)));
		$sizeError = we_html_element::div(['data-index' => '##INDEX##', 'class' => 'elemSizeError'], we_html_element::span(['class' => 'fa-stack alertIcon'], '<i class="fa fa-exclamation-circle" ></i>') .
				we_html_element::div(['class' => 'textSizeError'], sprintf(g_l('import', '[upload][size_limit_set_to]'), we_fileupload::getMaxUploadSizeMB())));

		$btnRotateLeft = we_html_button::create_button('fa:, fa-lg fa-rotate-left', "javascript:void(0);", 'fuOpts_addRotationLeft', '', false, false, '', false, g_l('importFiles', '[correct_orientation_left]'), 'fuOpts_addRotationLeft');
		$btnRotateRight = we_html_button::create_button('fa:, fa-lg fa-rotate-right', "javascript:void(0);", 'fuOpts_addRotationRight', '', false, false, '', false, g_l('importFiles', '[correct_orientation_right]'), 'fuOpts_addRotationRight');
		$hiddenFuOpts_rotate = we_html_element::hidden('fuOpts_rotate', 0, '', ['class' => 'fuOpts_rotate']);

		$divInfoTop = we_html_element::div(['class' => 'infoTop'], we_html_element::div(['class' => 'leftDiv' /* bold */], g_l('importFiles', '[edit_pixel_width]') . ' x ' . g_l('importFiles', '[edit_pixel_height]') . ':') . we_html_element::div([
					'class' => 'rowDimensionsProcessed rightDiv'], '--') . $scaleNotice . $forceLongestNotice);
		$divInfoMiddle = we_html_element::div(['class' => 'infoMiddle'], we_html_element::div(['class' => 'leftDiv' /* bold */], g_l('importFiles', '[rotation]') . ':') . we_html_element::div([
					'class' => 'rowRotation rightDiv'], ''));
		$divInfoBottom = we_html_element::div(['class' => 'infoBottom'], we_html_element::div(['class' => 'leftDiv'], g_l('importFiles', '[original_dimensions]') . ':') . we_html_element::div([
					'class' => 'rowDimensionsOriginal rightDiv'], '--'));
		$divInfo = we_html_element::div(['class' => 'elemInfo'], $divInfoTop . $divInfoMiddle . $divInfoBottom);

		$divControllsItem = we_html_element::div(['class' => 'controllsItem', 'style' => 'top:6px; left: 332px;'], we_html_element::div(['data-index' => '##INDEX##'], $hiddenFuOpts_rotate . $btnRotateLeft . $btnRotateRight) . we_html_element::div([
					'data-index' => '##INDEX##', 'style' => 'margin-top: 4px;'], we_html_button::create_button(we_html_button::MAKE_PREVIEW, "javascript:", '', '', false, false, '_##INDEX##', false, g_l('importFiles', '[image][makePreview]'), 'btnImgEditRefresh_item rowBtnProcess') . we_html_element::div([
						'class' => 'fa fa-asterisk rowEditHot'], '')));

		return str_replace(["\r", "\n"], '', we_html_element::div(['class' => 'uploadItem', 'id' => 'uploadItem_##INDEX##', 'data-index' => '##INDEX##'], we_html_element::div([
					'class' => 'containerLeft'], we_html_element::div(['class' => 'elemImagePreview'], we_html_element::div(['class' => 'imgPreview'])
					) .
					we_html_element::div(['class' => 'elemIcon'])
				) .
				we_html_element::div(['class' => 'containerRight'], we_html_element::div(['class' => 'containerRightTop'], we_html_element::div(['class' => 'elemFilename'], we_html_element::div([
								'class' => 'elemEditFilename hidden'], we_html_element::input(['class' => 'inputEditFilename' . ($this->type !== 'importer' ? ' nodelete' : ''),
									'data-task' => 'showFilename',
									'data-index' => '##INDEX##',
									'type' => 'text',
									'name' => 'fuOpts_filenameInput',
									'value' => '##FILENAME##',
								]) . ' .##EXTENSION##'
							) .
							we_html_element::div(['class' => 'elemShowFilename',
								'title' => '##FILETEXT##',
								'data-task' => 'editFilename',
								'data-index' => '##INDEX##',
								'data-extension' => '##EXTENSION##'
								], $btnEditFilename . '<span data-index="##INDEX##" data-task="editFilename" class="textShowFilename">##FILE##</span>'
							)
						) .
						we_html_element::div(['class' => 'elemSizeDelete'], we_html_element::div(['class' => 'elemSize'], '##FILESIZE##') .
							we_html_element::div(['class' => 'elemSizeNok hidden'], '&gt; ' . self::getMaxUploadSizeMB() . ' MB' . $sizeError) .
							we_html_element::div(['class' => 'elemDelete' . ($this->type !== 'importer' ? ' hidden' : '')], $btnTrash)
						)
					) .
					we_html_element::div(['class' => 'containerRightBottom'], we_html_element::div(['class' => 'elemImageEdit hidden'], $divInfo . $divControllsItem) .
						we_html_element::div(['class' => 'elemProgress hidden'], $progressbar) .
						we_html_element::div(['class' => 'elemProcess hidden'], '<i class="fa fa-spinner fa-pulse"></i> ' . g_l('importFiles', '[text_elem_processing]')) .
						we_html_element::div(['class' => 'elemTypeError hidden'], we_html_element::span(['class' => 'textTypeErrorText'], g_l('importFiles', '[error][file_type_forbidden]')) . ': ' . we_html_element::span([
								'class' => 'textTypeErrorType']) . we_html_element::br() .
							we_html_element::span(['class' => 'textTypeErrorExplain'])
						)
					) .
					we_html_element::div(['class' => 'elemFilenameFull hidden'], we_html_element::div(['class' => 'filenameLong'], '##FILENAMEFULL##'))
				) .
				we_html_element::div(['class' => 'containerItemBusy hidden'], we_html_element::div(['class' => 'elemSpinner'], '<i class="fa fa-2x fa-spinner fa-pulse"></i>') .
					we_html_element::div(['class' => 'elemText'])
				)
		));
	}

	public function getCss(): string{
		return we_html_element::cssLink(CSS_DIR . 'we_fileupload.css');
	}

	public function getJs(): string{
		return self::getJSFileuploader() . self::getJSUtils() . $this->getJSInit();
	}

	protected function getJSInit(): string{
		return we_html_element::jsScript(JS_DIR . 'fileupload/we_fileupload_init.js', '', ['id' => 'loadVarWeFileupload_init', 'data-initobject' => setDynamicVar($this->getJSDynamic())]) .
			we_html_element::jsScript(WE_LIB_DIR . 'pngChunksExtract/crc32.js') .
			(we_base_browserDetect::inst()->getBrowser() === we_base_browserDetect::IE ? we_html_element::jsScript(WE_LIB_DIR . 'lie/lie.polyfill.js') : '') .
			we_gui_progressBar::getJSCode();
	}

	public static function getJSFileuploader(): string{
		return
			we_html_element::jsScript(JS_DIR . 'fileupload/we_fileupload_uploader.js', '', ['defer' => 'defer']) .
			we_html_element::jsScript(JS_DIR . 'fileupload/class/we_fileupload_controller.js', '', ['defer' => 'defer']) .
			we_html_element::jsScript(JS_DIR . 'fileupload/class/we_fileupload_sender.js', '', ['defer' => 'defer']) .
			we_html_element::jsScript(JS_DIR . 'fileupload/class/we_fileupload_view.js', '', ['defer' => 'defer']) .
			we_html_element::jsScript(JS_DIR . 'fileupload/class/we_fileupload_imageEdit.js', '', ['defer' => 'defer']) .
			we_html_element::jsScript(JS_DIR . 'fileupload/class/we_fileupload_utils.js', '', ['defer' => 'defer']);
	}

	public static function getJSUtils(bool $toWE = false): string{
		return
			we_html_element::jsScript(WE_LIB_DIR . 'ExifReader/ExifReader.js', ($toWE ? 'WE().layout.fileupload.ExifReader = window.ExifReader;' : ''), ['defer' => 'defer']) .
			we_html_element::jsScript(WE_LIB_DIR . 'pngChunksEncode/index.js', ($toWE ? 'WE().layout.fileupload.encodehunks = window.encodeChunks;' : ''), ['defer' => 'defer']) .
			we_html_element::jsScript(WE_LIB_DIR . 'pngChunksExtract/index.js', ($toWE ? 'WE().layout.fileupload.extractChunks = window.extractChunks' : ''), ['defer' => 'defer']) .
			we_html_element::jsScript(WE_LIB_DIR . 'nodeca/pica.js', ($toWE ? 'WE().layout.fileupload.pica = window.pica' : ''), ['defer' => 'defer']);
	}

	protected function getExtToMime(){
		$ret = ['.gif' => 'image/gif',
			'.jpg' => 'image/jpeg',
			'.jpeg' => 'image/jpeg',
			'.png' => 'image/png',
			'.svg' => 'image/*',
			'.svgz' => 'image/*',
			'.jfif' => 'image/*',
			'.webp' => 'image/*',
		];

		foreach([we_base_ContentTypes::AUDIO,
				we_base_ContentTypes::CSS,
				we_base_ContentTypes::HTACCESS,
				we_base_ContentTypes::HTML,
				we_base_ContentTypes::JS,
				we_base_ContentTypes::PDF,
				we_base_ContentTypes::TEXT,
				we_base_ContentTypes::VIDEO,
				we_base_ContentTypes::XML] as $type){

			$ret = array_merge($ret, array_combine(($extensions = we_base_ContentTypes::getExtension($type)),
				array_fill(0, count($extensions), $type)));
		}

		return $ret;
	}

	public function getJSDynamic(): array{
		return [
			'uiType' => $this->type,
			'isMultiUpload' => ($this->isMultiUpload ? 1 : 0),
			'fieldName' => $this->name,
			'genericFilename' => $this->genericFilename,
			'doCommitFile' => $this->doCommitFile,
			'rpcUrl' => $this->rpcUrl,
			'maxUploadSize' => $this->maxUploadSizeBytes,
			'typeCondition' => str_replace(['\n\r', '\r\n', '\r', '\n'], '', $this->typeCondition),
			'isDragAndDrop' => $this->isDragAndDrop,
			'isPreset' => $this->isPreset,
			'cmdUploadFinished' => $this->cmds['cmdUploadFinished'],
			'cmdFileselectOnclick' => $this->cmds['cmdFileselectOnclick'],
			'chunkSize' => self::CHUNK_SIZE,
			'htmlUploadItem' => $this->getHtmlUploadItem(),
			'responseClass' => $this->responseClass,
			'imageeditMaxLongest' => (FILE_UPLOAD_IMG_MAX_SIZE && !we_base_permission::hasPerm([we_base_perm::UPLOAD_DISABLE_DIM_LIMIT]) ? FILE_UPLOAD_IMG_MAX_SIZE : -1),
			'controls' => $this->controls,
			'showFilenamesFull' => ($this->showFilenamesFull ? 1 : 0),
			'isIE' => we_base_browserDetect::inst()->getBrowser() === we_base_browserDetect::IE ? 1 : 0,
			'isBinaryReplacement' => (empty($this->fileinfo['isempty']) ? 1 : 0),
			'htmlExternalProgress' => $this->getHTMLExternalProgress(),
			'extToMime' => $this->getExtToMime(),
		];
	}

	public static function getJSLangConsts(): string{
		return we_base_util::setLangString([
				'btnCancel' => g_l('button', '[cancel][value]'),
				'btnClose' => g_l('button', '[close][value]'),
				'btnProcess' => g_l('importFiles', '[btnProcess]'),
				'btnUpload' => g_l('button', '[upload][value]'),
				'cancelled' => g_l('importFiles', '[cancelled]'),
				'cancelledShort' => g_l('importFiles', '[cancelledShort]'),
				'alertUploadPending' => g_l('importFiles', '[alertUploadPending]'),
				'doImport' => g_l('importFiles', '[do_import]'),
				'dropText' => g_l('importFiles', '[dragdrop_text]'),
				'editNotEdited' => g_l('importFiles', '[not_edited]'),
				'editQuality' => g_l('weClass', '[quality]'),
				'editRotation' => g_l('importFiles', '[rotation]'),
				'editRotationLeft' => g_l('global', '[left]'),
				'editRotationRight' => g_l('global', '[right]'),
				'editScaled' => g_l('importFiles', '[scaled_to]'),
				'editTargetsizeTooLarge' => g_l('importFiles', '[targettsize_too_large]'),
				'errorFileSize' => g_l('importFiles', '[error][file_size]'),
				'errorFileSizeType' => g_l('importFiles', '[error][size_type]'),
				'errorFileType' => g_l('importFiles', '[error][file_type]'),
				'errorNoFileSelected' => g_l('importFiles', '[error][no_file]'),
				'errorSelectedNotValid' => g_l('importFiles', '[error][selected_not_valid]'),
				'errorTooBigEditFailed' => g_l('importFiles', '[error][size_and_editFailed]'),
				'errorUploadNotFinished' => g_l('importFiles', '[error][upload_not_finished]'),
				'errorEditFailed' => g_l('importFiles', '[error][editFailed]'),
				'file' => g_l('importFiles', '[file]'),
				'maskImporterProcessImages' => sprintf(g_l('importFiles', '[maskImporterProcessImages]'), '<span id="we_fileUploadImporter_messageNr"></span>'),
				'maskImporterReadImages' => sprintf(g_l('importFiles', '[maskImporterReadImages]'), '<span id="we_fileUploadImporter_messageNr"></span>'),
				'maskProcessImage' => g_l('importFiles', '[maskProcessImage]'),
				'maskReadImage' => g_l('importFiles', '[maskReadImage]'),
				'sizeTextNok' => g_l('weEditorInfo', '[file_size]') . ': &gt; ' . we_fileupload::getMaxUploadSizeMB() . ' MB',
				'sizeTextOk' => g_l('weEditorInfo', '[file_size]') . ': ',
				'typeNokExplainNoPerms' => g_l('importFiles', '[typeError_noPerms]'),
				'typeNokExplainNotAlloud' => g_l('importFiles', '[typeError_notAlloud]'),
				'typeTextOk' => g_l('import', '[filetype]') . ': ',
				'uploadCancelled' => g_l('importFiles', '[upload_cancelled]'),
				'editMaxLongest' => g_l('importFiles', '[imgedit_maxLongest]'),
				'editScaleMandatory' => g_l('importFiles', '[imgedit_mandatoryScale]'),
				'uploadedBeforeCancelling' => g_l('importFiles', '[uploaded_before_cancelling]'),
				'autocorrectLandscape' => g_l('importFiles', '[autocorrect_landscape]'),
				'successfullImports' => g_l('importFiles', '[import_successes]'),
				'failedImports' => g_l('importFiles', '[import_failures]'),
		]);
	}

}
