<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 3633f50bccd9 $
 * $Author: Lukas Imhof $
 * $Date: 2020-05-04 13:56 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_fileupload_ui_wedoc extends we_fileupload_ui_base{
	protected $documentCT = we_base_ContentTypes::APPLICATION;

	/**
	 *
	 * @param string $name
	 * @param string[] $acceptedCT
	 * @param string[] $acceptedExt
	 * @param string[] $forbiddenCT
	 * @param string[] $forbiddenExt
	 * @param array<string,mixed> $fileinfo
	 */
	public function __construct(string $name, array $acceptedCT = [], array $acceptedExt = [], array $forbiddenCT = [], array $forbiddenExt = [], array $fileinfo = [
		]){
		$this->documentCT = empty($acceptedCT[0]) ? $this->documentCT : $acceptedCT[0];
		$this->noPermsCT = $this->getForbiddenCtsFromPerms();

		parent::__construct('we_File', $acceptedCT, $acceptedExt, $forbiddenCT, $forbiddenExt);

		$this->type = 'wedoc';
		$this->fileinfo = $fileinfo;
		$this->doCommitFile = true;
		$this->doImport = true;
		$this->responseClass = 'we_fileupload_resp_import';

		$this->controls = [
			'showControls' => !empty($this->fileinfo['isempty']) ? self::CONTROLS_SHOW_TRUE : self::CONTROLS_SHOW_AUTO,
			'showControlsInitialState' => !empty($this->fileinfo['isempty']) ? self::CONTROLS_SHOW_TRUE : self::CONTROLS_SHOW_FALSE,
			'showBtnsUploadCancel' => self::CONTROLS_SHOW_TRUE,
			'showEditOptions' => $this->documentCT === we_base_ContentTypes::IMAGE ? self::CONTROLS_SHOW_TRUE : self::CONTROLS_SHOW_FALSE,
			// showImportMeta temporarily deactivated for PDFs: import of pfd-metas to custom metadata fields not yet implemented
			'showImportMeta' => in_array($this->documentCT, [we_base_ContentTypes::IMAGE/* , we_base_ContentTypes::PDF */], true) ? self::CONTROLS_SHOW_TRUE : self::CONTROLS_SHOW_FALSE
		];
	}

	public function getHTML(bool $withHeader = true): string{
		return ($withHeader ? $this->getJs() . $this->getCss() : '') . $this->getHiddens() . we_html_element::div(['class' => 'fileuploadWedoc'], $this->getFormUploader($this->getDropZoneContent()));
	}

	public function getJs(): string{
		// if IE we load uploader utils on editor
		return (we_base_browserDetect::inst()->getBrowser() === we_base_browserDetect::IE ? self::getJSUtils() : '') . $this->getJSFileuploader() . $this->getJSInit();
	}

}
