<?php

declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: d256bb2814a9 $
 * $Author: Marc Kraemer <marc.kraemer@webedition.org> $
 * $Date: 2021-01-29 12:12 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_fragment_copyFolder extends we_fragment_base{

	/** @var string */
	private $copyToPath = '';

	/** @var string */
	private $text = '';

	protected function init(): void{
		$fromID = we_base_request::int('we_cmd', 0, 1);
		$toID = we_base_request::int('we_cmd', 0, 2);
		$table = we_base_request::_(we_base_request::TABLE, 'we_cmd', '', 4);
		$OverwriteObjects = we_base_request::string('OverwriteObjects', we_exim_ExIm::COLLISION_SKIP);
		$ObjectCopyNoFolders = we_base_request::bool('DoNotCopyFolders');
		$CreateTemplate = we_base_permission::hasPerm([we_base_perm::WORK_TEMPLATE_NEW]) && we_base_request::bool('CreateTemplate');
		$CreateDoctypes = we_base_permission::hasPerm([we_base_perm::WORK_DOCTYPE_EDIT]) && we_base_request::bool('CreateDoctypes');
		$CreateMasterTemplate = we_base_request::bool('CreateMasterTemplate');
		$InheritLangFromToID = we_base_request::_(we_base_request::BOOL, 'InheritLangFromToID');
		$KeepLanglinks = $InheritLangFromToID ? we_base_request::_(we_base_request::BOOL, 'KeepLanglinks') : false;

		$CreateTemplateInFolderID = we_base_request::int('CreateTemplateInFolderID', 0);
		$OverwriteCategories = we_base_request::bool('OverwriteCategories', false);
		$vals = [];
		foreach($_REQUEST as $name => $val){
			if(!is_array($val)){
				if(preg_match('%^me(.*)variant0_me(.*)_item%i', $name)){
					$vals[] = $val;
				}
			}
		}
		$newCategories = implode(',', path_to_idA($vals, CATEGORY_TABLE));

		if(isset($_SESSION['weS']['WE_CREATE_DOCTYPE'])){
			unset($_SESSION['weS']['WE_CREATE_DOCTYPE']);
		}
		if(isset($_SESSION['weS']['WE_CREATE_TEMPLATE'])){
			unset($_SESSION['weS']['WE_CREATE_TEMPLATE']);
		}
		switch($table){
			default:
				if($fromID && $toID){
					//  "fromID"  cannot be a parent  of "toID"

					$fromPath = id2path($fromID);
					$db = new DB_WE();
					$this->alldata = [];
					$merge = [
						'CopyFromId' => $fromID,
						'CopyFromPath' => $fromPath,
						'CopyToId' => $toID,
						'CreateDoctypes' => $CreateDoctypes,
						'CreateIncludedTemplate' => $CreateMasterTemplate,
						'CreateMasterTemplate' => $CreateMasterTemplate,
						'CreateTemplate' => $CreateTemplate,
						'CreateTemplateInFolderID' => $CreateTemplateInFolderID,
						'IsWeFile' => false,
						'Language' => $InheritLangFromToID ? $this->getLanguageById($toID) : '',
						'KeepLanglinks' => $KeepLanglinks,
						'OverwriteCategories' => $OverwriteCategories,
						'newCategories' => $newCategories,
						'TheTable' => FILE_TABLE,
					];

					// make it twice to be sure that all linked IDs are correct
					$data = $db->getAllq('SELECT ID,ParentID,Text,Path,IsFolder,ContentType,Category FROM ' . FILE_TABLE . " WHERE (Path LIKE'" . $db->escape($fromPath) . "/%') AND ContentType != '" . we_base_ContentTypes::WEDOCUMENT . "' ORDER BY IsFolder DESC,Path");
					foreach($data as $cur){
						'@phan-var array $cur';
						$this->alldata[] = array_merge($cur, $merge);
					}

					$merge['IsWeFile'] = 1;
					for($num = 0; $num < 2; $num++){
						$merge['num'] = $num;
						$data = $db->getAllq('SELECT
	f.ID,f.ParentID,f.Text,f.TemplateID,f.Path,f.IsFolder,f.ContentType,f.Category,t.ID IS NOT NULL AS hasTempl
FROM
	' . FILE_TABLE . ' f LEFT JOIN
	' . TEMPLATES_TABLE . " t ON (f.TemplateID=t.ID)
WHERE
	(f.Path LIKE'" . $db->escape($fromPath) . "/%') AND f.ContentType='" . we_base_ContentTypes::WEDOCUMENT . "'
ORDER BY
	f.IsFolder DESC,f.Path");
						foreach($data as $cur){
							'@phan-var array $cur';
							$merge['CreateTemplate'] = $CreateTemplate && $cur['hasTempl'] ? 1 : 0;
							$this->alldata[] = array_merge($cur, $merge);
						}
					}
				}
				break;

			case OBJECT_FILES_TABLE:
				$_SESSION['weS']['WE_COPY_OBJECTS'] = true;
				$fromPath = id2path($fromID, OBJECT_FILES_TABLE);

				$qfolders = ($ObjectCopyNoFolders ? ' ParentID=' . $fromID . ' AND IsFolder=0 AND ' : '');

				$db = DB_WE::$inst;
				$this->alldata = [];
				$merge = [
					'CreateIncludedTemplate' => false,
					'CreateMasterTemplate' => false,
					'CopyFromId' => $fromID,
					'CopyFromPath' => $fromPath,
					'CopyToId' => $toID,
					'TargetLanguage' => $InheritLangFromToID ? $this->getLanguageById($toID, OBJECT_FILES_TABLE) : '',
					'CreateDoctypes' => false,
					'CreateTemplate' => false,
					'CreateTemplateInFolderID' => false,
					'IsWeFile' => false,
					'ObjectCopyNoFolders' => $ObjectCopyNoFolders,
					'OverwriteCategories' => false,
					'OverwriteObjects' => $OverwriteObjects,
					'TheTable' => OBJECT_FILES_TABLE,
					'newCategories' => '',
				];
				$data = $db->getAllq('SELECT
	ID,ParentID,Text,Path,IsFolder,ContentType,Publish_Date
FROM
	' . OBJECT_FILES_TABLE . ' `of`
WHERE
	' . $qfolders . " (Path LIKE'" . $db->escape($fromPath) . "/%')
ORDER BY
	IsFolder DESC,Path");
				foreach($data as $cur){
					'@phan-var array $cur';
					$this->alldata[] = array_merge($cur, $merge);
				}
		}
	}

	protected function doTask(): void{
		if(!is_array($this->data)){
			return;
		}
		we_base_WE::$rebuildActive = true;
		switch($this->data['TheTable']){
			case FILE_TABLE:
				if(!$this->copyFile()){
					we_base_WE::$rebuildActive = false;
					t_e('Error importing File: ' . $this->data['Path']);
					exit('Error importing File: ' . $this->data['Path']);
				}
				$this->text = addslashes(($this->data['IsWeFile'] && $this->data['num'] ?
						sprintf(g_l('copyFolder', '[rewrite]'), basename($this->data['Path'])) :
						sprintf(g_l('copyFolder', $this->data['IsFolder'] ? '[copyFolder]' : '[copyFile]'), basename($this->data['Path'])))
				);
				we_base_WE::$rebuildActive = false;
				return;

			case OBJECT_FILES_TABLE:
				if(!$this->copyObjects()){
					we_base_WE::$rebuildActive = false;
					t_e('Error importing Object: ' . $this->data['Path']);
					exit('Error importing Object: ' . $this->data['Path']);
				}
				$this->text = addslashes(sprintf(g_l('copyFolder', $this->data['IsFolder'] ? '[copyFolder]' : '[copyFile]'), basename($this->data['Path'])));
				we_base_WE::$rebuildActive = false;
		}
	}

	protected function updateProgressBar(): void{
		$this->jsCmd->addCmd('setProgress', [
			'progress' => (int) ($this->alldata ? ((100 / count($this->alldata)) * (1 + $this->currentTask)) : 100),
			'name' => 'pbar1',
			'text' => $this->text
		]);
	}

	private function getLanguageById($id = 0, $table = FILE_TABLE): string{
		$db = new DB_WE();
		return fS('SELECT Language FROM ' . $db->escape($table) . ' WHERE ID=' . intval($id));
	}

	private function getObjectPid(string $path): int{
		$path = dirname($path);
		if($path === '/'){
			return 0;
		}
		return fI('SELECT ID FROM ' . OBJECT_FILES_TABLE . ' WHERE Path="' . DB_WE::$inst->escape($path) . '"');
	}

	private function copyObjects(): bool{
		try{
			$obj = $this->getObjectFile();
		}catch(Throwable $e){
			return false;
		}
		$this->copyToPath = id2path($this->data['CopyToId'], OBJECT_FILES_TABLE);
		$path = preg_replace('|^' . $this->data['CopyFromPath'] . '/|', $this->copyToPath . '/', $this->data['Path']);
		if($this->data['IsFolder']){
			assert($obj instanceof we_folder_objectFile);
			if(!$obj->initByPath($path, OBJECT_FILES_TABLE) || !$obj->we_save()){
				return false;
			}
			return true;
		}
		assert($obj instanceof we_contents_objectFile);
		$obj->copyDoc($this->data['ID']);
		$obj->Filename = $this->data['Filename'] ?? $this->data['Text'];
		$obj->OldPath = '';
		$pid = $this->getObjectPid($path);
		$obj->setParentID($pid);
		$ObjectExists = $this->CheckForSameObjectName($pid, $obj->Filename);
		$obj = $this->data['Language'] ?: $obj;
		$obj->LangLinks = $this->data['KeepLanglinks'] ? $obj->LangLinks : [];
		$obj->setIgnoreLangLinkErr(true);

		if($ObjectExists){
			switch($this->data['OverwriteObjects']){
				case we_exim_ExIm::COLLISION_SKIP:
					return true;

				case we_exim_ExIm::COLLISION_RENAME:
					$obj->Filename .= '_copy';
					$obj->Path .= '_copy';
					while($this->CheckForSameObjectName($pid, $obj->Filename)){
						$obj->Filename .= '_copy';
						$obj->Path .= '_copy';
					}
					break;
				case we_exim_ExIm::COLLISION_OVERWRITE:
					$obj->ID = $ObjectExists;
					break;
			}
		}
		if(!$obj->we_save()){
			return false;
		}
		return ($this->data['Publish_Date'] ?
				$obj->we_publish() :
				true);
	}

	private function CheckForSameObjectName(int $pid, string $name): int{
		return fI('SELECT ID FROM ' . OBJECT_FILES_TABLE . ' WHERE ParentID=' . $pid . ' AND Filename="' . DB_WE::$inst->escape($name) . '"');
	}

	private function getObjectFile(): we_base_mainmodel{
		switch($this->data['ContentType']){
			case we_base_ContentTypes::FOLDER_OBJECTFILE:
				return new we_folder_objectFile();
			case we_base_ContentTypes::OBJECT_FILE:
				return new we_contents_objectFile();
			default:
				throw new InvalidArgumentException();
		}
	}

	private function copyFile(): bool{
		$this->copyToPath = id2path($this->data['CopyToId']);
		$path = preg_replace('|^' . $this->data['CopyFromPath'] . '/|', $this->copyToPath . '/', $this->data['Path']);
		$doc = we_base_ContentTypes::getObject($this->data['ContentType']);
		if($this->data['IsFolder']){
			assert($doc instanceof we_folder_document);
			$doc->initByPath($path);
			return $doc->we_save();
		}
		assert($doc instanceof we_contents_root);
		$doc->initByID($this->data['ID']);
		// if file  exists the file will overwritten, if not a new one (with no id) will be created
		$doc->ID = fI('SELECT ID FROM ' . FILE_TABLE . ' WHERE Path="' . DB_WE::$inst->escape($path) . '"');
		$pid = $this->getPid($path);
		$doc->setParentID($pid);
		$doc->Path = $doc->getPath();
		$doc->OldPath = '';
		$doc->Language = $this->data['Language'] ?: $doc->Language;
		$doc->LangLinks = $this->data['KeepLanglinks'] ? $doc->LangLinks : [];
		$doc->setIgnoreLangLinkErr(true);

		switch($doc->ContentType){
			case we_base_ContentTypes::WEDOCUMENT:
				assert($doc instanceof we_document_webEdition);
				$oldTemplateID = $doc->TemplateID;
				$this->parseWeDocument($doc);

				// check if we need to create a template
				if($this->data['CreateTemplate']){
					// check if a template was created from prior doc
					if(!(isset($_SESSION['weS']['WE_CREATE_TEMPLATE']) && isset($_SESSION['weS']['WE_CREATE_TEMPLATE'][$doc->TemplateID]))){
						$this->copyTemplate($doc->TemplateID, $this->data['CreateTemplateInFolderID'], $this->data['CreateMasterTemplate'], $this->data['CreateIncludedTemplate']);
					}

					$doc->setTemplateID((int) $_SESSION['weS']['WE_CREATE_TEMPLATE'][$doc->TemplateID]);
				}

				$doc->Category = ($this->data['OverwriteCategories'] ?
						we_base_request::filterIntList($this->data['newCategories']) :
						// remove duplicates
						array_filter(array_unique(array_merge($doc->Category, explode(',', $this->data['newCategories']))))
						);

				if($doc->DocType && $this->data['CreateDoctypes']){
					$this->createDocType($doc, $oldTemplateID);
				}

				// bugfix 0001582
				$doc->OldPath = $doc->Path;
				break;
			case we_base_ContentTypes::HTML:
			case we_base_ContentTypes::TEXT:
			case we_base_ContentTypes::CSS:
			case we_base_ContentTypes::HTACCESS:
			case we_base_ContentTypes::JS:
				assert($doc instanceof we_document_text);
				$this->parseTextDocument($doc);
				break;
		}

		if(!$doc->we_save()){
			return false;
		}

		return ($doc->Publish_Date ?
				$doc->we_publish() : true);
	}

	private function createDocType(we_document_webEdition $doc, int $oldTemplateID): void{
		// check if a doctype was created from prior doc
		if(!(isset($_SESSION['weS']['WE_CREATE_DOCTYPE']) &&
				isset($_SESSION['weS']['WE_CREATE_DOCTYPE'][$doc->DocType]))){

			$dt = new we_doctype_docTypes();

			$dt->load($doc->DocType);
			$dt->ID = 0;
			$dt->Text .= '_copy';
			// if file exists we need  to create a new one!
			if(fI('SELECT ID FROM ' . DOC_TYPES_TABLE . ' WHERE Text="' . DB_WE::$inst->escape($dt->Text) . '"')){
				$z = 0;
				$footext = $dt->Text . '_' . $z;
				while(fI('SELECT ID FROM ' . DOC_TYPES_TABLE . ' WHERE Text="' . DB_WE::$inst->escape($footext) . '"')){
					$z++;
					$footext = $dt->Text . '_' . $z;
				}
				$dt->Text = $footext;
			}
			$path = id2path($dt->ParentID);
			if($this->mustChange($path)){
				$parentPath = $this->getNewPath($path);
				$dt->ParentID = path_to_id($parentPath, FILE_TABLE);
			}

			if($dt->Templates){
				$templArray = array_filter($dt->Templates);
				$newTemplateIDs = [];
				foreach($templArray as $id){
					if($id == $oldTemplateID){
						$newTemplateIDs[] = $doc->TemplateID;
					} else {
						$newTemplateIDs[] = $id;
					}
				}
				$dt->Templates = $newTemplateIDs;
				$dt->TemplateID = $doc->TemplateID;
			}

			$dt->save();
			$newID = $dt->ID;

			if(!isset($_SESSION['weS']['WE_CREATE_DOCTYPE'])){
				$_SESSION['weS']['WE_CREATE_DOCTYPE'] = [
					$doc->DocType => $newID
				];
			} else {
				$_SESSION['weS']['WE_CREATE_DOCTYPE'][$doc->DocType] = $newID;
			}
		}

		$doc->DocType = $_SESSION['weS']['WE_CREATE_DOCTYPE'][$doc->DocType];
	}

	private function copyTemplate(int $templateID, int $parentID, bool $CreateMasterTemplate = false, bool $CreateIncludedTemplate = false, int $counter = 0): array{
		$counter++;
		$templVars = [];
		if(!isset($_SESSION['weS']['WE_CREATE_TEMPLATE'])){
			$_SESSION['weS']['WE_CREATE_TEMPLATE'] = [];
		}
		if(!isset($_SESSION['weS']['WE_CREATE_TEMPLATE'][$templateID])){

			$templ = new we_contents_template($templateID);
			$templ->ID = 0;
			$templ->OldPath = '';
			$templ->setParentID($parentID);
			// if file exists we need  to create a new one!
			if(fI('SELECT ID FROM ' . TEMPLATES_TABLE . ' WHERE Path="' . DB_WE::$inst->escape($templ->Path) . '"')){
				$z = 0;
				$footext = $templ->Filename . '_' . $z . $templ->Extension;
				while(fI('SELECT ID FROM ' . TEMPLATES_TABLE . ' WHERE Text="' . DB_WE::$inst->escape($footext) . '" AND ParentID=' . intval($templ->ParentID))){
					$z++;
					$footext = $templ->Filename . '_' . $z . $templ->Extension;
				}
				$templ->Filename = $templ->Filename . '_' . $z;
				$templ->Path = $templ->getPath();
			}
			$this->ParseTemplate($templ);
			$templ->we_save();
			$newID = $templ->ID;
			$templVars['newID'] = $newID;

			$_SESSION['weS']['WE_CREATE_TEMPLATE'][$templateID] = $newID;
			if($counter < 10){
				if($CreateMasterTemplate && $templ->MasterTemplateID > 0){
					if(isset($_SESSION['weS']['WE_CREATE_TEMPLATE'][$templ->MasterTemplateID])){
						$templ->MasterTemplateID = $_SESSION['weS']['WE_CREATE_TEMPLATE'][$templ->MasterTemplateID];
					} else {
						$createdMasterVars = $this->copyTemplate($templ->MasterTemplateID, $parentID, $CreateMasterTemplate, $CreateIncludedTemplate, $counter);
						$templ->MasterTemplateID = $createdMasterVars['newID'];
					}
					$templ->we_save();
				}
				if($CreateIncludedTemplate && !empty($templ->IncludedTemplates)){
					$includedTemplates = $templ->IncludedTemplates;
					$code = $templ->getElementS('data');
					foreach($includedTemplates as $incTempl){
						if(!empty($incTempl) && $incTempl > 0){
							if(isset($_SESSION['weS']['WE_CREATE_TEMPLATE'][$incTempl])){
								$newTemplId = $_SESSION['weS']['WE_CREATE_TEMPLATE'][$incTempl];
							} else {
								$createdIncVars = $this->copyTemplate(intval($incTempl), $parentID, $CreateMasterTemplate, $CreateIncludedTemplate, $counter);
								$newTemplId = $createdIncVars['newID'];
							}
							$tp = new we_weTag_parser($code);
							foreach($tp->getTagsWithAttributes() as $tag){
								if($tag['name'] === 'include' && !empty($tag['attribs']['type']) && $tag['attribs']['type'] === 'template'){
									$xid = (!empty($tag['attribs']['id']) ? $tag['attribs']['id'] : 0);
									if($xid == $incTempl){
										$newtag = preg_replace('|id *= *" *' . $xid . ' *"|i', 'id="' . $newTemplId . '"', $tag['weTag']);
										$code = str_replace($tag['weTag'], $newtag, $code);
									}
								}
							}
						}
					}
					$templ->setElementS('data', $code);
					$templ->we_save();
				}
			}
		}
		return $templVars;
	}

	private function ParseTemplate(we_contents_template &$we_doc): void{//FIXME: this is the same code as used for import
		// parse hard  coded  links in template  :TODO: check for ="/Path ='Path and =Path
		$we_doc->setElementS('data', str_replace($this->data['CopyFromPath'] . '/', $this->copyToPath . '/', $we_doc->getElementS('data')));

		$changed = false;
		$tp = new we_weTag_parser($we_doc->getElementS('data'));
		foreach($tp->getTagsWithAttributes() as $tag){
			$destTag = $tag['weTag'];
			$tagname = $tag['name'];
			if(!$tag['close'] && !empty(we_exim_searchPatterns::TEMPLATE_ID_PATTERNS[$tagname])){ // starttag found
				foreach(we_exim_searchPatterns::TEMPLATE_ID_PATTERNS[$tagname] as $attribname){
					if(!empty($tag['attribs'][$attribname]) && is_numeric($tag['attribs'][$attribname])){
						$path = id2path((int) $tag['attribs'][$attribname], FILE_TABLE);
						if($this->mustChange($path)){
							$changed = true;
							$pathTo = $this->getNewPath($path);
							$idTo = path_to_id($pathTo, FILE_TABLE) ?: we_base_constants::PLACEH_PATH . $pathTo . ' ' . we_base_constants::PLACEH_PATH;
							$destTag = preg_replace('/' . $attribname . '="' . $tag['attribs'][$attribname] . '"/', $attribname . '="' . $idTo . '"', $destTag);
						}
					}
				}
			}
			if($changed){
				$changed = false;
				$we_doc->setElementS('data', str_replace($tag, $destTag, $we_doc->getElementS('data')));
			}
		}
	}

	private function parseWeDocument(we_document_webEdition $we_doc): void{
		foreach($we_doc->elements as $k => $v){
			switch($v['type'] ?? ''){
				case 'txt' :
				case 'link'://since old fields are txt we have to handle it here
					$regs = [];
					$elem = $we_doc->getElementS($k);
					if(!$elem){//maybe bdid
						continue 2;
					}
					if(preg_match('|(.+)' . we_base_link::MAGIC_INFIX . '(.+)|', $k, $regs)){ // is a we:href field
						$v['type'] = 'href';
					} elseif(we_base_util::isSerialized($elem) && is_array($link = we_unserialize($elem))){ // is a we:link field
						if(isset($link['type']) && ($link['type'] === we_base_link::TYPE_INT)){
							$intID = (int) $link['id'];
							$path = id2path($intID, FILE_TABLE);
							if($this->mustChange($path)){
								$pathTo = $this->getNewPath($path);
								$link['id'] = path_to_id($pathTo, FILE_TABLE);
								$we_doc->setElementS($k, we_serialize($link), 'link');
							}
						}
					} else { // its a normal  text field
						$this->parseInternalLinks($we_doc->getElementS($k));
						// :TODO: check for ="/Path ='Path and =Path
						$we_doc->setElementS($k, str_replace($this->data['CopyFromPath'] . '/', $this->copyToPath . '/', $we_doc->getElementS($k)));
					}
					if($v['type'] != 'href'){
						break;
					}//fall through to correct field
				case 'href':
					$regs = [];
					if(preg_match('|(.+)' . we_base_link::MAGIC_INFIX . '(.+)|', $k, $regs)){ // is a we:href field
						$int = $we_doc->getElementI($regs[1] . we_base_link::MAGIC_INT_LINK);
						if($int){
							if(($intID = $we_doc->getElementI($regs[1] . we_base_link::MAGIC_INT_LINK_ID))){
								$path = id2path($intID, FILE_TABLE);
								if($this->mustChange($path)){
									$pathTo = $this->getNewPath($path);
									$idTo = path_to_id($pathTo, FILE_TABLE);
									if($idTo){
										$we_doc->setElementI($regs[1] . we_base_link::MAGIC_INT_LINK_ID, $idTo, 'href');
									} else {
										//FIXME: can this make trouble?!
										$we_doc->setElementS($regs[1] . we_base_link::MAGIC_INT_LINK_ID, we_base_constants::PLACEH_PATH . $pathTo . ' ' . we_base_constants::PLACEH_PATH, 'href');
									}
								}
							}
						} elseif(($path = $we_doc->getElementS($regs[1]))){
							if($this->mustChange($path)){
								$we_doc->setElementS($regs[1], $this->getNewPath($path));
							}
						}
					}
					break;

				case 'img' :
					$path = id2path($we_doc->getElementI($k), FILE_TABLE);
					if($this->mustChange($path)){
						$pathTo = $this->getNewPath($path);
						$idTo = path_to_id($pathTo, FILE_TABLE);
						if($idTo){
							$we_doc->setElementI($k, $idTo, 'img');
						} else {
							//FIXME: can this make trouble?!
							$we_doc->setElementS($k, we_base_constants::PLACEH_PATH . $pathTo . ' ' . we_base_constants::PLACEH_PATH, 'img');
						}
					}
					break;
			}
		}
	}

	private function getNewPath(string $oldPath): string{
		if($oldPath == $this->data['CopyFromPath']){
			return $this->copyToPath;
		}
		// :TODO: check for ='/Path ='Path and =Path
		return preg_replace('|^' . $this->data['CopyFromPath'] . '/|', $this->copyToPath . '/', $oldPath);
	}

	private function mustChange(string $path): bool{
		return str_starts_with($path, $this->data['CopyFromPath']);
	}

	private function parseTextDocument(we_document_text $we_doc): void{
		//:TODO: check for ='/Path ='Path and =Path
		$doc = str_replace($this->data['CopyFromPath'] . '/', $this->copyToPath . '/', $we_doc->getDocumentData());
		$we_doc->setElementS('data', $doc);
	}

	//FIXME: check why this is different to we_document::parseInternalLinks
	private function parseInternalLinks(string $text): string{
		$regs = [];
		if(!preg_match_all('/(href|src)="' . we_base_link::TYPE_INT_PREFIX . '([^" ]+)/i', $text, $regs, PREG_SET_ORDER)){
			return $text;
		}
		$allIds = id2pathA(array_column($regs, 2), FILE_TABLE);

		foreach($regs as $reg){
			$id = intval($reg[2]);
			$path = $allIds[$id] ?? '';
			if($path && $this->mustChange($path)){
				$pathTo = $this->getNewPath($path);
				$idTo = path_to_id($pathTo, FILE_TABLE) ?: we_base_constants::PLACEH_PATH . $pathTo . ' ' . we_base_constants::PLACEH_PATH;
				$text = preg_replace('#(href|src)="' . we_base_link::TYPE_INT_PREFIX . $id . '#i', $reg[1] . '="' . we_base_link::TYPE_INT_PREFIX . $idTo, $text);
			}
		}
		return $text;
	}

	private function getPid(string $path): int{
		$path = dirname($path);
		if($path === '/'){
			return 0;
		}
		return fI('SELECT ID FROM ' . FILE_TABLE . ' WHERE Path="' . DB_WE::$inst->escape($path) . '"');
	}

	protected function finish(): void{
		//$cancelButton = we_html_button::create_button(we_html_button::CANCEL, 'javascript:top.close()');

		if(isset($_SESSION['weS']['WE_CREATE_DOCTYPE'])){
			unset($_SESSION['weS']['WE_CREATE_DOCTYPE']);
		}

		if(isset($_SESSION['weS']['WE_CREATE_TEMPLATE'])){

			$this->jsCmd->addCmd('setProgress', [
				'progress' => 0,
				'name' => 'pbar1',
				'text' => g_l('copyFolder', '[prepareTemplates]')
			]);
			$this->jsCmd->addCmd('location', ['doc' => 'document', 'loc' => WEBEDITION_DIR . 'we_cmd.php?we_cmd[0]=we_folder_copy&finish=1']);
			#unset($_SESSION['weS']['WE_CREATE_TEMPLATE']);
		} elseif(!isset($_SESSION['weS']['WE_COPY_OBJECTS'])){
			$this->jsCmd->addCmd('we_cmd', ['load', FILE_TABLE]);
			$this->jsCmd->addMsg(g_l('copyFolder', '[copy_success]'), we_base_util::WE_MESSAGE_NOTICE);
			$this->jsCmd->addCmd('we_cmd', ['close']);
		} else {
			unset($_SESSION['weS']['WE_COPY_OBJECTS']);
			$this->jsCmd->addCmd('we_cmd', ['load', OBJECT_FILES_TABLE]);
			$this->jsCmd->addMsg(g_l('copyFolder', '[copy_success]'), we_base_util::WE_MESSAGE_NOTICE);
			$this->jsCmd->addCmd('we_cmd', ['close']);
		}
	}

}
