<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 4bec8f6a4c70 $
 * $Author: Marc Kraemer <marc.kraemer@webedition.org> $
 * $Date: 2020-12-15 12:10 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_gui_headermenu{

	/**
	 * creates a new messageConsole
	 *
	 * @param string $consoleName
	 * @return string
	 */
	public static function createMessageConsole(string $consoleName = 'NoName'): string{
		return '
<div id="messageConsole" onclick="_console_.openMessageConsole();">
<table><tr>
	<td style="vertical-align:middle"><div class="small messageConsoleMessage" id="messageConsoleMessage' . $consoleName . '">--</div></td>
	<td><div class="navigation" id="messageConsoleImageDiv"><i id="messageConsoleImage' . $consoleName . '" class="fa fa-lg fa-bell"></i></div></td>
	</tr></table>
</div>';
	}

	static function css(): string{
		return
			we_html_element::cssLink(CSS_DIR . 'menu.css') .
			we_html_element::jsScript(JS_DIR . 'menu/clickMenu.js');
	}

	//FIXME: this should not be executed directly, this should be a command which loads the data
	static function getMenuReloadCode(): array{
		$menu = self::getMenu();

		return ['updateMenu', $menu ? $menu->getHTML() : ''];
	}

	private static function getMenu(): ?we_gui_menu{
		if(// menu for normalmode
			we_editor_functions::isNormalMode()){
			return we_gui_mainMenu::getMenu();
		} // menu for seemode
		if(!we_base_permission::hasPerm([we_base_perm::SEEM_CAN_SEE_MENUE])){
			return null;
		}
		return we_gui_mainMenu::getMenu();
	}

	static function pbody(): string{
// all available elements
		$jmenu = self::getMenu();

		if(we_editor_functions::isNormalMode() ||
			we_base_permission::hasPerm([we_base_perm::SEEM_CAN_SEE_MENUE])){
			$navigationButtons = [];
		} else {
//  no menu in this case !
			$navigationButtons = [
				['onclick' => "top.we_cmd('dologout');", 'i' => 'close', 'text' => g_l('menu_global', '[close]')]
			];
		}
		$navigationButtons = array_merge($navigationButtons, [
			['onclick' => "WE().layout.navigationHistory.navigateReload();", 'i' => 'refresh', 'text' => g_l('menu_global', '[reload]')],
			/* ['onclick' => 'WE().layout.navigationHistory.navigateBack();', 'i' => 'caret-left', 'text' => g_l('menu_global', '[back]')],
			  ['onclick' => 'WE().layout.navigationHistory.navigateNext();', 'i' => 'caret-right', 'text' => g_l('menu_global', '[next]')], */
			]
		);
		$navi = '';
		foreach($navigationButtons as $button){
			$navi .= we_html_element::div(['class' => 'navigation', 'onclick' => $button['onclick']], '<i class="fa fa-' . $button['i'] . '" title="' . $button['text'] . '"></i>');
		}
		$versionInfo = we_base_util::updateAvailable();
		$update = ( $versionInfo ?
			we_html_element::div(['id' => 'newUpdate', 'class' => 'navigation'],
												'<i class="fa fa-lg fa-exclamation-circle" onclick="' . (we_base_permission::hasPerm([we_base_perm::ADMINISTRATOR]) ? "top.we_cmd('update');" : "") . '" title="' . printf(g_l('sysinfo', '[newWEAvailable]'), $versionInfo['dotted'] . ' (revision ' . $versionInfo['svnrevision'] . ')', $versionInfo['date']) . '"></i>') :
			'');

		return we_html_element::div([], we_html_element::div(['id' => 'home', 'class' => 'navigation', 'onclick' => "top.we_cmd('start_multi_editor');"], '<i class="fa fa-lg fa-home" title="' . g_l('menu_global', '[home]') . '"></i>') .
				we_html_element::div(['id' => 'weMainMenu'], ($jmenu ? $jmenu->getHTML() : '')) .
				we_html_element::div(['id' => 'navigationHistory'], $navi) .
				we_html_element::div(['id' => 'weHeaderRight'],
												 $update .
					(SIDEBAR_DISABLED ? '' :
					we_html_element::div(['class' => 'navigation', 'title' => g_l('weTag', '[sidebar][defaultvalue]'), 'onclick' => "WE().layout.sidebar.toggle();"], '<i class="fa fa-lg fa-toggle-right"></i>')
					) .
					self::createMessageConsole('mainWindow') .
					we_html_element::div(['id' => 'logout', 'class' => 'navigation', 'onclick' => "top.we_cmd('dologout');"], '<i class="fa fa-lg fa-power-off"></i>')
				)
		);
	}

}
