<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 980ead98a857 $
 * $Author: Marc Kraemer <marc.kraemer@webedition.org> $
 * $Date: 2020-12-17 19:11 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
abstract class we_gui_webEdition{

	private static $treeHtml = '';

	/**
	 * @return array
	 * @desc prints the functions needed for the tree.
	 */
	private static function getMainTree(we_base_jsCmd $jsCmd): array{
		$Tree = new we_tree_main($jsCmd);
		switch($_SESSION['weS']['we_mode']){
			default:
			case we_base_constants::MODE_NORMAL:
				return [$Tree->getJSTreeCode(), $Tree->getHTMLConstruct()];
			case we_base_constants::MODE_SEE:
				return [we_html_element::jsScript(JS_DIR . 'treeSeeMode.js'), $Tree->getHTMLConstruct()];
		}
	}

	private static function getTableToLoad(): string{
		if(we_base_permission::hasPerm([we_base_perm::WORK_DOCUMENTS_CAN_SEE])){
			return FILE_TABLE;
		} else if(we_base_permission::hasPerm([we_base_perm::WORK_TEMPLATES_CAN_SEE])){
			return TEMPLATES_TABLE;
		} else if(we_modules_info::isActive(we_modules_info::OBJECT) && we_base_permission::hasPerm([we_base_perm::OBJECTFILES_CAN_SEE])){
			return OBJECT_FILES_TABLE;
		} else if(we_modules_info::isActive(we_modules_info::OBJECT) && we_base_permission::hasPerm([we_base_perm::OBJECTS_CAN_SEE])){
			return OBJECT_TABLE;
		} else if(we_base_permission::hasPerm([we_base_perm::WORK_COLLECTIONS_CAN_SEE])){
			return VFILE_TABLE;
		}
		return '';
	}

//	Here begins the code for showing the correct frameset.
	private static function multiContentFrame(): string{
		$isAppleTouch = (we_base_browserDetect::inst()->getSystem() == we_base_browserDetect::SYS_IPAD || we_base_browserDetect::inst()->getSystem() == we_base_browserDetect::SYS_IPHONE);
//	To improve readability the different cases are outsourced
		$count = (isset($_SESSION) && we_editor_functions::isSeeMode()) ? 1 : 32;
		$frames = '';
//	in several functions, for SEEM, normal or edit_include-Mode.
		for($i = 0; $i < $count; $i++){

			$frames .= '<iframe style="' . ($i == 0 ? '' : (we_base_browserDetect::isChrome() ? 'display:none;' : 'width:0;height:0;')) . '" class="multiEditFrame" name="multiEditFrame_' . $i . '" id="multiEditFrame_' . $i . '" noresize ></iframe>';
		}

		return '
<div id="multiEditorDocumentTabsFrameDiv">
	<div id="weMultiTabs">
		<div id="tabContainer" name="tabContainer"></div>
		<div class="hidden" id="tabDummy" onclick="WE().layout.multiTabs.selectFrame(this)">
			<span class="spacer status" id="###loadId###" title="" ></span>
			<span id="###tabTextId###" class="cutText text"></span>
			<span class="spacer trailing">
				<i class="fa fa-asterisk modified" id="###modId###"></i>
				<span class="close" id="###closeId###" onclick="WE().layout.multiTabs.onCloseTab(this)">
					<i class="fa fa-close fa-lg "></i>
				</span>
		</div>
	</div>
</div>
<div id="multiEditorEditorFramesetsDiv" class="' . ($isAppleTouch ? 'iframeScrollIpad' : '') . '">' .
				$frames . '
</div>'
		;
	}

//	Here begins the code for showing the correct frameset.
//	To improve readability the different cases are outsourced
//	in several functions, for SEEM, normal or edit_include-Mode.

	private static function getSidebarWidth(): int{
// Get the width of the sidebar
		if(SIDEBAR_DISABLED != 1 && SIDEBAR_SHOW_ON_STARTUP == 1){
			return SIDEBAR_DEFAULT_WIDTH;
		}
		return 0;
	}

	private static function getVTabs(int $treewidth): string{
		$tabs = [];
//set/update locks & active documents
		$vtab = [
			'FILE_TABLE' => [
				'show' => we_base_permission::hasPerm([we_base_perm::WORK_DOCUMENTS_CAN_SEE]),
				'icon' => 'fa-file-o',
				'desc' => g_l('global', '[documents]'),
			],
			'TEMPLATES_TABLE' => [
				'show' => we_base_permission::hasPerm([we_base_perm::WORK_TEMPLATES_CAN_SEE]),
				'icon' => 'fa-file-code-o',
				'desc' => g_l('global', '[templates]'),
			],
			'OBJECT_FILES_TABLE' => [
				'show' => we_modules_info::isActive(we_modules_info::OBJECT) && we_base_permission::hasPerm([we_base_perm::OBJECTFILES_CAN_SEE]),
				'icon' => 'fa-file',
				'desc' => g_l('global', '[objects]'),
			],
			'OBJECT_TABLE' => [
				'show' => we_modules_info::isActive(we_modules_info::OBJECT) && we_base_permission::hasPerm([we_base_perm::OBJECTS_CAN_SEE]),
				'icon' => 'fa-code',
				'desc' => g_l('modules_object', '[menu][classes]'),
			],
			'VFILE_TABLE' => [
				'show' => we_modules_info::isActive(we_modules_info::COLLECTION) && we_base_permission::hasPerm([we_base_perm::WORK_COLLECTIONS_CAN_SEE]),
				'icon' => 'fa-archive',
				'desc' => g_l('global', '[vfile]'),
			]
		];
		foreach($vtab as $tab => $val){
			if($val['show']){
				$tabs[] = we_html_element::div([
							'class' => 'tab tabNorm',
							'onclick' => "WE().layout.vtab.click(this,'" . constant($tab) . "');",
							'data-table' => constant($tab)],
								'<span class="description middlefont" title="' . $val['desc'] . '"><i class="icon fa ' . $val['icon'] . '"></i> <span class="text">' . $val['desc'] . '</span></span>');
			}
		}
		return we_html_element::div(['id' => 'vtabs'],
						implode('', $tabs) .
						we_html_element::div(['id' => 'baumArrows'], we_html_element::div([
									'class' => "baumArrow",
									'id' => "incBaum",
									'title' => g_l('global', '[tree][grow]'),
									'style' => ($treewidth <= 100 ? 'background-color: grey' : ''),
									'onclick' => "WE().layout.tree.inc();"],
										'<i class="fa fa-plus"></i>'
								) .
								we_html_element::div([
									'class' => "baumArrow",
									'id' => "decBaum",
									'title' => g_l('global', '[tree][reduce]'),
									'style' => ($treewidth <= 100 ? 'background-color: grey' : ''),
									'onclick' => "WE().layout.tree.dec();"],
										'<i class="fa fa-minus"></i>'
								)
						)
		);
	}

	private static function getTreeFrame(int $treewidth, string $treeHtml): string{
		ob_start();
		?>
		<div id="treeFrameDiv">
			<div id="treeControl">
				<span id="treeName" class="middlefont"></span>
				<span id="reloadTree" onclick="we_cmd('loadVTab', top.treeData.table, 0);"><i class="fa fa-refresh"></i></span>
				<span id="toggleTree" onclick="WE().layout.tree.toggle();" title="<?= g_l('global', '[tree][minimize]'); ?>"><i id="arrowImg" class="fa fa-lg fa-caret-<?= ($treewidth <= 100) ? "right" : "left"; ?>" ></i></span>
			</div>
			<div id="treeContent">
				<div id="bm_treeheaderDiv">
					<iframe src="about:blank" name="treeheader"></iframe>
				</div>
				<?= $treeHtml; ?>
				<div id="bm_searchField">
					<div id="infoField" class="defaultfont"></div>
					<form name="<?= we_base_constants::WE_FORM; ?>" onsubmit="WE().layout.tree.handleSearch(document.we_form.keyword.value, top.treeData.table);
									return false;">
						<div id="search">
							<?=
							we_html_tools::textInput('keyword', we_base_request::string('keyword', ''), 0, ['id' => 'searchInputTree', 'placeholder' => g_l('searchtool', '[placeholder]'),
								'onkeyup' => "WE().layout.tree.setSearchButtonIcon(window,top.treeData.table)"], 'search') .
							we_html_button::create_button(we_html_button::SEARCH, "javascript:WE().layout.tree.handleSearch(document.we_form.keyword.value, top.treeData.table);", '', '', false, false, '', false, '', '', 'searchButtonTree');
							?>
						</div>
					</form>
				</div>
			</div>
		</div><?php
		return ob_get_clean() ?: '';
	}

//	unlock everything old, when a new window is opened.
	private static function init(): void{





		if(empty($_SESSION['weS']['we_mode'])){
			$_SESSION['weS']['we_mode'] = we_base_constants::MODE_NORMAL;
		}
//	check session
		we_html_tools::protect([], WEBEDITION_DIR . 'index.php');
		self::cleanup();
	}

//we_tool_lookup::getJsCmdInclude($jsWeCmd);
	private static function cleanup(): void{
		we_base_file::cleanTempFiles();

//set/update locks & active documents
		we_users_user::updateActiveUser(true);

//	unlock everything old, when a new window is opened.
		DB_WE::$inst->query('DELETE FROM ' . LOCK_TABLE . ' WHERE lockTime<NOW()');
		DB_WE::$inst->query('UPDATE ' . USER_TABLE . '	SET Ping=NULL WHERE Ping<(NOW()-INTERVAL ' . (we_base_constants::PING_TIME + we_base_constants::PING_TOLERANZ) . ' second)');
	}

	private static function getConsts(array $jsmods): array{
		$const = [
			//'g_l' => [],
			'contentTypes' => [
				'APPLICATION' => we_base_ContentTypes::APPLICATION,
				'AUDIO' => we_base_ContentTypes::AUDIO,
				'CATEGORIES' => we_base_ContentTypes::CATEGORIES,
				'CATEGORY' => we_base_ContentTypes::CATEGORY,
				'COLLECTION' => we_base_ContentTypes::COLLECTION,
				'CSS' => we_base_ContentTypes::CSS,
				'CUSTOMER' => we_base_ContentTypes::CUSTOMER,
				'CUSTOMERGROUP' => we_base_ContentTypes::CUSTOMERGROUP,
				'FOLDER_COMMON' => we_base_ContentTypes::FOLDER_COMMON,
				'FOLDER_DOCUMENT' => we_base_ContentTypes::FOLDER_DOCUMENT,
				'FOLDER_OBJECTFILE' => we_base_ContentTypes::FOLDER_OBJECTFILE,
				'FOLDER_TEMPLATE' => we_base_ContentTypes::FOLDER_TEMPLATE,
				'FOLDER_PHPCLASS' => we_base_ContentTypes::FOLDER_PHPCLASS,
				'FOLDER_VFILE' => we_base_ContentTypes::FOLDER_VFILE,
				'GLOSSAR' => we_base_ContentTypes::GLOSSAR,
				'HTACCESS' => we_base_ContentTypes::HTACCESS,
				'HTML' => we_base_ContentTypes::HTML,
				'IMAGE' => we_base_ContentTypes::IMAGE,
				'JS' => we_base_ContentTypes::JS,
				'NAVIGATION' => we_base_ContentTypes::NAVIGATION,
				'NEWSLETTER' => we_base_ContentTypes::NEWSLETTER,
				'OBJECT' => we_base_ContentTypes::OBJECT,
				'OBJECT_FILE' => we_base_ContentTypes::OBJECT_FILE,
				'PDF' => we_base_ContentTypes::PDF,
				'PHPCLASS' => we_base_ContentTypes::PHPCLASS,
				'TEMPLATE' => we_base_ContentTypes::TEMPLATE,
				'TEXT' => we_base_ContentTypes::TEXT,
				'USER' => we_base_ContentTypes::USER,
				'USERALIAS' => we_base_ContentTypes::USERALIAS,
				'USERGROUP' => we_base_ContentTypes::USERGROUP,
				'VIDEO' => we_base_ContentTypes::VIDEO,
				'VOTING' => we_base_ContentTypes::VOTING,
				'WEDOCUMENT' => we_base_ContentTypes::WEDOCUMENT,
				'XML' => we_base_ContentTypes::XML,
			],
			'dirs' => [
				'WEBEDITION_DIR' => WEBEDITION_DIR,
				'WE_LIB_DIR' => WE_LIB_DIR,
				'WE_JS_TINYMCE_DIR' => WE_JS_TINYMCE_DIR,
			],
			'modules' => [
				'active' => we_modules_info::getAllActiveModules(),
				'jsmods' => $jsmods,
				'inactive' => array_values(we_modules_info::getAllInactiveModules()),
			],
			'global' => [
				'WE_EDITPAGE_CONTENT' => we_base_constants::WE_EDITPAGE_CONTENT,
				'WE_EDITPAGE_PREVIEW' => we_base_constants::WE_EDITPAGE_PREVIEW,
				'WE_EDITPAGE_PREVIEW_TEMPLATE' => we_base_constants::WE_EDITPAGE_PREVIEW_TEMPLATE,
				'DISABLED_FOOTER_PAGES' => [
					we_base_constants::WE_EDITPAGE_INFO,
					we_base_constants::WE_EDITPAGE_VALIDATION,
					we_base_constants::WE_EDITPAGE_DOCLIST,
					we_base_constants::WE_EDITPAGE_TEMPLATE_UNUSEDELEMENTS
				],
				'PING_TIME' => (we_base_constants::PING_TIME * 1000),
				'DEFAULT_DYNAMIC_EXT' => DEFAULT_DYNAMIC_EXT,
				'DEFAULT_STATIC_EXT' => DEFAULT_STATIC_EXT,
			],
			'browserDetect' => [
				'browser' => we_base_browserDetect::inst()->getBrowser(),
				'system' => we_base_browserDetect::inst()->getSystem(),
			],
			'folderAction' => [
				'ACTION_CATEGORY_ADD' => we_folder_base::ACTION_CATEGORY_ADD,
				'ACTION_CATEGORY_DELETE' => we_folder_base::ACTION_CATEGORY_DELETE,
			],
			'message' => [
				'WE_MESSAGE_INFO' => we_base_util::WE_MESSAGE_INFO,
				'WE_MESSAGE_FRONTEND' => we_base_util::WE_MESSAGE_FRONTEND,
				'WE_MESSAGE_NOTICE' => we_base_util::WE_MESSAGE_NOTICE,
				'WE_MESSAGE_WARNING' => we_base_util::WE_MESSAGE_WARNING,
				'WE_MESSAGE_ERROR' => we_base_util::WE_MESSAGE_ERROR,
			],
			'linkPrefix' => [
				'TYPE_OBJ_PREFIX' => we_base_link::TYPE_OBJ_PREFIX,
				'TYPE_INT_PREFIX' => we_base_link::TYPE_INT_PREFIX,
				'TYPE_MAIL_PREFIX' => we_base_link::TYPE_MAIL_PREFIX,
				'TYPE_THUMB_PREFIX' => we_base_link::TYPE_THUMB_PREFIX,
				'TYPE_INT' => we_base_link::TYPE_INT,
				'TYPE_EXT' => we_base_link::TYPE_EXT,
				'TYPE_ALL' => we_base_link::TYPE_ALL,
				'EMPTY_EXT' => we_base_link::EMPTY_EXT,
			],
			'linkContent' => [
				'CONTENT_TEXT' => we_base_link::CONTENT_TEXT
			],
			'size' => [
				'tree' => [
					'hidden' => we_tree_base::HiddenWidth,
					'defaultWidth' => we_tree_base::DefaultWidth,
					'min' => we_tree_base::MinWidth,
					'max' => we_tree_base::MaxWidth,
					'step' => we_tree_base::StepWidth,
					'moveWidth' => we_tree_base::MoveWidth,
					'addToCollectionWidth' => we_tree_base::AddToCollectionWidth,
					'deleteWidth' => we_tree_base::DeleteWidth,
				],
				'dialog' => [
					'xtiny' => 160,
					'tiny' => 300,
					'smaller' => 400,
					'small' => 600,
					'medium' => 800,
					'big' => 1000,
					'bigger' => 1200,
					'fullScreen' => 2500
				],
				'sidebar' => [
					'defaultWidth' => intval(defined('SIDEBAR_DEFAULT_WIDTH') ? SIDEBAR_DEFAULT_WIDTH : 0),
				]
			],
			'tables' => [
				'TBL_PREFIX' => TBL_PREFIX,
				//FIXME: in js we check for 'OBJECT_FILES_TABLE' or 'OBJECT_TABLE' if active
				'OBJECT_FILES_TABLE' => we_modules_info::isActive(we_modules_info::OBJECT) ? OBJECT_FILES_TABLE : 'OBJECT_FILES_TABLE',
				'OBJECT_TABLE' => we_modules_info::isActive(we_modules_info::OBJECT) ? OBJECT_TABLE : 'OBJECT_TABLE',
			],
			'graphic' => [
				'gdSupportedTypes' => [],
				'canRotate' => intval(function_exists('ImageRotate')),
			],
			'tabs' => [
				'PREVIEW' => we_base_constants::WE_EDITPAGE_PREVIEW,
				'navigation' => [
					'PROPERTIES' => we_navigation_frames::TAB_PROPERTIES,
					'CONTENT' => we_navigation_frames::TAB_CONTENT,
					'CUSTOMER' => we_navigation_frames::TAB_CUSTOMER,
					'PREVIEW' => we_navigation_frames::TAB_PREVIEW,
				]
			],
			'exim' => [
				'TYPE_CSV' => we_exim_ExIm::TYPE_CSV,
				'TYPE_XML' => we_exim_ExIm::TYPE_XML,
				'TYPE_WE' => we_exim_ExIm::TYPE_WE,
				'TYPE_LOCAL_FILES' => we_exim_ExIm::TYPE_LOCAL_FILES,
				'TYPE_SITE' => we_exim_ExIm::TYPE_SITE,
				'export' => [
					'SELECTIONTYPE_CLASSNAME' => we_exim_Export::SELECTIONTYPE_CLASSNAME,
					'SELECTIONTYPE_DOCTYPE' => we_exim_Export::SELECTIONTYPE_DOCTYPE,
					'SELECTIONTYPE_DOCUMENT' => we_exim_Export::SELECTIONTYPE_DOCUMENT
				]
			],
			'uncache' => md5(WE_VERSION . filemtime(WE_INCLUDES_PATH . 'we_version.php')),
		];

		foreach(we_base_imageEdit::supported_image_types() as $v){
			$const['graphic']['gdSupportedTypes'][$v] = true;
		}

		foreach(we_base_request::getAllTables() as $k => $v){
			$const['tables'][$k] = $v;
		}
//all relevant settings for current session
		return $const;
	}

	/**
	 * all relevant settings for current session
	 * @return array
	 */
	private static function getSession(): array{
		$session = [
			'seemode' => (int) we_editor_functions::isSeeMode(),
			'userID' => we_users_user::getID(),
//permissions set for the user
			'permissions' => [],
			'sess_id' => session_id(),
			'specialUnload' => intval(!(we_base_browserDetect::isChrome() || we_base_browserDetect::isSafari() || we_base_browserDetect::isFF() || we_base_browserDetect::isEdge() )),
			'lang' => [
				'short' => array_search(we_base_WE::$Language, we_base_country::getWELangs(), true),
				'long' => we_base_WE::$Language
			],
			'messageSettings' => (!empty($_SESSION['prefs']['message_reporting']) ? we_base_util::WE_MESSAGE_INFO | we_base_util::WE_MESSAGE_ERROR | $_SESSION['prefs']['message_reporting'] : PHP_INT_MAX),
			'isChrome' => we_base_browserDetect::isChrome(),
			'isAppleTouch' => (/* we_base_browserDetect::inst()->isSafari() */ (we_base_browserDetect::inst()->getSystem() == we_base_browserDetect::SYS_IPAD || we_base_browserDetect::inst()->getSystem() == we_base_browserDetect::SYS_IPHONE)),
			'isMac' => we_base_browserDetect::isMAC(),
			'deleteMode' => 0,
		];
		foreach($_SESSION['perms'] as $perm => $access){
			$session['permissions'][$perm] = (!empty($_SESSION['perms'][we_base_perm::ADMINISTRATOR]) ? 1 : intval($access));
		}

		return $session;
	}

	private static function getHead(): string{
		$jsCmd = new we_base_jsCmd();

		$jsWeCmd = $jsmods = [];
		foreach(we_modules_info::getAllActiveModules() as $mod){
			if(file_exists(JS_PATH . 'we/we_cmd_modules/we_webEditionCmd_' . $mod . '.js')){
				$jsWeCmd[$mod] = JS_DIR . 'we/we_cmd_modules/we_webEditionCmd_' . $mod . '.js';
			}
			$jsmods[] = $mod;
		}
		$jsmods[] = 'base';

		$head = we_html_element::jsScript(JS_DIR . 'we/webEdition.js', '', [
					'id' => 'loadWEData',
					'data-session' => setDynamicVar(self::getSession()),
					'data-consts' => setDynamicVar(self::getConsts($jsmods)),
				]) .
				we_html_element::jsScript(JS_DIR . 'globalWE.js') .
				we_html_element::jsScript(JS_DIR . 'we/layout.js') .
				we_html_element::jsScript(JS_DIR . 'we/we_cmd_modules/we_webEditionCmd_base.js') .
				we_html_element::jsScript(JS_DIR . 'we/layout/navigationHistory.js', '', ['defer' => 'defer']) .
				we_html_element::jsScript(JS_DIR . 'we/handler/keyListener.js') .
				we_html_element::jsScript(JS_DIR . 'we/layout/jsWindow.js') .
				we_html_element::jsScript(JS_DIR . 'we/layout/overlay.js') .
				we_html_element::jsScript(JS_DIR . 'we/layout/we_tabs.js') .
				we_html_element::jsScript(JS_DIR . 'we/layout/weSuggest.js') .
				we_html_element::jsScript(JS_DIR . 'we/layout/messageConsole.js') .
				we_html_element::jsScript(JS_DIR . 'we/layout/sidebar.js') .
				we_html_element::jsScript(JS_DIR . 'we/layout/toggleBox.js', '', ['defer' => 'defer']) .
				we_html_element::jsScript(JS_DIR . 'we/layout/combobox.js', '', ['defer' => 'defer']) .
				we_html_element::jsScript(JS_DIR . 'we/layout/button.js', '', ['defer' => 'defer']) .
				we_html_element::jsScript(JS_DIR . 'we/util.js') .
				we_html_element::jsScript(JS_DIR . 'we/util/getTreeIcon.js') .
				we_html_element::jsScript(JS_DIR . 'we/util/showConfirm.js') .
				we_html_element::jsScript(JS_DIR . 'we/util/showMessage.js') .
				we_html_element::jsScript(JS_DIR . 'we_users_ping.js') .
				we_html_element::jsScript(JS_DIR . 'we/util/multi_edit.js') .
				(we_modules_info::isActive(we_modules_info::COLLECTION) ?
				we_html_element::jsScript(JS_DIR . 'we/layout/collection.js', '', ['defer' => 'defer']) : '') .
				we_html_element::jsScript(JS_DIR . 'we/layout/ImageEditTools.js', '', ['defer' => 'defer']) .
				we_fileupload_ui_base::getJSFileuploader() .
				we_fileupload_ui_base::getJSUtils(true) .
				we_html_element::jsScript(WE_JS_TINYMCE_DIR . 'weTinyMce_config.js', '', ['defer' => 'defer']) .
				we_html_element::jsScript(WE_JS_TINYMCE_DIR . 'weTinyMce_functionsTop.js', '', [
					'id' => 'loadWeicons',
					'data-weicons' => setDynamicVar(we_wysiwyg_editor::getWeicons()),
					'defer' => 'defer'
				]) .
				we_html_element::jsScript(WE_JS_TINYMCE_DIR . 'weTinyMce_tinyWrapper.js', '', ['defer' => 'defer']) .
				we_gui_headermenu::css() .
				we_html_element::cssLink(CSS_DIR . 'sidebar.css') .
				we_html_element::cssLink(CSS_DIR . 'multiEditor.css') .
				we_html_element::jsScript(JS_DIR . 'we/layout/TabView.js') .
				we_html_element::jsScript(JS_DIR . 'we/layout/EditorFrame.js') .
				we_html_element::jsScript(JS_DIR . 'we/layout/EditorFrameController.js');

		foreach($jsWeCmd as $cur){
			$head .= we_html_element::jsScript($cur);
		}

		[$jsTree, self::$treeHtml] = self::getMainTree($jsCmd);

		$head .= $jsTree . $jsCmd->getCmds();
		return $head;
	}

	private static function getBody(): string{
		//$versionInfo = empty($_SESSION['perms'][we_base_perm::ADMINISTRATOR]) ? [] : we_base_util::updateAvailable();
		ob_start();
		/* updateCheck(<?= (empty($versionInfo) ? '0,0,0' : '1,\'' . $versionInfo['dotted'] . ' (revision ' . $versionInfo['svnrevision'] . ')\',\'' . $versionInfo['date'] . '\'') ?>);
		 */
		?>
		<body id="weMainBody" onload="initWE();
						startMultiEditor(['<?= implode("','", we_gui_multiEditor::start()); ?>']);
						top.start('<?= self::getTableToLoad(); ?>');
						top.showMainWindow();
						startMsg();
						checkPwd(<?= intval(empty($_SESSION['WE_USER_PASSWORD_NOT_SUFFICIENT'])); ?>);
						self.focus();"
					onbeforeunload ="return doUnload();"
					onresize ="if(WE().layout.multiTabs){WE().layout.multiTabs.setFrameSize()}"
					><?= we_html_element::div(['id' => 'loginScreen'], '<div id="loading"><i class="fa fa-5x fa-spinner fa-pulse"></i></div>'); ?>
			<dialog id="alertBox"></dialog>
			<div id="weMainDiv"><?=
				we_html_element::div(['id' => 'headerDiv', 'class' => MAINTENANCE_MODE],
						we_html_element::div(['id' => 'weMainHeader'],
								we_gui_headermenu::pbody())
				);
				?>
				<div id="resizeFrame"><?php
					$sidebarwidth = self::getSidebarWidth();
					switch($_SESSION['weS']['we_mode']){
						default:
						case we_base_constants::MODE_NORMAL:
							$treewidth = isset($_COOKIE['treewidth_main']) && ($_COOKIE['treewidth_main'] >= we_tree_base::MinWidth) ? intval($_COOKIE['treewidth_main']) : we_tree_base::DefaultWidth;
							$treeStyle = 'display:block;';
							break;
						case we_base_constants::MODE_SEE:
							$treewidth = 0;
							$treeStyle = '';
							break;
					}
					echo we_html_element::div([
						'id' => 'bframeDiv',
						'style' => 'width:' . $treewidth . 'px;' . $treeStyle,
							],
							self::getVTabs($treewidth) .
							self::getTreeFrame($treewidth, self::$treeHtml)
					) .
					we_html_element::div([
						'id' => 'bm_content_frameDiv',
						'style' => 'right:' . $sidebarwidth . 'px;left:' . $treewidth . 'px;'],
							self::multiContentFrame()) .
					(SIDEBAR_DISABLED ? '' :
							we_html_element::div([
								'id' => 'sidebarDiv',
								'style' => 'width:' . $sidebarwidth . 'px;'
									],
									we_sidebar_frames::getHTML('')
							)
					);
					?>
				</div>
				<div id="cmdDiv">
					<iframe src="about:blank" name="load"></iframe>
					<iframe src="about:blank" name="load2"></iframe>
					<iframe src="about:blank" name="plugin"></iframe>
				</div>
				<div style="display:block;position:absolute; left: 10000px;" id="tinyIconsDiv">
				</div>
		</body>
		<?php
		return ob_get_clean() ?: '';
	}

	public static function start(): void{
		self::init();

		echo we_html_tools::getHtmlTop('webEdition - ' . $_SESSION['user']['Username'], self::getHead(), self::getBody());
		unset($_SESSION['WE_USER_PASSWORD_NOT_SUFFICIENT']);
		if(we_modules_info::isActive(we_modules_info::SCHEDULER)){
			we_base_request::finishRequest();
// trigger scheduler
			we_schedule_scheduler::trigger_schedule(30);
		}
	}

}
