<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: f727db85ca06 $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2021-01-16 13:18 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
abstract class we_import_functions{

	/**
	 * @return boolean
	 * @param integer $parentID
	 * @param integer $templateID
	 * @param array<string,mixed> $fields
	 * @param string[] $dateFields
	 * @param integer $doctypeID
	 * @param int[] $categories
	 * @param string $filename
	 * @param boolean $isDynamic
	 * @param string $extension
	 * @param boolean $publish
	 * @param boolean $IsSearchable
	 * @desc imports a document into webedition
	 */
	public static function importDocument(int $parentID, int $templateID, array $fields, array $dateFields, int $doctypeID, array $categories, string $filename, bool $isDynamic, string $extension, bool $publish, bool $IsSearchable, string $conflict = we_exim_ExIm::COLLISION_RENAME): bool{

		// erzeugen eines neuen webEdition-Dokument-Objekts
		$we_doc = new we_document_webEdition();

		$we_doc->we_new();

		$we_doc->Extension = $extension;
		if($filename){
			$filename = self::correctFilename($filename);
			$we_doc->Filename = $filename;
		}

		$we_doc->setParentID($parentID);
		// IF NAME OF OBJECT EXISTS, WE HAVE TO CREATE A NEW NAME
		if(($file_id = fI('SELECT ID FROM ' . FILE_TABLE . ' WHERE Path="' . DB_WE::$inst->escape($we_doc->Path) . '"'))){
			if($conflict === we_exim_ExIm::COLLISION_RENAME){
				$z = 0;
				$footext = $we_doc->Filename . '_' . $z . $we_doc->Extension;
				while(fI('SELECT ID FROM ' . FILE_TABLE . ' WHERE Text="' . DB_WE::$inst->escape($footext) . '" AND ParentID=' . intval($we_doc->ParentID))){
					$z++;
					$footext = $we_doc->Filename . '_' . $z . $we_doc->Extension;
				}
				$we_doc->Filename = $we_doc->Filename . '_' . $z;
				$we_doc->Path = $we_doc->getPath();
			} else if($conflict === we_exim_ExIm::COLLISION_REPLACE){
				$we_doc->initById($file_id);
			} else {
				return true;
			}
		}

		$we_doc->DocType = $doctypeID;
		$we_doc->setTemplateID($templateID);
		$we_doc->Category = $categories;

		$we_doc->ContentType = we_base_ContentTypes::WEDOCUMENT;

		$we_doc->IsDynamic = $isDynamic;
		$we_doc->IsSearchable = $IsSearchable;
		foreach($fields as $fieldName => $fieldValue){
			//quick hack for numbers and dates
			if(is_numeric($fieldValue)){
				$we_doc->setElementI($fieldName, (int) $fieldValue, in_array($fieldName, $dateFields, true) ? 'date' : 'txt');
			} else {
				$we_doc->setElementS($fieldName, (string) $fieldValue);
			}
		}

		// SAVE DOCUMENT
		if(!$we_doc->we_save()){
			return false;
		}
		// PUBLISH OR EXIT
		return ($publish ?
			$we_doc->we_publish() :
			true
			);
	}

	/**
	 * @return boolean
	 * @param integer $classID
	 * @param array<string,mixed> $fields
	 * @param int[] $categories
	 * @param string $filename
	 * @param boolean $publish
	 * @desc imports an object into webEdition
	 */
	public static function importObject(int $classID, array $fields, array $categories, string $filename, bool $publish, bool $issearchable, int $parentID = 0, string $conflict = we_exim_ExIm::COLLISION_RENAME): bool{

		// INIT OBJECT
		$object = new we_contents_objectFile();
		$object->we_new();
		$object->TableID = $classID;
		$object->setRootDirID(true);
		$object->resetParentID();
		$object->restoreDefaults();
		if($categories){
			$object->Category = $categories;
		}
		$object->IsSearchable = $issearchable;
		if($parentID){
			$object->setParentID($parentID);
		}

		// IF WE HAVE TO GIVE THE OBJECT A NAME
		if($filename || $filename === '0'){
			$object->Filename = we_import_functions::correctFilename($filename);
			$object->Path = $object->getParentPath() . (($object->getParentPath() != '/') ? '/' : '') . $object->getText();
			// IF NAME OF OBJECT EXISTS, WE HAVE TO CREATE A NEW NAME
			if(($file_id = fI('SELECT ID FROM ' . OBJECT_FILES_TABLE . ' WHERE ParentID=' . intval($object->ParentID) . ' AND Filename="' . DB_WE::$inst->escape($object->Filename) . '"'))){
				switch($conflict){
					case we_exim_ExIm::COLLISION_REPLACE:
						$object->initByID($file_id);
						break;
					case we_exim_ExIm::COLLISION_RENAME:
						$z = 0;
						$footext = $object->getText($z);
						while(fI('SELECT ID FROM ' . OBJECT_FILES_TABLE . ' WHERE Filename="' . DB_WE::$inst->escape($footext) . '" AND ParentID=' . intval($object->ParentID))){
							$z++;
							$footext = $object->getText($z);
						}
						$object->Filename = $footext;
						$object->Path = $object->getParentPath() . (($object->getParentPath() != '/') ? '/' : '') . $object->getText();
						break;
					default:
						return true;
				}
			}
		}

		// FILL FIELDS OF OBJECT
		foreach($fields as $fieldName => $fieldValue){
			$object->setElementS($fieldName, $fieldValue);
		}
		// SAVE OBJECT
		if(!$object->we_save()){
			return false;
		}
		// PUBLISH OR EXIT
		return ($publish ?
			$object->we_publish() :
			true);
	}

	/**
	 * @return string
	 * @param string $filename
	 * @desc corrects the filename if it contains invalid chars
	 */
	static function correctFilename(string $filename, bool $allowPath = false): string{
		$filename = preg_replace('%[^a-z0-9\._+\-@' . ($allowPath ? '/' : '') . ']%i', '', trim(we_base_util::convertDisallowedChars($filename), '/'));

		if(!$allowPath && strlen($filename) > 100){
			$pos = (int) strrpos($filename, '.');
			$ext = (string) substr($filename, $pos + 1, 16);
			$name = (string) substr($filename, 0, min($pos, 100 - strlen($ext)));
			$filename = $name . '.' . $ext;
		}
		return $filename ?: 'newfile';
	}

	/**
	 * @return int
	 * @param string $datestring
	 * @param string $format
	 * @desc converts a $datestring which represent a date to an unix timestamp with the given $format. If $format is empty, $datestring has to be a valid English date format
	 */
	static function date2Timestamp(string $datestring, string $format = ''): int{
		if(!$format){
			return strtotime($datestring) ?: 0;
		}

		$replaceorder = [];

		$formatchars = ['Y', 'y', 'm', 'n', 'd', 'j', 'H', 'G', 'i', 's'];

		$eregchars = implode('', $formatchars);

		foreach($formatchars as $char){
			$format = str_replace("\\" . $char, we_base_constants::PLACEHOLDER . ord($char) . we_base_constants::PLACEHOLDER, $format);
		}
		$matches = [];
		if(preg_match_all('/[' . $eregchars . ']/', $format, $matches, PREG_SET_ORDER)){
			foreach($matches as $match){
				if(is_array($match) && isset($match[0])){
					$replaceorder[] = $match[0];
				}
			}
		}

		$eregformat = preg_replace('/([' . $eregchars . '])/', '([0-9]+)', str_replace('/', '\\/', preg_quote($format)));

		foreach($formatchars as $char){
			$eregformat = str_replace(we_base_constants::PLACEHOLDER . ord($char) . we_base_constants::PLACEHOLDER, '\\' . $char, $eregformat);
		}

		$outarray = [
			'hour' => 1,
			'minute' => 0,
			'second' => 0,
			'month' => 1,
			'day' => 1,
			'year' => 1970
		];

		if(preg_match_all('/' . $eregformat . '/', $datestring, $matches, PREG_SET_ORDER)){

			if(isset($matches[0]) && is_array($matches[0])){
				for($i = 1; $i < count($matches[0]); $i++){
					if(isset($replaceorder[$i - 1])){
						switch($replaceorder[$i - 1]){
							case 'y':
							case 'Y':
								$outarray['year'] = (int) $matches[0][$i];
								break;
							case 'm':
							case 'n':
								$outarray['month'] = (int) $matches[0][$i];
								break;
							case 'd':
							case 'j':
								$outarray['day'] = (int) $matches[0][$i];
								break;
							case 'H':
							case 'G':
								$outarray['hour'] = (int) $matches[0][$i];
								break;
							case 'i':
								$outarray['minute'] = (int) $matches[0][$i];
								break;
							case 's':
								$outarray['second'] = (int) $matches[0][$i];
								break;
						}
					}
				}
			}

			return (int) mktime($outarray['hour'], $outarray['minute'], $outarray['second'], $outarray['month'], $outarray['day'], $outarray['year']);
		}
		return 0;
	}

}
