<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 3b8044c13b9c $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-12-16 00:05 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_navigation_dirSelector extends we_selector_directory{

	public function __construct(int $id, string $JSIDName = '', string $JSTextName = '', string $JSCommand = '', string $order = '', string $FolderText = ''){
		$this->fields = 'ID,ParentID,Text,Path,IsFolder,IF(IsFolder,"' . we_base_ContentTypes::FOLDER_COMMON . '","' . we_base_ContentTypes::NAVIGATION . '") AS ContentType';
		parent::__construct($id, NAVIGATION_TABLE, stripslashes($JSIDName), stripslashes($JSTextName), $JSCommand, $order, $FolderText);
		$this->title = g_l('fileselector', '[navigationDirSelector][title]');
		$this->userCanMakeNewFolder = true;
	}

	protected function printHeaderHeadlines(): string{
		return '<th class="selector selectoricon"></th>
		<th class="selector"colspan="2">' . g_l('navigation', '[name]') . '</th>';
	}

	protected function printFooterTable(): string{
		$cancel_button = we_html_button::create_button(we_html_button::CANCEL, "javascript:WE().util.dialogTop(window).exit_close();");
		$yes_button = we_html_button::create_button(we_html_button::OK, "javascript:press_ok_button();");
		return '
<table id="footer" class=" defaultfont">
	<tr>
		<td class="description">' . g_l('navigation', '[name]') . '</td>
		<td style="text-align:left"><div id="showDiv" style="width:100%; height:2.2ex; background-color: #dce6f2; border: #AAAAAA solid 1px;"></div><div style="display:none;">' . we_html_tools::textInput('fname', (string) $this->values['Text'], 0, [
				'style' => "width:100%", 'readonly' => "readonly"]) . '</div></td>
	</tr>
</table><div id="footerButtons">' . we_html_button::position_yes_no_cancel($yes_button, '', $cancel_button) . '</div>';
	}

	protected function printHeaderTable(string $extra = '', bool $append = false): string{
		$makefolderState = we_base_permission::hasPerm([we_base_perm::NAVIGATION_EDIT]);
		$this->jsCmd->addCmd('updateSelectData', [
			'makefolderState' => $makefolderState
		]);
		return parent::printHeaderTable('<td>' . we_html_button::create_button(we_html_button::NEW_DIR, "javascript:if(WE().util.dialogTop(window).fileSelect.data.makefolderState){WE().util.dialogTop(window).drawNewFolder();}", '', "", !$makefolderState) .
				'</td>');
	}

	protected function printCmdAddEntriesHTML(): void{
		$entries = [];
		$this->query();

		foreach($this->query() as $item){
			$entries[] = [
				(int) $item['ID'],
				$item['Text'],
				(int) $item['IsFolder'],
				$item['Path']
			];
		}
		$this->jsCmd->addCmd('addEntries', $entries);
		$this->jsCmd->addCmd('writeBody');
	}

	protected function printCreateFolderHTML(): void{
		$this->FolderText = rawurldecode($this->FolderText);
		$txt = rawurldecode(we_base_request::_(we_base_request::FILE, 'we_FolderText_tmp', ''));

		$this->jsCmd->addCmd('clearEntries');
		$folder = we_folder_base::instance($this->table);
		$folder->we_new($this->table, $this->dir, $txt);

		if(!$txt){
			$this->jsCmd->addMsg(g_l('navigation', '[wrongtext]'), we_base_util::WE_MESSAGE_ERROR);
		} elseif(fB('SELECT 1 FROM ' . $this->table . ' WHERE Path="' . $folder->Path . '"')){
			$this->jsCmd->addMsg(g_l('navigation', '[folder_path_exists]'), we_base_util::WE_MESSAGE_ERROR);
		} elseif(we_navigation_navigation::filenameNotValid($folder->getText())){
			$this->jsCmd->addMsg(g_l('navigation', '[wrongtext]'), we_base_util::WE_MESSAGE_ERROR);
		} else {
			$folder->we_save();
			if($this->canSelectDir){
				$this->jsCmd->addCmd('updateSelectData', [
					'currentPath' => $folder->Path,
					'currentID' => $folder->ID,
					'crrentText' => $folder->getText(),
				]);
			}
			$this->jsCmd->addCmd('makeNewTreeEntry', [
				'id' => $folder->ID,
				'parentid' => $folder->ParentID,
				'text' => $txt,
				'open' => 1,
				'contenttype' => we_base_ContentTypes::FOLDER_COMMON,
				'table' => $this->table,
				'published' => 0,
				'order' => 0
			]);
		}

		$this->printCmdAddEntriesHTML();
		$this->setWriteSelectorData();
		echo we_html_tools::getHtmlTop('', $this->jsCmd->getCmds(), we_html_element::body());
	}

	protected function query(): array{
		return $this->db->getAllq('SELECT ' . $this->fields . ' FROM ' . $this->table . ' WHERE IsFolder=1 AND ParentID=' . intval($this->dir) . ' ' . self::getWsQuery(NAVIGATION_TABLE) . ' ORDER BY Ordn, (text REGEXP "^[0-9]") DESC,ABS(text),Text');
	}

	protected function getFramsetJSFile(): string{
		return parent::getFramsetJSFile() . we_html_element::jsScript(JS_DIR . 'selectors/naviagationDir_selector.js');
	}

	public function printHTML(string $what = we_selector_file::FRAMESET, bool $withPreview = true): void{
		parent::printHTML($what, false);
	}

}
