<?php

declare(strict_types=1);

/**
 * webEdition CMS
 *
 * collection of the navigation items
 *
 * $Rev: fce080e8e7af $
 * $Author: Marc Kraemer <marc.kraemer@webedition.org> $
 * $Date: 2020-12-07 23:42 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 *
 * @license		http://www.gnu.org/copyleft/gpl.html  GPL
 * @version    	SVN: $Id$
 */
class we_navigation_items{

	const TEMPLATE_DEFAULT_CURRENT = 'current';
	const TEMPLATE_DEFAULT_POSITION = 'position';
	const TEMPLATE_DEFAULT_LEVEL = 'level';
	const TYPE_ROOT = 'root';
	const TYPE_FOLDER = 'folder';
	const TYPE_ITEM = 'item';

	/** @var array<int,array<int,we_navigation_item>> */
	private static $cache = [];

	/** @var int */
	public $depth = 0;

	/** @var array<int,we_navigation_item> */
	var $items = [];

	/** @var array<string,array<string,array<string,array<string,string>>>> */
	private $templates = [
		self::TYPE_ROOT => [
			self::TEMPLATE_DEFAULT_LEVEL => [
				self::TEMPLATE_DEFAULT_CURRENT => [
					self::TEMPLATE_DEFAULT_POSITION => '<?= we_weTag_util::tag(\'navigationEntries\'); ?>'
				]]],
		self::TYPE_FOLDER => [
			self::TEMPLATE_DEFAULT_LEVEL => [
				self::TEMPLATE_DEFAULT_CURRENT => [
					self::TEMPLATE_DEFAULT_POSITION => '<li><a href="<?= we_weTag_util::tag(\'navigationField\', [\'name\' => \'href\']); ?>"><?= we_weTag_util::tag(\'navigationField\', [\'name\' => \'text\']); ?></a><?php if(we_weTag_util::tag(\'ifHasEntries\')){ ?><ul><?= we_weTag_util::tag(\'navigationEntries\'); ?></ul><?php } ?></li>'
				]]],
		self::TYPE_ITEM => [
			self::TEMPLATE_DEFAULT_LEVEL => [
				self::TEMPLATE_DEFAULT_CURRENT => [
					self::TEMPLATE_DEFAULT_POSITION => '<li><a href="<?= we_weTag_util::tag(\'navigationField\', [\'name\' => \'href\']); ?>"><?= we_weTag_util::tag(\'navigationField\', [\'name\' => \'text\']); ?></a></li>'
				]]],
	];

	/** @var int */
	private $rootItem = 0;

	/** @var bool */
	private $currentFound = false;

	/** @var we_navigation_rule[] */
	private $currentRules = [];

	/** @var we_navigation_item[] */
	public $weNavigationItemArray = [];

	/** @var string this is set for having different functionnames in case the navigation is included more than once (with different templates) */
	public $funcPrefix = '';

	/** @var array{items:array<int,array{ID:int,ParentID:int,Path:string,Creation_Date:DateTime,Mod_Date:DateTime,Publish_Date:DateTime,CreatorID:int,ModifierID:int,RestrictOwners:bool,Owners:string,OwnersReadOnly:string,Text:string,Display:string,IsFolder:bool,IconID:int,Selection:string,SelectionType:string,DynamicSelection:string,LinkSelection:string,Url:string,UrlID:int,LinkID:int,CurrentOnUrlPar:bool,CurrentOnAnker:bool,FolderID:int,DocTypeID:int,ClassID:int,Categories:string,CatAnd:bool,TitleField:string,Sort:string,ShowCount:int,Ordn:int,Depended:bool,WorkspaceID:int,CatParameter:string,Parameter:string,Attributes:string,UseDocumentFilter:bool,IsPublished:bool}>,ids:array<int,string>} */
	private static $Storage = [
		'items' => [],
		'ids' => [0 => '/'],
	];

	private function initRulesFromDB(): void{
		$this->currentRules = we_navigation_ruleControl::getAllNavigationRules();
		we_navigation_cache::saveRules($this->currentRules);
	}

	private function loopAllRules(): void{
		if($this->currentFound){
			return;
		}
		$this->checkCurrent();
	}

	public function init(int $parentid = 0, bool $showRoot = true): void{
		try{
			$this->initFromCache($parentid, $showRoot);
		}catch(we_exception_cache $e){
			//make sure we use cache next time!
			$this->initById($parentid, $showRoot);
			we_navigation_cache::saveCacheNavigation($parentid, $this->items);
		}
	}

	/**
	 *
	 * @param int $parentid
	 * @param bool $showRoot
	 * @return void
	 * @throws we_exception_cache in case entry is not found
	 */
	private function initFromCache(int $parentid = 0, bool $showRoot = true): void{
		$this->rootItem = $parentid;

		if(isset(self::$cache[$parentid])){
			$this->items = self::$cache[$parentid];
		} else {
			$this->items = we_navigation_cache::getCacheFromFile($parentid);
			self::$cache[$parentid] = $this->items;
		}

		if($this->items['id' . $parentid]){
			$this->items['id' . $parentid]->type = !$showRoot || $parentid == 0 ? self::TYPE_ROOT : self::TYPE_FOLDER;
		}
		try{
			$this->currentRules = we_navigation_cache::getCachedRule();
		}catch(we_exception_cache $e){
			$this->currentRules = [];
			$this->initRulesFromDB();
		}

		foreach($this->items as $item){
			//flag currentFound temporarily deactivated. FIXME: reactivate flag seraching for more currents in same branch of navigation tree
			//$this->currentFound = $this->currentFound || ($item->isCurrent($this));
			$item->isCurrent($this);
		}
		unset($item);
		$this->loopAllRules();
	}

	private function initById(int $parentid = 0, bool $showRoot = true): void{
		$this->items = [];
		$this->rootItem = $parentid;
		$navigation = new we_navigation_navigation();
		self::readItemsFromDb($this->rootItem);
		$poolItem = self::getItemFromPool($parentid) ?: ['ID' => 0, 'Path' => '/'];
		$navigation->initByRawData($poolItem);

		[$table, $linkid] = $navigation->getTableIdForItem();
		$this->items['id' . $navigation->ID] = new we_navigation_item($navigation->ID, $linkid, $table, $navigation->Text, $navigation->Display, $navigation->getHref(), (!$showRoot || $navigation->ID == 0 ? self::TYPE_ROOT : ($navigation->IsFolder ? self::TYPE_FOLDER : self::TYPE_ITEM)), self::id2path($navigation->IconID), $navigation->Attributes, $navigation->customerFilter, $navigation->CurrentOnUrlPar, $navigation->CurrentOnAnker);

		$parents = $this->getDynamicPreview($navigation->ID, true);
//reduce Memory consumption!
		self::$Storage['items'] = [];
		foreach($parents as $id => $parentid){
			if(isset($this->items['id' . $parentid])){
				$this->items['id' . $parentid]->addChild($this->items['id' . $id]);
			}
		}
		foreach($this->items as $item){
			//flag currentFound temporarily deactivated. FIXME: reactivate flag seraching for more currents in same branch of navigation tree
			//$this->currentFound = $this->currentFound || ($this->items['id' . $item['id']]->isCurrent($this));
			$item->isCurrent($this);
		}

		$this->initRulesFromDB();
		$this->loopAllRules();

//make avail in cache
		self::$cache[$parentid] = $this->items;
	}

	/**
	 *
	 * @param int $parentID
	 * @param bool $rules
	 * @return array<int,int> returns itemids with parents
	 */
	private function getDynamicPreview(int $parentID, bool $rules = false): array{
		$parents = [];
		$iconIDs = id2pathA(array_unique(array_column(self::$Storage['items'], 'IconID')), FILE_TABLE, false, true);
		foreach(self::$Storage['items'] as $item){
			if($item['ParentID'] != $parentID){
				continue;
			}
			$nav = new we_navigation_navigation(0, DB_WE::$inst);
			$nav->initByRawData($item);

			[$table, $linkid] = $nav->getTableIdForItem();
			if($item['IsFolder'] || $item['Selection'] !== we_navigation_navigation::SELECTION_DYNAMIC){
				$this->items['id' . $item['ID']] = new we_navigation_item($item['ID'], (int) $linkid, (string) $table, ($item['Display'] ?: $item['Text']), ($item['Display'] ?: ''), $nav->getHref(), $item['IsFolder'] ? we_navigation_items::TYPE_FOLDER : we_navigation_items::TYPE_ITEM, ((string) empty($iconIDs[$item['IconID']]) ? '' : $iconIDs[$item['IconID']]), (is_array($item['Attributes']) ? $item['Attributes'] : ($item['Attributes'] ? we_unserialize($item['Attributes']) : [])), $nav->customerFilter, (bool) $item['CurrentOnUrlPar'], (bool) $item['CurrentOnAnker'], $item['SelectionType'] === we_navigation_navigation::STYPE_CATLINK, $item['CatParameter']);
				//'depended' => (bool) $item['Depended']
				$parents[$item['ID']] = $item['ParentID'];
			}

			if($item['IsFolder']){
				//keep index!
				$parents += $this->getDynamicPreview($item['ID'], $rules);
			} elseif($item['Selection'] === we_navigation_navigation::SELECTION_DYNAMIC){
				$dyn_items = $nav->getDynamicEntries();
				foreach($dyn_items as $dyn){
					$href = $nav->getHref($dyn['id']);
					$id = (-100 * $item['ID']) + $dyn['id'];
					$this->items['id' . $id] = new we_navigation_item($id, (int) $dyn['id'], (($item['DynamicSelection'] === we_navigation_navigation::DYN_CLASS || $item['SelectionType'] === we_navigation_navigation::STYPE_OBJLINK) ? OBJECT_FILES_TABLE : FILE_TABLE), $dyn['field'] ?: $dyn['text'], $dyn['display'] ?? '', $href, we_navigation_items::TYPE_ITEM, we_navigation_items::id2path($item['IconID']), (is_array($item['Attributes']) ? $item['Attributes'] : we_unserialize($item['Attributes'])), $nav->customerFilter, $item['CurrentOnUrlPar'], $item['CurrentOnUrlPar'], false, '');
					$parents[$id] = $item['ParentID'];

					if($rules){
						//FIXME: $cats should already be an array
						$cats = $item['Categories'] ? we_base_request::filterIntList($item['Categories']) : [];
						$this->currentRules[] = we_navigation_rule::getWeNavigationRule('defined_' . ($dyn['field'] ?: $dyn['text']), (int) $item['ID'], (string) $item['SelectionType'], (int) $item['FolderID'], (int) $item['DocTypeID'], (int) $item['ClassID'], $cats, (int) $item['WorkspaceID'], $href, false);
					}
				}
			}
		}

		return $parents;
	}

	/**
	 *
	 * @param int[] $idsRule
	 * @param int[] $idDoc
	 * @return bool
	 */
	private static function checkCategories(array $idsRule, array $idDoc): bool{
		if(!$idsRule){
			return true;
		}
		$diff = array_intersect($idDoc, $idsRule);
		return !empty($diff);
	}

	function setCurrent(int $navigationID): void{
		if(isset($this->items['id' . $navigationID])){
			$this->items['id' . $navigationID]->setCurrent($this);
		}
	}

	private function checkCurrent(): bool{
		if(!we_base_WE::$mainDoc['doc']){
			return false;
		}

		$candidate = $pathLen = 0;
		$score = 3;
		$mainCats = we_base_WE::$mainDoc['doc']->Category;
		try{
			$obj = we_base_WE::getWeObj();
			$isObject = true;
			$currentWorkspace = (string) (defined('WE_REDIRECTED_SEO') ? //webEdition object uses SEO-URL
					(substr(WE_REDIRECTED_SEO, 0, strripos(WE_REDIRECTED_SEO, $obj->Url) ?: 0) ?: '') : //@phan-suppress-current-line PhanParamSuspiciousOrder
					(parse_url(urldecode($_SERVER['REQUEST_URI']), PHP_URL_PATH) ?: '')
					);
		}catch(we_exception_base $ex){
			$isObject = false;
			$currentWorkspace = we_base_WE::$mainDoc['doc']->Path;
		}



		foreach($this->currentRules as $rule){
			$ponder = 4;
			$rulePath = '';

			if(($cats = $rule->Categories)){
				if(!self::checkCategories($cats, $mainCats)){
					continue; // remove from selection
				}
				$ponder--;
			}

			switch($rule->SelectionType){
				case we_navigation_navigation::DYN_DOCTYPE:
					if($isObject){
						continue 2; // remove from selection
					}
					if($rule->DoctypeID){
						if(!we_base_WE::$mainDoc['doc']->DocType || ($rule->DoctypeID != we_base_WE::$mainDoc['doc']->DocType)){
							continue 2;
						}
						$ponder--;
					}

					if($rule->FolderID){
						$rulePath = rtrim(self::id2path($rule->FolderID), '/') . '/';
					}
					break;
				case we_navigation_navigation::DYN_CLASS:
					if(!$isObject){
						continue 2; // remove from selection
					}
					if($rule->ClassID){
						if($obj->TableID != $rule->ClassID){
							continue 2; // remove from selection
						}
						$ponder--;
					}

					if($rule->WorkspaceID){
						$rulePath = rtrim(self::id2path($rule->WorkspaceID), '/') . '/';
					}
					break;
			}

			if(!empty($rulePath) && str_contains($currentWorkspace, $rulePath)){
				$ponder--;
				if(($currPathLen = strlen($rulePath)) >= $pathLen){ //the longest path wins
					if($pathLen > 0){//no ponder for first match
						$ponder--;
					}
					$pathLen = $currPathLen;
				}
			}

			if($ponder <= $score){
				if(NAVIGATION_RULES_CONTINUE_AFTER_FIRST_MATCH){
					$this->setCurrent($rule->NavigationID);
				} else {
					$score = $ponder;
					$candidate = $rule->NavigationID;
				}
			}
		}

		if($candidate){
			$this->setCurrent($candidate);
			return true;
		}

		return false;
	}

	/**
	 * @param int $id
	 * @return int[]
	 */
	function getItemIds(int $id): array{
		$items = [$id];

		foreach($this->items[$id]->items as $key => $val){
			if($val->type == self::TYPE_FOLDER){
				$items = array_merge($items, $this->getItemIds((int) $key));
			} else {
				$items[] = $key;
			}
		}

		return $items;
	}

	/**
	 * @param int $id
	 * @return int[]
	 */
	function getItems(int $id = 0): array{
		return ($id ?
				$this->getItemIds($id) :
				array_keys($this->items)); //@phan-suppress-current-line PhanPartialTypeMismatchReturn
	}

	function getItem(string $id): ?we_navigation_item{
		return $this->items[$id] ?? null;
	}

	/**
	 * @param we_navigation_item $item
	 * @return mixed
	 */
	function getTemplate(we_navigation_item $item): string{
		$currentPos = we_navigation_item::$currentPosition[$item->level];
// get correct Level
		$level = (isset($this->templates[$item->type][$item->level]) ? $item->level : self::TEMPLATE_DEFAULT_LEVEL);
		$useTemplate = $this->templates[$item->type][$level];
		$fkt = $item->type . '_' . $level;
// use current true/false
		$cur = ($item->current ? '1' : '0');
		if(isset($useTemplate[$cur])){
			$useTemplate = $useTemplate[$cur];
			$fkt .= '_' . $cur;
		} elseif(isset($useTemplate[self::TEMPLATE_DEFAULT_CURRENT])){
			$useTemplate = $useTemplate[self::TEMPLATE_DEFAULT_CURRENT];
			$fkt .= '_' . self::TEMPLATE_DEFAULT_CURRENT;
		} else {
			return self::getDefaultTemplate($item->type);
		}
//get position
// is last entry??
		if(isset($useTemplate['last']) &&
// check if item is last
				($this->items['id' . $item->getParentId()]->getLastVisibleItemID() == $item->id)){
			return $fkt . '_last';
		}

		if(isset($useTemplate[$currentPos])){
			return $fkt . '_' . $currentPos;
		}

		if(isset($useTemplate['odd']) && $currentPos % 2 === 1){
			return $fkt . '_odd';
		}

		if(isset($useTemplate['even']) && $currentPos % 2 === 0){
			return $fkt . '_even';
		}

		if(isset($useTemplate[self::TEMPLATE_DEFAULT_POSITION])){
			return $fkt . '_' . self::TEMPLATE_DEFAULT_POSITION;
		}

		return self::getDefaultTemplate($item->type);
	}

	private static function getDefaultTemplate(string $type): string{
		return $type . '_' . self::TEMPLATE_DEFAULT_LEVEL . '_' . self::TEMPLATE_DEFAULT_CURRENT . '_' . self::TEMPLATE_DEFAULT_POSITION;
	}

	/**
	 *
	 * @param array<string,array> $templates
	 * @param string $prefix
	 * @return string
	 */
	private static function getTemplateCode(array $templates, string $prefix = ''): string{
		$ret = '';
		foreach($templates as $key => $cur){
			$key = ($prefix ? $prefix . '_' : '') . $key;
			if(is_array($cur)){
				$ret .= self::getTemplateCode($cur, $key);
				continue;
			}
			$ret .= '
public static function ' . $key . '():void{
	?>' . $cur . '<?php
}
';
		}
		return $ret;
	}

	public function writeNavigation(int $depth = 0): string{
		$this->depth = $depth;
		$this->funcPrefix = 'nav_' . sprintf('%u', crc32(serialize($this->templates)));
		$filename = $this->funcPrefix . '.php';
		$file = TEMP_PATH . $filename;
		if(!file_exists($file)){
			$code = self::getTemplateCode($this->templates);
			$file = we_base_file::saveTemp('<?php class ' . $this->funcPrefix . '{' . $code . '}', $filename, 24 * 3600);
		}
		include_once($file);
		if(!empty($this->items['id' . $this->rootItem])){
			if($this->items['id' . $this->rootItem]->type === self::TYPE_FOLDER && $depth !== 0){
// if initialised by id => root item is on lvl0 -> therefore decrease depth
// this is to make it equal init by id, parentid
				$depth--;
			}
			we_navigation_item::$currentPosition = [];
			ob_start();
			$this->items['id' . $this->rootItem]->writeItem($this, $depth);
			return ob_get_clean() ?: '';
		}

		return '';
	}

	public function setTemplate(string $content, string $type, string $level, string $current, string $position): void{
		$pos = ($position === 'first' ? 1 : $position);
		$this->templates[$type][$level][$current][$pos] = $content;
	}

	private static function readItemsFromDb(int $id): void{
		$db = DB_WE::$inst;
		$ids = [];

		self::$Storage['items'] = $db->getAllFirstq('SELECT n.ID AS i,n.* FROM ' . NAVIGATION_TABLE . ' n ' . ($id ? ' WHERE n.Path LIKE(SELECT CONCAT(nn.Path,"/%") FROM ' . NAVIGATION_TABLE . ' nn WHERE nn.ID=' . $id . ') OR n.ID=' . $id : '') . ' ORDER BY n.Ordn', true, MYSQLI_ASSOC);

		$allFilter = (we_modules_info::isActive(we_modules_info::CUSTOMER) && self::$Storage['items'] ? [
			'nav' => $db->getAllFirstq('SELECT
	f.modelId AS `id`,
	f.*
FROM ' .
					CUSTOMER_FILTER_TABLE . ' f
WHERE
	modelId IN (' . implode(', ', array_keys(self::$Storage['items'])) . ') AND
	modelTable="' . DB_WE::$inst->escape(stripTblPrefix(NAVIGATION_TABLE)) . '"', true, MYSQLI_ASSOC),
			'doc' => $db->getAllFirstq('SELECT
	n.ID AS `id`,
	f.*
FROM ' .
					CUSTOMER_FILTER_TABLE . ' f JOIN ' .
					NAVIGATION_TABLE . ' n ON f.modelId=n.LinkID
WHERE
	n.ID IN (' . implode(', ', array_keys(self::$Storage['items'])) . ') AND
	n.SelectionType="' . we_navigation_navigation::STYPE_DOCLINK . '" AND
	n.UseDocumentFilter=1 AND
	f.modelTable="' . DB_WE::$inst->escape(stripTblPrefix(FILE_TABLE)) . '"', true, MYSQLI_ASSOC),
			'obj' => $db->getAllFirstq('SELECT
	n.ID AS `id`,
	f.*
FROM ' .
					CUSTOMER_FILTER_TABLE . ' f JOIN ' .
					NAVIGATION_TABLE . ' n ON f.modelId=n.LinkID
WHERE
	n.ID IN (' . implode(', ', array_keys(self::$Storage['items'])) . ') AND
	n.SelectionType="' . we_navigation_navigation::STYPE_OBJLINK . '" AND
	n.UseDocumentFilter=1 AND
	f.modelTable="' . DB_WE::$inst->escape(stripTblPrefix(OBJECT_FILES_TABLE)) . '"', true, MYSQLI_ASSOC)
				] : [
			'nav' => [],
			'doc' => [],
			'obj' => [],
		]);
		'@phan-var array<string,array<int,array>> $allFilter';
		foreach(self::$Storage['items'] as $id => &$tmpItem){
			assert(is_array($tmpItem));
			foreach(we_navigation_navigation::PERSISTENTS as $name => $type){
				if(!isset($tmpItem[$name])){
					continue;
				}
				$tmpItem[$name] = we_base_request::filterVar($tmpItem[$name], $type); //@phan-suppress-current-line PhanDeprecatedFunction
			}
			$tmpItem['Name'] = $tmpItem['Text'];

			$tmpItem['customerFilter'] = (isset($allFilter['nav'][$id]) ?
					we_navigation_customerFilter::getFilterByDbHash($allFilter['nav'][$id]) :
					(isset($allFilter['doc'][$id]) ?
					we_navigation_customerFilter::getFilterByDbHash($allFilter['doc'][$id]) :
					(isset($allFilter['obj'][$id]) ?
					we_navigation_customerFilter::getFilterByDbHash($allFilter['obj'][$id]) :
					null
					)));

			switch($tmpItem['SelectionType']){
				case we_navigation_navigation::STYPE_DOCLINK:
					if($tmpItem['Selection'] === we_navigation_navigation::SELECTION_STATIC){
						$ids[] = $tmpItem['LinkID'];
					}
					break;
				case we_navigation_navigation::STYPE_CATLINK:
					if($tmpItem['LinkSelection'] !== we_navigation_navigation::LSELECTION_EXTERN){
						$ids[] = $tmpItem['UrlID'];
					}
					break;
			}

			if($tmpItem['DynamicSelection'] === we_navigation_navigation::DYN_CATEGORY && $tmpItem['LinkSelection'] !== we_navigation_navigation::LSELECTION_EXTERN){
				$ids[] = $tmpItem['UrlID'];
			}

			if($tmpItem['IconID']){
				$ids[] = $tmpItem['IconID'];
			}
		}
		$allids = $ids ? array_diff(array_unique($ids), array_keys(self::$Storage['ids'])) : [];
		if($allids){
			//keep array index
			self::$Storage['ids'] += $db->getAllFirstq('SELECT f.ID,IF(f.IsPublished=1,f.Path,"") FROM ' . FILE_TABLE . ' f WHERE f.ID IN(' . implode(',', $allids) . ') ORDER BY f.ID', false);
		}
	}

	/**
	 *
	 * @param int $id
	 * @return ?array{ID:int,Path:string}
	 */
	private static function getItemFromPool(int $id): ?array{
		return (self::$Storage['items'][$id]) ?? null;
	}

	public static function id2path(int $id): string{
		if(isset(self::$Storage['ids'][$id])){
			return self::$Storage['ids'][$id];
		}
		if(!$id){
			return '/';
		}
		return (self::$Storage['ids'][$id] = id2path($id, FILE_TABLE));
	}

}
