<?php
declare(strict_types=1);

/* webEdition CMS
 *
 * $Rev: c564388e074d $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-07-17 13:01 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */

abstract class we_tag_ifShopVat extends we_weTag_ifTag{

	/**
	 * @param array<string,mixed> $attribs parsed tag-attributes
	 * @param string $content content between openingtag and closing tag @unused-param
	 * @param bool $internal if used by we internally @unused-param
	 * @return string output to print
	 * @throws we_exception_parser if any attribute is missing
	 */
	public static function tag(array $attribs = [], string $content = '', bool $internal = false): bool{
		if(!we_shop_category::isCategoryMode()){
			self::attributeMissing($attribs, ['id' => false], __CLASS__);
			self::$doc = we_base_WE::getWeDoc();

			$id = self::getAttribute('id', $attribs, -1, we_base_request::INT);

			$vatId = (we_tag_listview::$current && we_tag_listview::$current->f(we_shop_shop::VAT_FIELD_NAME) ?
				we_tag_listview::$current->f(we_shop_shop::VAT_FIELD_NAME) :
				self::$doc->getElementI(we_shop_shop::VAT_FIELD_NAME));


			if(!$vatId){
				try{
					$shopVat = we_shop_vats::getStandardShopVat();
					$vatId = $shopVat->id;
				} catch (Throwable $e){

				}
			}

			return ($id == $vatId);
		} else {
			$match = intval(self::getAttribute('match', $attribs, 0, we_base_request::INT));
			$field = self::getAttribute('field', $attribs, 'id', we_base_request::STRING);
			$lvOnly = self::getAttribute('lvOnly', $attribs, false, we_base_request::BOOL);

			if(we_tag_listview::$current){
				$catID = we_tag_listview::$current->f(we_shop_shop::CATEGORY_FIELD_NAME) ?: 0;
				$wedocCategory = we_tag_listview::$current->f(we_listview_base::CATEGORY_FIELD) ?: '';
			} elseif(!$lvOnly){
				$catID = self::$doc->getElementI(we_shop_shop::CATEGORY_FIELD_NAME);
				$wedocCategory = self::$doc->Category ?: '';
			} else {
				return false;
			}
			$attribs['shopcategoryid'] = $catID;
			$attribs['wedoccategories'] = $wedocCategory;
			$vatField = we_weTag_util::tag('shopVat', $attribs);

			switch($field){
				case 'id':
					return $match === 0 ? ((bool) $vatField) : $match === intval($vatField);
				default:
					return (bool) ($vatField);
			}
		}
	}

}
