<?php
declare(strict_types=1);

/* webEdition CMS
 *
 * $Rev: c564388e074d $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-07-17 13:01 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */

abstract class we_tag_img extends we_weTag_tag{

	/**
	 * @param array<string,mixed> $attribs parsed tag-attributes
	 * @param string $content content between openingtag and closing tag @unused-param
	 * @param bool $internal if used by we internally @unused-param
	 * @return string output to print
	 * @throws we_exception_parser if any attribute is missing
	 */
	public static function tag(array $attribs = [], string $content = '', bool $internal = false): string{
		if(!($tagId = self::getAttribute('id', $attribs, 0, we_base_request::INT))){
			self::attributeMissing($attribs, ['name' => false], __CLASS__);
		}
		self::$doc = we_base_WE::getWeDoc();

		$name = self::getAttribute('name', $attribs, '', we_base_request::STRING);
		$showcontrol = self::getAttribute('showcontrol', $attribs, true, we_base_request::BOOL);
		$showThumb = (bool) self::getAttribute('showthumbcontrol', $attribs, false, we_base_request::BOOL);
		$showimage = self::getAttribute('showimage', $attribs, true, we_base_request::BOOL);
		$showinputs = self::getAttribute('showinputs', $attribs, SHOWINPUTS_DEFAULT, we_base_request::BOOL);

		$tagAttribs = self::removeAttribs($attribs, ['id', 'showcontrol', 'showthumbcontrol', 'showimage', 'showinputs', 'startid', 'parentid']);
		$tagAttribs['thumbnail'] = self::getAttribute('thumbnail', $attribs, '', we_base_request::STRING);
		$tagAttribs['thumbnailid'] = self::getAttribute('thumbnailid', $attribs, 0, we_base_request::INT);

		if($name){
			$id = (int) ((self::$doc->getElementI($name) ?:
				self::$doc->getElementS($name)) ?:
				$tagId);
		} else {
			$showThumb = $showcontrol = false;
			$id = (int) $tagId;
		}

		//look if image exists in tblfile, and is an image
		if(!fB('SELECT 1 FROM ' . FILE_TABLE . ' WHERE ContentType="' . we_base_ContentTypes::IMAGE . '" AND ID=' . $id)){
			$id = 0;
		}

		$altField = $name . we_document_image::ALT_FIELD;
		$titleField = $name . we_document_image::TITLE_FIELD;
		$thumbField = $name . we_document_image::THUMB_FIELD;

		$fname = 'we_' . self::$doc->Name . '_img[' . $name . '#' . we_contents_base::ETYPE_NUMBER . ']';

		if($id){
			$img = new we_document_image($id);

			$tagAttribs['alt'] = self::getAttribute('alt', $attribs, $img->getElementS('alt'), we_base_request::STRING);
			$tagAttribs['title'] = self::getAttribute('title', $attribs, $img->getElementS('title'), we_base_request::STRING);
		}
		$thumbattr = 0;
		// images can now have custom attribs
		if(!we_base_WE::$editmode || !(we_base_request::string('we_cmd', '', 0) === 'reload_editpage' &&
			($name === we_base_request::string('we_cmd', '', 1)) &&
			we_base_request::string('we_cmd', '', 2) === 'change_image') &&
			self::$doc->issetElement($altField)){ // if no other image is selected.
			$alt = self::$doc->getElementS($altField);
			$title = self::$doc->getElementS($titleField);
			$tagAttribs['alt'] = $alt ?: (empty($tagAttribs['alt']) ? '' : $tagAttribs['alt']);
			$tagAttribs['title'] = $title ?: (empty($tagAttribs['title']) ? '' : $tagAttribs['title'] );
		} elseif(self::$doc){
			$alt = self::$doc->getElementS($altField);
			$title = self::$doc->getElementS($titleField);
			if(we_base_request::string('we_cmd', '', 2) === 'change_image'){
				//in case of changed images give them priority to already set text
				$tagAttribs['alt'] = (empty($tagAttribs['alt']) ? $alt : $tagAttribs['alt'] );
				$tagAttribs['title'] = (empty($tagAttribs['title']) ? $title : $tagAttribs['title'] );
			} else {
				$tagAttribs['alt'] = $alt ?: (empty($tagAttribs['alt']) ? '' : $tagAttribs['alt']);
				$tagAttribs['title'] = $title ?: (empty($tagAttribs['title']) ? '' : $tagAttribs['title'] );
			}
		}
		if($showThumb && self::$doc){
			$thumbattr = self::$doc->getElementI($thumbField);
			$tagAttribs['thumbnailid'] = $thumbattr;
		}

		if(we_base_WE::$editmode && !$showimage){
			$out = '';
		} elseif($id){
			if($tagId){//bug 6433: später wird so ohne weiteres gar nicht mehr auf die id zurückgegriffen
				$tagAttribs['id'] = $tagId; //siehe korrespondierende Änderung in we_document::getField
				$tagAttribs['showcontrol'] = $showcontrol; //sicherstellen das es boolean iost
			}
			$out = (string) self::$doc->getField($tagAttribs, 'img');
		} elseif(we_base_WE::$editmode && self::$doc->InWebEdition){
			$out = we_html_element::div(['class' => 'tag_img_icon largeicons grayicons',
					'data-contenttype' => we_base_ContentTypes::IMAGE]);
		} else {
			$out = '';
		}

		if(we_base_WE::$editmode && $out && $showcontrol){ //in editMode we surround image with dropzone
			$cmdTree = $cmdExt = 'tag_weimg_insertImage,' . $name . ',' . $fname . ',' . self::$doc->ID . ',' . (we_base_WE::$transaction ?? '') . ',' . self::$doc->Table . ',' . self::$doc->EditPageNr;
			$out = we_fileupload_ui_base::getExternalDropZone(self::$doc->Name . '_' . $name, $out, 'width:auto;height:auto;padding:0;', true, true, $cmdTree, $cmdExt);
		}

		if(!$id && (!we_base_WE::$editmode)){
			return '';
		}

		return ($showcontrol && we_base_WE::$editmode ?
			self::edit($attribs, $out, $id, $showinputs, $name, $tagAttribs, $showThumb, $thumbattr) :
			$out
			);
	}

	public static function edit(array $attribs, string $out, int $id, bool $showinputs, string $name, array $tagAttribs, bool $showThumb, int $thumbattr): string{
		$parentid = self::getAttribute('parentid', $attribs, 0, we_base_request::INT);
		$startid = self::getAttribute('startid', $attribs, 0, we_base_request::INT);

		$altField = $name . we_document_image::ALT_FIELD;
		$titleField = $name . we_document_image::TITLE_FIELD;
		$thumbField = $name . we_document_image::THUMB_FIELD;

		$fname = 'we_' . self::$doc->Name . '_img[' . $name . '#' . we_contents_base::ETYPE_NUMBER . ']';
		$out = '
<table class="we we_img weEditTable">
	<tr><td style="text-align:center">' . $out . '<input onchange="_EditorFrame.setEditorIsHot(true);" type="hidden" name="' . $fname . '" value="' . $id . '" /></td></tr>' .
			($showinputs ? //  only when wanted
			'
	<tr><td>
		<table class="weEditTable">
			<tr>
				<td>' . g_l('weClass', '[alt_kurz]') . ':&nbsp;</td>
				<td>' . we_html_tools::textInput('we_' . self::$doc->Name . '_txt[' . $altField . ']', $tagAttribs['alt'], 0, ['onchange' => "_EditorFrame.setEditorIsHot(true);"]) . '</td>
			</tr>
			<tr>
				<td>' . g_l('weClass', '[Title]') . ':&nbsp;</td>
				<td>' . we_html_tools::textInput('we_' . self::$doc->Name . '_txt[' . $titleField . ']', $tagAttribs['title'], 0, ['onchange' => "_EditorFrame.setEditorIsHot(true);"]) . '</td>
			</tr>
		</table></td></tr>' : ''
			);

		if($showThumb){ //  only when wanted
			$db = DB_WE::$inst;
			$thumbs = $db->getAllq('SELECT ID,Text FROM ' . THUMBNAILS_TABLE . ' ORDER BY Text');
			if($thumbs){
				$thumbnails = '<select name="we_' . self::$doc->Name . '_thumb[' . $thumbField . '#' . we_contents_base::ETYPE_NUMBER . ']" onchange="top.we_cmd(\'reload_editpage\'); _EditorFrame.setEditorIsHot(true);">' .
					'<option value=""' . (!$thumbattr ? ' selected="selected"' : '') . '></option>';
				foreach($thumbs as $rec){
					$thumbnails .= '<option value="' . $rec['ID'] . '"' . ($thumbattr == $rec['ID'] ? ' selected="selected"' : '') . '>' . $rec['Text'] . '</option>';
				}
				$thumbnails .= '</select>';
				$out .= '
	<tr><td>
		<table class="weEditTable">
			<tr>
				<td style="color: black; font-size: 12px;">' . g_l('weClass', '[thumbnails]') . ':&nbsp;</td>
				<td>' . $thumbnails . '</td>
			</tr>
		</table>
	</td></tr>';
			}
		}

		return $out . '
	<tr><td style="text-align:center">' .
			($id ?
			//	show edit_image_button
			//	we use hardcoded Content-Type - because it must be an image -> <we:img  >
			we_html_button::create_button(we_html_button::OPEN_TO_EDIT, "javascript:WE().layout.weEditorFrameController.openDocument(WE().consts.tables.FILE_TABLE,$id,WE().consts.contentTypes.IMAGE)") :
			// disable edit_image_button
			we_html_button::create_button(we_html_button::OPEN_TO_EDIT, "#", '', "", true)
			) .
			we_html_button::create_button(we_html_button::IMAGE_SELECT, "javascript:we_cmd('we_selector_image', {id:'" . ($id ?: $startid) . "', table:WE().consts.tables.FILE_TABLE,targetID:'" . $fname . "',jsCmd:'" . 'tag_weimg_insertImage,' . $name . ',' . $fname . ',' . self::$doc->ID . ',' . (we_base_WE::$transaction ?? '') . ',' . self::$doc->Table . ',' . self::$doc->EditPageNr . "',rootDirID:" . $parentid . ",open_doc:" . (we_base_permission::hasPerm([
					we_base_perm::WORK_WORKSPACE_CAN_SELECT_OTHER_FILES]) ? 0 : 1) . "})") .
			we_html_button::create_button(we_html_button::TRASH, "javascript:we_cmd('remove_image', '" . $name . "')") .
			'</td></tr></table>';
	}

}
