<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 86857b889a8e $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-12-17 11:02 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_versions_preview{
	/** @var we_database_base */
	private $db;

	/** @var array<string,mixed> */
	private $document = [];

	/** @var array{ID:int,documentID:int,documentTable:string,documentElements:string,documentScheduler:string,documentCustomFilter:string,logDate:string,status:string,version:int,binaryPath:string,modifiedFields:string,modifierID:int,ContentType:string,ParentID:int,Creation_Date:string,CreatorID:int,Path:string,TemplateID:int,Filename:string,Extension:string,IsDynamic:int,IsSearchable:int,DocType:int,Category:string,RestrictOwners:int,Owners:string,OwnersReadOnly:string,Language:string,WebUserID:int,Workspaces:string,Templates:string,MasterTemplateID:int,TableID:int,IsClassFolder:int,active:int,src:string,resetFromVersion:int,InGlossar:int,_logDate:string} */
	private $newDoc;

	/** @var array{ID:int,documentID:int,documentTable:string,documentElements:string,documentScheduler:string,documentCustomFilter:string,logDate:string,status:string,version:int,binaryPath:string,modifiedFields:string,modifierID:int,ContentType:string,ParentID:int,Creation_Date:string,CreatorID:int,Path:string,TemplateID:int,Filename:string,Extension:string,IsDynamic:int,IsSearchable:int,DocType:int,Category:string,RestrictOwners:int,Owners:string,OwnersReadOnly:string,Language:string,WebUserID:int,Workspaces:string,Templates:string,MasterTemplateID:int,TableID:int,IsClassFolder:int,active:int,src:string,resetFromVersion:int,InGlossar:int,_logDate:string} */
	private $oldDoc;

	/** @var int */
	private $compareID = 0;

	/** @var bool */
	private $isObj = false;

	/** @var bool */
	private $isTempl = false;

	/** @var string */
	private $contentDiff = '';

	/** @var string */
	private $contentNew = '';

	/** @var string */
	private $contentOld = '';

	//const NOTMARK = ['timestamp', 'version'];
	const DO_NOT_SHOW = [
		'ID',
		'documentElements',
		'documentScheduler',
		'documentCustomFilter',
		'documentTable',
		'binaryPath',
		'ContentType',
		'modifiedFields',
		'IP',
		'Browser',
		'Creation_Date',
		'Path',
		'TableID',
		'IsClassFolder',
	];
	const TAB_DIFF = 1;
	const TAB_NEW = 2;
	const TAB_OLD = 3;

	public function __construct(){
		$this->db = DB_WE::$inst;
		$this->document = [
			'table' => we_base_request::_(we_base_request::TABLE, 'we_cmd', 0, 1),
			'ID' => we_base_request::int('we_cmd', 0, 2),
			'version' => we_base_request::int('we_cmd', 0, 3),
		];

		$this->newDoc = we_versions_version::loadVersion(' WHERE documentTable="' . stripTblPrefix($this->document['table']) . '" AND documentID=' . $this->document['ID'] . ' AND version=' . $this->document['version']);

		$this->compareID = we_base_request::int('we_cmd', 0, 4);
		$this->oldDoc = we_versions_version::loadVersion(
				' WHERE documentTable="' . $this->db->escape(stripTblPrefix($this->newDoc['documentTable'])) . '" AND documentID=' . intval($this->newDoc['documentID']) .
				($this->compareID ?
				' AND version=' . $this->compareID :
				' AND version<' . intval($this->newDoc['version']) . ' ORDER BY version DESC LIMIT 1'));


		switch($this->newDoc['ContentType']){
			case we_base_ContentTypes::TEMPLATE:
				$this->isTempl = true;
				break;
			case we_base_ContentTypes::OBJECT_FILE:
				$this->isObj = true;
				break;
		}
		$this->contentDiff = $this->getPropChanges() .
			$this->getElementsDiff() .
			$this->getChangesScheduler() .
			$this->getChangesCustomerFilter();
	}

	private function getTabsBody(): string{
		$we_tabs = new we_gui_tabs();
		$we_tabs->addTab(g_l('versions', '[versionDiffs]'), '', false, (string) (self::TAB_DIFF), ['id' => 'tab_' . self::TAB_DIFF]);
		if(!$this->isObj){
			$we_tabs->addTab(g_l('versions', '[previewVersionNew]'), '', false, (string) (self::TAB_NEW), ['id' => 'tab_' . self::TAB_NEW]);
		}
		if(!empty($this->oldDoc) && !$this->isObj){
			$we_tabs->addTab(g_l('versions', '[previewVersionOld]'), '', false, (string) (self::TAB_OLD), ['id' => 'tab_' . self::TAB_OLD]);
		}

		return $we_tabs->getHTML();
	}

	private function getChangesCustomerFilter(): string{
		$contentDiff = '<table class="propDiff"><thead>
<tr>
	<td colspan="3" class="defaultfont bold">' . g_l('versions', '[customerMod]') . '</td>
</tr></thead>';

		if($this->newDoc['documentCustomFilter']){
			$newCustomFilter = we_unserialize($this->newDoc['documentCustomFilter']);
		} else {
			$newCustomFilter = [];
		}
		'@phan-var array<string,mixed> $newCustomFilter';
		if(isset($this->oldDoc['documentCustomFilter'])){
			if($this->oldDoc['documentCustomFilter']){
				$oldCustomFilter = we_unserialize($this->oldDoc['documentCustomFilter']);
			} else {
				$oldCustomFilter = [];
			}
			'@phan-var array<string,mixed> $oldCustomFilter';
		}

		$mark = '';

		if(empty($newCustomFilter) && empty($oldCustomFilter)){
			$contentDiff .= '<tr><td>-</td></tr>';
		} elseif(empty($newCustomFilter) && !empty($oldCustomFilter)){

			foreach($oldCustomFilter as $key => $val){
				$name = g_l('versions', '[' . $key . ']');
				$newVal = '';
				if(!is_array($val)){
					$oldVal = we_versions_version::showValueCustomerFilter($key, $val);
				} else {
					$oldVal = (is_array($val) ?
						we_versions_version::showValueCustomerFilter($key, $val) :
						'');
				}

				$contentDiff .= '<tr>
	<td style="' . $mark . '"><strong>' . $name . '</strong></td>
	<td style="' . $mark . 'border-right:1px solid #000;">' . $newVal . '</td>' .
					(empty($this->oldDoc) ? '' : '<td style="' . $mark . '">' . $oldVal . '</td>') . '
</tr>';
			}
		} else {
			foreach($newCustomFilter as $key => $val){

				$name = g_l('versions', '[' . $key . ']');

				$mark = false;

				if(!is_array($val)){
					$newVal = we_versions_version::showValueCustomerFilter($key, $val);
					if(!empty($oldCustomFilter)){
						$oldVal = (!is_array($oldCustomFilter[$key]) ?
							we_versions_version::showValueCustomerFilter($key, $oldCustomFilter[$key]) :
							'');

						if($newVal != $oldVal){
							$mark = true;
						}
					} else {
						$oldVal = '';
					}
				} else {
					$newVal = we_versions_version::showValueCustomerFilter($key, $val);
					if(!empty($oldCustomFilter)){
						$oldVal = (isset($oldCustomFilter[$key]) && is_array($oldCustomFilter[$key]) ?
							we_versions_version::showValueCustomerFilter($key, $oldCustomFilter[$key]) :
							'');

						if($newVal != $oldVal){
							$mark = true;
						}
					} else {
						$oldVal = '';
					}
				}

				$contentDiff .= '<tr' . ($mark ? ' class="changedElement"' : '') . '>
	<td><strong>' . $name . '</strong></td>
	<td>' . $newVal . '</td>' .
					(empty($this->oldDoc) ? '' : '<td>' . $oldVal . '</td>') . '
</tr>';
			}
		}

		return $contentDiff . '</table>';
	}

	private function getChangesScheduler(): string{
		//scheduler
		$contentDiff = '<table class="propDiff"><thead>
<tr>
	<td colspan="3" class="defaultfont bold">' . g_l('versions', '[schedulerMod]') . '</td>
</tr></thead>';

		if($this->newDoc['documentScheduler']){
			$newDocScheduler = $this->newDoc['documentScheduler'] ? we_unserialize($this->newDoc['documentScheduler']) : [];
		} else {
			$newDocScheduler = [];
		}
		if(isset($this->oldDoc['documentScheduler'])){
			if($this->oldDoc['documentScheduler']){
				$oldDocScheduler = $this->oldDoc['documentScheduler'] ? we_unserialize($this->oldDoc['documentScheduler']) : [];
			} else {
				$oldDocScheduler = [];
			}
		}

		if(empty($newDocScheduler) && empty($oldDocScheduler)){
			$contentDiff .= '<tr><td>-</td></tr>';
		} elseif(empty($newDocScheduler) && !empty($oldDocScheduler)){

			foreach($oldDocScheduler as $k => $v){
				$number = $k + 1;
				$contentDiff .= '<tr>
	<td style="background-color:#FFF; "><strong>' . g_l('versions', '[scheduleTask]') . ' ' . $number . '</strong></td>
	<td style="background-color:#FFF;"></td>
	<td style="background-color:#FFF;"></td>
</tr>';

				foreach($v as $key => $val){
					$contentDiff .= '<tr><td ><strong>' . g_l('versions', '[' . $key . ']') . '</strong></td><td></td><td >' . we_versions_version::showValue($key, $val, $this->oldDoc['documentTable']) . '</td></tr>';
				}
			}
		} else {
			foreach($newDocScheduler as $k => $v){
				$number = $k + 1;

				$contentDiff .= '<tr>
	<td style="background-color:#FFF; "><strong>' . g_l('versions', '[scheduleTask]') . ' ' . $number . '</strong></td>
	<td style="background-color:#FFF;"></td>' .
					(empty($this->oldDoc) ? '' : '<td style="background-color:#FFF;"></td>') . '
</tr>';


				foreach($v as $key => $val){
					$mark = false;
					$name = g_l('versions', '[' . $key . ']');

					if(!is_array($val)){
						$newVal = we_versions_version::showValue($key, $val, $this->newDoc['documentTable']);

						if(!empty($oldDocScheduler)){
							$oldVal = (isset($oldDocScheduler[$k][$key]) && !is_array($oldDocScheduler[$k][$key]) ?
								we_versions_version::showValue($key, $oldDocScheduler[$k][$key], $this->oldDoc['documentTable']) :
								'');

							if($newVal != $oldVal){
								$mark = true;
							}
						} else {
							$oldVal = '';
						}
					} else {
						$newVal = we_versions_version::showValue($key, $val, $this->newDoc['documentTable']);
						if(!empty($oldDocScheduler)){
							$oldVal = (isset($oldDocScheduler[$k][$key]) && is_array($oldDocScheduler[$k][$key]) ?
								we_versions_version::showValue($key, $oldDocScheduler[$k][$key], $this->oldDoc['documentTable']) :
								'');

							if($newVal != $oldVal){
								$mark = true;
							}
						} else {
							$oldVal = '';
						}
					}

					$contentDiff .= '<tr' . ($mark ? ' class="changedElement"' : '') . '>
	<td ><strong>' . $name . '</strong></td>
	<td >' . $newVal . '</td>' .
						(empty($this->oldDoc) ? '' : '<td>' . $oldVal . '</td>') . '
</tr>';
				}
			}
		}

		return $contentDiff . '</table>';
	}

	private function getElementsDiff(): string{
		$contentDiff = '<table>
<thead>
	<tr><td colspan="3" class="defaultfont bold">' . g_l('versions', '[contentElementsMod]') . '</td></tr>
</thead>';
		if($this->newDoc['documentElements']){
			$newDocElements = we_unserialize($this->newDoc['documentElements']);
		} else {
			$newDocElements = [];
		}

		$oldDocElements = [];

		if(isset($this->oldDoc['documentElements'])){
			if($this->oldDoc['documentElements']){
				$oldDocElements = we_unserialize($this->oldDoc['documentElements']);
			} else {
				$oldDocElements = [];
			}
		}
		if($newDocElements){
			foreach($newDocElements as $name => $v){
				'@phan-var array $v';
				$oldVersion = true;
				//skip this value - it is of no interest; everything is in data
				if($this->isTempl && $name === we_contents_template::FULLCODE){
					continue;
				}

				$newVal = ($name == we_base_variants::ELEMENT_NAME ?
					we_versions_version::showValue($name, $v['dat']) :
					(empty($v['dat']) ? '' : $v['dat'])
					);

				$mark = false;
				if($this->oldDoc){

					$oldVal = ($name === we_base_variants::ELEMENT_NAME ?
						we_versions_version::showValue($name, $oldDocElements[$name]['dat'] ?? '') :
						(empty($oldDocElements[$name]['dat']) ? '' : $oldDocElements[$name]['dat'])
						);

					if($newVal != $oldVal){
						$mark = true;
					}
				} else {
					$oldVersion = false;
					$oldVal = '';
				}

				//make sure none of them is an array
				if(is_array($newVal)){
					$newVal = print_r($newVal, true);
				}
				if(is_array($oldVal)){
					$oldVal = print_r($oldVal, true);
				}

				//if one of them contains newlines, format it as pre-block
				[$pre, $div] = (preg_match("/(%0A|%0D|\\n+|\\r+)/i", $newVal) || preg_match("/(%0A|%0D|\\n+|\\r+)/i", $oldVal) ?
					['<pre class="preview>', ''] :
					['', '<div class="preview">']);

				$contentDiff .= '<tr' . ($mark ? ' class="changedElement"' : '') . '>
<td><strong>' . $name . '</strong></td>';
				if($pre){
					$oldVal = explode("\n", str_replace("\r", "\n", str_replace("\r\n", "\n", $oldVal)));
					$newVal = explode("\n", str_replace("\r", "\n", str_replace("\r\n", "\n", $newVal)));
					$diff = new Horde_Text_Diff('Native', [($oldVersion ? $oldVal : []), is_array($newVal) ? $newVal : []]);
					$renderer = new Horde_Text_Diff_Renderer_Inline(['ins_prefix' => '<span class="insA">+<span class="bold insB">', 'ins_suffix' => '</span>+</span>',
						'del_prefix' => '<span class="delA">-<span class="bold delB">-', 'del_suffix' => '</span>-</span>',]);

					$text = $renderer->render($diff);

					$contentDiff .= '<td colspan="2" >' . $pre . $text . '</pre></td>';
				} else {
					if($newVal && $name != we_base_variants::ELEMENT_NAME){
						$newVal = htmlspecialchars($newVal);
					}

					$contentDiff .= '<td>' . $div . $newVal . ($div ? '</div>' : '') . '</td>';
					if($oldVersion){
						if($oldVal && $name != we_base_variants::ELEMENT_NAME){
							$oldVal = htmlspecialchars($oldVal);
						}
						$contentDiff .= '<td>' . $div . $oldVal . ($div ? '</div>' : '') . '</td>';
					}
				}
				$contentDiff .= '</tr>';
			}
		}

		return $contentDiff . '</table>';
	}

	private function getPropChanges(): string{
		if(!($this->isObj || $this->isTempl)){
			//get path of preview-file
			$binaryPathNew = $this->newDoc['binaryPath'] ?:
				fS('SELECT binaryPath FROM ' . VERSIONS_TABLE . ' WHERE binaryPath!="" AND version<' . ((int) $this->newDoc['version']) . ' AND documentTable="' . $this->db->escape(stripTblPrefix($this->newDoc['documentTable'])) . '" AND documentID=' . ((int) $this->newDoc['documentID']) . ' ORDER BY version DESC LIMIT 1');

			$binaryPathOld = ($this->oldDoc ?
				($this->oldDoc['binaryPath'] ?:
				fS('SELECT binaryPath FROM ' . VERSIONS_TABLE . ' WHERE binaryPath!="" AND version<' . ((int) $this->oldDoc['version']) . ' AND documentTable="' . $this->db->escape(stripTblPrefix($this->oldDoc['documentTable'])) . '" AND documentID=' . ((int) $this->oldDoc['documentID']) . ' ORDER BY version DESC LIMIT 1')) :
				'');
			$filePathNew = WE_VERSION_PATH . $binaryPathNew;
			$fileOld = (!empty($this->oldDoc) && file_exists(WE_VERSION_PATH . $binaryPathOld) ? $binaryPathOld : '');

			$fileNew = (file_exists($filePathNew) ? $binaryPathNew : '');
			$this->contentNew = ($fileNew ?
				'<iframe name="previewNew" src="' . WEBEDITION_DIR . 'we_cmd.php?we_cmd[0]=we_base_showTemp&file=' . str_replace(WEBEDITION_DIR, '', WE_VERSION_DIR . $fileNew) . '" style="border:0;width:100%;height:100%;overflow: hidden;"></iframe>' :
				'<iframe name="previewNew" src="' . WEBEDITION_DIR . 'we_cmd.php?we_cmd[0]=we_versions_showPreview&id=' . $this->newDoc['ID'] . '" style="border:0;width:100%;height:100%;overflow: hidden;"></iframe>');

			$this->contentOld = ($fileOld ? '<iframe name="previewOld" src="' . WEBEDITION_DIR . 'we_cmd.php?we_cmd[0]=we_base_showTemp&file=' . str_replace(WEBEDITION_DIR, '', WE_VERSION_DIR . $fileOld) . '" style="border:0;width:100%;height:100%;overflow: hidden;"></iframe>' :
				($this->oldDoc ?
				'<iframe name="previewOld" src="' . WEBEDITION_DIR . 'we_cmd.php?we_cmd[0]=we_versions_showPreview&id=' . $this->oldDoc['ID'] . '" style="border:0;width:100%;height:100%;overflow: hidden;"></iframe>' :
				'no content'));
		}
		if($this->isTempl){
			$nDocElements = ($this->newDoc['documentElements'] ? we_unserialize($this->newDoc['documentElements']) : []);
			$this->contentNew = '<textarea style="width:99%;height:99%">' . ($nDocElements ? $nDocElements['data']['dat'] : '') . '</textarea>';
			if($this->oldDoc){
				$oDocElements = ($this->oldDoc['documentElements'] ? we_unserialize($this->oldDoc['documentElements'] ?? '') : []);
				$this->contentOld = '<textarea style="width:99%;height:99%">' . ($oDocElements ? $oDocElements['data']['dat'] : '') . '</textarea>';
			}
		}

		$versionOld = (empty($this->oldDoc) ? '' : ' AND version!=' . intval($this->oldDoc['version']));

		$versions_time_days = new we_html_select([
			'name' => 'versions_time_days',
			'style' => '',
			'class' => 'weSelect',
			'onchange' => "previewVersion('" . $this->document['table'] . "'," . $this->document['ID'] . ',' . $this->document['version'] . ", this.value);"
			]
		);

		$versions = $this->db->getAllFirstq('SELECT
			v.version AS ID,
			CONCAT("' . g_l('versions', '[version]') . ' ",v.version," ' . g_l('versions', '[from]') . ' ",DATE_FORMAT(logDate,"' . g_l('weEditorInfo', '[mysql_date_format]') . '")) AS `option`
FROM ' .
			VERSIONS_TABLE . ' v
WHERE
	v.documentID=' . intval($this->newDoc['documentID']) . ' AND
	v.documentTable="' . $this->db->escape(stripTblPrefix($this->newDoc['documentTable'])) . '" AND
	v.version!=' . intval($this->newDoc['version']) . ' ' . $versionOld . '
ORDER BY
	v.version ASC', false);

		$versions_time_days->addOption('', g_l('versions', '[pleaseChoose]'));
		foreach($versions as $k => $v){
			$versions_time_days->addOption((string) $k, (string) $v);
		}

		$contentDiff = we_html_element::div(['id' => "top"], g_l('versions', '[VersionChangeTxt]') . we_html_element::br(2) .
				g_l('versions', '[VersionNumber]') . " " . $versions_time_days->getHtml() .
				we_html_element::div(['style' => "margin:20px 0 0 0;", 'class' => 'defaultfont'], '<a href="javascript:window.print()">' . g_l('versions', '[printPage]') . '</a>')) .
			we_html_element::div(['id' => 'topPrint'], '
<strong>' . g_l('versions', '[versionDiffs]') . ':</strong>' . we_html_element::br(2) . '
<strong>' . g_l('versions', '[Text]') . ':</strong> ' . $this->newDoc['Filename'] . ($this->newDoc['Extension'] ?? '') . we_html_element::br() . '
<strong>' . g_l('versions', '[documentID]') . ':</strong> ' . $this->newDoc['documentID'] . we_html_element::br() . '
<strong>' . g_l('versions', '[path]') . ':</strong> ' . $this->newDoc['Path']
			) . '
			<table class="propDiff">
			<thead><tr>
			<td></td>
	  		<td class="defaultfont bold">' . g_l('versions', '[VersionNew]') . '</td>' .
			(empty($this->oldDoc) ? '' :
			'<td class="defaultfont bold">' . g_l('versions', '[VersionOld]') . '</td>') .
			'</tr></thead>';

		foreach(array_keys($this->newDoc) as $k){
			if(in_array($k, self::DO_NOT_SHOW, true)){
				continue;
			}
			$name = ($k === '_logDate' ? g_l('versions', '[logDate]') : g_l('versions', '[' . $k . ']'));

			$oldVersion = true;
			$newVal = ($k === 'ParentID' ?
				$this->newDoc['Path'] :
				we_versions_version::showValue($k, $this->newDoc[$k], $this->newDoc['documentTable'])
				);

			if($k === 'Owners' && $this->newDoc[$k] == ''){
				$newVal = g_l('versions', '[CreatorID]');
			}

			$mark = false;
			$oldVal = '';
			if(!empty($this->oldDoc)){
				$oldVal = ($k === 'ParentID' ?
					$this->oldDoc['Path'] :
					we_versions_version::showValue($k, $this->oldDoc[$k], $this->oldDoc['documentTable']));

				if($k === 'Owners' && $this->oldDoc[$k] == ''){
					$oldVal = g_l('versions', '[CreatorID]');
				}
				//if(!(in_array($k, self::NOTMARK))){
				if($newVal != $oldVal){
					$mark = true;
				}
				//}
			} else {
				$oldVersion = false;
			}

			$contentDiff .= '<tr' . ($mark ? ' class="changedElement"' : '') . '>
<td><strong>' . $name . '</strong></td>
<td>' . $newVal . '</td>' .
				($oldVersion ? '<td>' . $oldVal . '</td>' : '') .
				'</tr>';
		}

		return $contentDiff . '</table>';
	}

	public function showHtml(): void{
		echo we_html_tools::getHtmlTop('webEdition - ' . g_l('versions', '[versioning]'), we_html_element::cssLink(CSS_DIR . 'we_version_preview.css') .
			we_html_element::cssLink(CSS_DIR . 'we_tab.css') .
			we_html_element::jsScript(JS_DIR . 'initTabs.js') .
			we_html_element::jsScript(JS_DIR . 'versions_preview.js'), we_html_element::body(['class' => 'weDialogBody', 'onload' => "setTab(1);"],
																																										we_html_element::div(['id' => 'eHeaderBody'], $this->getTabsBody()) .
				we_html_element::div(['id' => 'content'],
												 we_html_element::div(['id' => 'tab1', 'style' => "display:block;width:100%;"], $this->contentDiff) .
					we_html_element::div(['id' => 'tab2', 'style' => "display:none;height:100%;width:100%"], ($this->isObj ? '' : $this->contentNew)) .
					we_html_element::div(['id' => 'tab3', 'style' => "display:none;height:100%;width:100%"], ($this->isObj ? '' : $this->contentOld))
				) .
				we_html_element::div(['class' => 'editfooter'], we_html_button::create_button(we_html_button::CLOSE, "javascript:WE().util.dialogTop(window).doClose();"))));
	}

}
