<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 3b8044c13b9c $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-12-16 00:05 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_versions_view{
	/** @var we_database_base */
	private $db;

	/** @var we_versions_search */
	protected $searchclass;

	/** @var we_versions_model */
	protected $Model;

	/** @var int */
	protected $foundItems = 0;

	/**
	 *  Constructor for class 'weVersionsView'
	 */
	public function __construct(we_versions_model $model = null){
		$this->db = new DB_WE();
		$this->Model = $model ?: new we_versions_model(we_base_WE::$transaction);
		$this->searchclass = new we_versions_search($this);
	}

	/**
	 * create javascript-Code for versions-tab
	 * @return string javascript-code
	 */
	public function getJS(): string{

		//add height of each input row to calculate the scrollContent-height
		$h = 0;
//		$addinputRows = '';
		if($this->Model->mode){
			$h += 37;
			/* 		$addinputRows = '
			  for(i=0;i<newID;i++) {
			  scrollheight = scrollheight + 26;
			  }'; */
		}

		return we_html_element::jsScript(JS_DIR . 'versions_view.js', '', ['id' => 'loadVarVersionView',
				'data-searchClass' => setDynamicVar([
					'scrollHeight' => $h,
					'anzahl' => $this->Model->getPropertyInt('currentAnzahl'),
					'searchFields' => we_html_tools::select('searchFields[__we_new_id__]', $this->searchclass->getFields(), 1, '', false, [
						'id' => 'searchFields[__we_new_id__]', 'onchange' => "changeit(this.value, __we_new_id__);"]),
					'locationFields' => we_html_tools::select('location[__we_new_id__]', we_search_search::getLocation(), 1, '', false, ['class' => "defaultfont",
						'disabled' => 'disabled', 'id' => "location[__we_new_id__]"]),
					'locationFieldsDate' => we_html_tools::select('location[__we_new_id__]', we_search_search::getLocation('date'), 1, '', false, [
						'disabled' => 'disabled', 'id' => "location[__we_new_id__]"]),
					'locationFieldsText' => we_html_tools::select('location[__we_new_id__]', we_search_search::getLocation('text'), 1, '', false, [
						'disabled' => 'disabled', 'id' => "location[__we_new_id__]"]),
					'search' => we_html_tools::select('search[__we_new_id__]', we_versions_search::getModFields(), 1, '', false, ['class' => "defaultfont",
						'style' => "width:190px;", 'id' => "search[__we_new_id__]"]),
					'trash' => we_html_button::create_button(we_html_button::TRASH, "javascript:delRow(__we_row__)"),
					'searchUsers' => we_html_tools::select('search[__we_new_id__]', $this->searchclass->getUsers(), 1, '', false, ['class' => "defaultfont",
						'style' => "width:190px;", 'id' => "search[__we_new_id__]"]),
					'searchStats' => we_html_tools::select('search[__we_new_id__]', $this->searchclass->getStats(), 1, '', false, ['class' => "defaultfont",
						'style' => "width:190px;", 'id' => "search[__we_new_id__]"])
					]
				),
				/* 'data-doc' => setDynamicVar([
				  'ID' => intval(we_base_WE::$we_doc->ID),
				  'Table' => we_base_WE::$we_doc->Table,
				  'Text' => we_base_WE::$we_doc->getText(),
				  ]), */
				'data-props' => setDynamicVar([
					'transaction' => we_base_WE::$transaction,
					'rows' => (isset($_REQUEST['searchFields']) && is_array($_REQUEST['searchFields']) ? count((array) $_REQUEST['searchFields']) - 1 : 0)
				])
		]);
	}

	/**
	 * create html-Code for filter-selects
	 * @return string html-Code
	 */
	public function getBodyTop(): string{
		$currentSearch = $this->Model->getProperty('currentSearch');
		$currentSearchFields = $this->Model->getProperty('currentSearchFields');
		$currentLocation = $this->Model->getProperty('currentLocation');

		$out = '<table class="default" id="defSearch" style="width:550px;margin-left:20px;display:' . ($this->Model->mode ? 'none' : 'block') . ';"><tbody>
<tr>
	<td class="weSearchHeadline">' . g_l('versions', '[weSearch]') . '</td>
	<td>' . we_html_button::create_button(we_html_button::DIRRIGHT, "javascript:switchSearch(1)") . '</td>
	<td style="width:100%"></td>
</tr>
</table>
<table class="default" id="advSearch" style="width:550px;margin-left:2em;display:' . ($this->Model->mode ? 'block' : 'none') . ';">
<tr>
	<td class="weSearchHeadline">' . g_l('versions', '[weSearch]') . '</td>
	<td>' . we_html_button::create_button(we_html_button::DIRDOWN, "javascript:switchSearch(0)") . '</td>
	<td style="width:100%"></td>
</tr>
</table>
<table id="advSearch2" style="margin-left:20px;display:' . ($this->Model->mode ? 'block' : 'none') . ';">
<tbody id="filterTable">
<tr>
	<td>' . we_contents_base::hiddenTrans() . '</td>
</tr>';

		for($i = 0; $i < count($currentSearchFields); $i++){

			$button = we_html_button::create_button(we_html_button::TRASH, "javascript:delRow(" . $i . ");", '', '', false, false);
			$search = we_html_tools::select('search[' . $i . ']', we_versions_search::getModFields(), 1, ($currentSearch[$i] ?? ''), false, [
					'style' => 'width:190px;', 'id' => 'search[' . $i . ']']);
			$locationDisabled = '';
			$handle = '';

			if(isset($currentSearchFields[$i])){
				switch($currentSearchFields[$i]){
					case 'allModsIn':
						$search = we_html_tools::select('search[' . $i . ']', we_versions_search::getModFields(), 1, ($currentSearch[$i] ?? ''), false, [
								'style' => "width:190px;", 'id' => 'search[' . $i . ']']);
						$locationDisabled = 'disabled';
						$currentLocation[$i] = we_search_base::OP_IS;
						break;
					case 'modifierID':
						$search = we_html_tools::select('search[' . $i . ']', $this->searchclass->getUsers(), 1, ($currentSearch[$i] ?? ''), false, [
								'style' => "width:190px;", 'id' => 'search[' . $i . ']']);
						$locationDisabled = 'disabled';
						$currentLocation[$i] = we_search_base::OP_IS;
						break;
					case 'status':
						$search = we_html_tools::select('search[' . $i . ']', $this->searchclass->getStats(), 1, ($currentSearch[$i] ?? ''), false, [
								'style' => "width:190px;", 'id' => 'search[' . $i . ']']);
						$locationDisabled = 'disabled';
						$currentLocation[$i] = we_search_base::OP_IS;
						break;
					case 'logDate':
						$locationDisabled = '';
						$handle = 'date';
						$search = we_html_tools::getNewDateInput('search[' . $i . ']', new DateTime($currentSearch[$i] ?? 'now'), we_html_tools::FORMAT_DATE);
				}
			}

			$out .= '
				<tr id="filterRow_' . $i . '">
					<td>' . we_html_tools::select('searchFields[' . $i . ']', $this->searchclass->getFields(), 1, ($currentSearchFields[$i] ?? ''), false, [
					'id' => 'searchFields[' . $i . ']', 'onchange' => "changeit(this.value, " . $i . ");"]) . '</td>
					<td id="td_location[' . $i . ']">' .
				we_html_tools::select('location[' . $i . ']', we_search_search::getLocation($handle), 1, ($currentLocation[$i] ?? ''), false, [
					$locationDisabled => $locationDisabled, 'id' => 'location[' . $i . ']']) . '</td>
					<td id="td_search[' . $i . ']">' . $search . '</td>
					<td id="td_delButton[' . $i . ']">' . $button . '</td>
					<td id="td_hiddenLocation[' . $i . ']">' . (!$locationDisabled ? '' : we_html_element::hidden('location[' . $i . ']', $currentLocation[$i])) . '</td>
				</tr>
				';
		}

		$out .= '</tbody></table>
<table class="default" id="advSearch3" style="margin:10px 0 20px 20px;display:' . ($this->Model->mode ? 'block' : 'none') . ';">
	<tr>
		<td style="width:215px;">' . we_html_button::create_button(we_html_button::ADD, "javascript:newinput();") . '</td>
		<td style="width:155px"></td>
		<td style="width:188px;text-align:right">' . we_html_button::create_button(we_html_button::SEARCH, "javascript:search(true);") . '</td>
		<td></td>
	</tr>
	</table>
	<div style="border-top: 1px solid #AFB0AF;clear:both;"></div>';

		return $out;
	}

	/**
	 * create html-Code for paging on top
	 * @return string html-Code
	 */
	public function getParameterTop(): string{
		$anzahl_all = [10 => 10, 25 => 25, 50 => 50, 100 => 100];

		$order = $this->Model->getProperty('currentOrder');
		$mode = $this->Model->mode;
		$height = $this->Model->height;
		$anzahl = $this->Model->getPropertyInt('currentAnzahl');
		$we_transaction = $this->Model->transaction;
		try{
			$doc = we_base_WE::getWeDoc();
		} catch (Exception $ex){
			$doc = null;
		}
		// FIXME: move to model or init view
		$Text = we_base_request::string('text', $doc ? ($doc->getText() ?: '') : '');
		$ID = we_base_request::int('id', $doc ? $doc->ID : 0);
		$Path = we_base_request::_(we_base_request::FILE, 'path', $doc ? $doc->Path : '/');

		return we_html_element::hiddens([
				'we_transaction' => $we_transaction,
				'order' => (string) ($order),
				'mode' => (string) ($mode),
				'height' => (string) ($height)
			]) .
			'<table class="default defaultfont" style="margin-top:20px;margin-bottom:12px;">
<tr id="beschreibung_print" class="defaultfont">
	<td>
	<strong>' . g_l('versions', '[versions]') . ':</strong><br/>
	<br/><strong>' . g_l('versions', '[Text]') . ':</strong> ' . $Text . '
	<br/><strong>' . g_l('versions', '[documentID]') . ':</strong> ' . $ID . '
	<br/><strong>' . g_l('versions', '[path]') . ':</strong> ' . $Path . '
	</td>
</tr>
 <tr>
	<td></td>
	<td id="eintraege_pro_seite" style="font-size:12px;width:130px;">' . g_l('versions', '[eintraege_pro_seite]') . ':</td>
	<td class="defaultfont lowContrast" style="width:70px;">' .
			we_html_tools::select('anzahl', $anzahl_all, 1, $anzahl, false, ['id' => "anzahl", 'onchange' => "this.form.elements.searchstart.value=0;search(false);"]) . '
	</td>
	<td id="eintraege">' . g_l('versions', '[eintraege]') . '</td>
	<td>' . $this->getNextPrev() . '</td>
	<td id="print" class="defaultfont"><button class="weBtn weIconButton" type="button" onclick="printScreen();" title="' . g_l('versions', '[printPage]') . '"><i class="fa fa-lg fa-print"></i></button></td>
</tr>
</table>';
	}

	/**
	 * create html-Code for paging on bottom
	 * @return string html-Code
	 */
	public function getParameterBottom(bool $setInputSearchstart = false): string{
		return '<table class="default printShow" style="margin-top:20px;margin-left:208px;">
<tr><td>' . $this->getNextPrev($setInputSearchstart) . '</td><td>' . (we_base_permission::hasPerm([we_base_perm::ADMINISTRATOR]) ? we_html_element::span([],
																																																																													we_html_button::create_button(we_html_button::TRASH, "javascript:deleteVers();")) : '') . '</td></tr>
</table>';
	}

	/**
	 * generates html for 'previous' / 'next'
	 * @return string html
	 */
	private function getNextPrev(bool $setInputSearchstart = false): string{
		$anzahl = $this->Model->getPropertyInt('currentAnzahl');
		$searchstart = $this->Model->getPropertyInt('currentSearchstart');
		$we_search_anzahl = $this->foundItems ?: ($_SESSION['weS']['versions']['foundItems']) ?? 0;
		$out = '<span style="display:inline-block">' .
			($searchstart ?
			we_html_button::create_button(we_html_button::BACK, "javascript:back(" . $anzahl . ");") :
			we_html_button::create_button(we_html_button::BACK, "", '', "", true)) .
			'<b>' . (($we_search_anzahl) ? $searchstart + 1 : 0) . '-' .
			(($we_search_anzahl - $searchstart) < $anzahl ?
			$we_search_anzahl :
			$searchstart + $anzahl) .
			' ' . g_l('global', '[from]') . ' ' . $we_search_anzahl . '</b> ' .
			(($searchstart + $anzahl) < $we_search_anzahl ?
			we_html_button::create_button(we_html_button::NEXT, "javascript:next(" . $anzahl . ");") : //bt_back
			we_html_button::create_button(we_html_button::NEXT, "", '', "", true));

		$pages = [];
		for($i = 0; $i < ceil($we_search_anzahl / $anzahl); $i++){
			$pages[($i * $anzahl)] = ($i + 1);
		}

		$page = ceil($searchstart / $anzahl) * $anzahl;

		$select = we_html_tools::select('page', $pages, 1, $page, false, ['id' => 'pageselect', 'onchange' => "this.form.elements.searchstart.value=this.value;search(false);"]);

		if(!$setInputSearchstart && !defined('searchstart')){
			define('searchstart', true);
			$out .= we_html_element::hidden('searchstart', $searchstart);
		}

		$out .= $select .
			'</span>';

		return $out;
	}

	/**
	 * generates content for versions found
	 * @return array with content
	 */
	private function getVersionsOfDoc(): array{
		// FIXME: move to model or init view
		$id = we_base_request::int('id', we_base_WE::$we_doc ? we_base_WE::$we_doc->ID : 0);
		$table = we_base_request::_(we_base_request::TABLE, 'table', we_base_WE::$we_doc ? we_base_WE::$we_doc->Table : FILE_TABLE);
		$where = $this->searchclass->getWhere($this->Model);

		$sort = explode(' ', $this->Model->getProperty('currentOrder'));
		switch($sort[0]){
			case 'modifierID':
				$order = $sort[0] . ' ' . ($sort[1] ?? 'ASC') . ', version DESC';
				break;
			case 'logDate':
			case 'version':
				$order = $sort[0] . ' ' . ($sort[1] ?? 'ASC');
				break;
			default:
				$order = 'version DESC';
				break;
		}

		$versions = we_versions_version::loadVersionsOfId($id, $table, $where, $order);
		$_SESSION['weS']['versions']['foundItems'] = count($versions);
		if(empty($versions)){
			return [];
		}

		$content = [];
		foreach($versions as $curVersion){
			'@phan-var array{modifierID:int,modifiedFields:string,status:string,version:int,active:int,src:string,resetFromVersion:int,ID:int,documentID:int,version:int,documentTable:string,ContentType:string,_logDate:string} $curVersion';
			$modificationText = $this->getTextForMod((string) $curVersion['modifiedFields'], (string) $curVersion['status']);
			$user = $curVersion['modifierID'] ? id2path((int) $curVersion['modifierID'], USER_TABLE) : g_l('versions', '[unknown]');
			$vers = $curVersion['version'];
			$disabledReset = ($curVersion['active'] === 1);
			if(!we_base_permission::hasPerm([we_base_perm::VERSIONS_RESET])){
				$disabledReset = true;
			}
			$fromScheduler = $fromImport = '';
			switch($curVersion['src'] ?? ''){
				case 'fromScheduler':
					$fromScheduler = g_l('versions', '[fromScheduler]');
					break;
				case 'fromImport':
					$fromImport = g_l('versions', '[fromImport]');
					break;
			}

			$resetFromVersion = ($curVersion['resetFromVersion']) ? '--' . g_l('versions', '[resetFromVersion]') . $curVersion['resetFromVersion'] . '--' : '';

			$content[] = [
				$vers,
				we_base_util::shortenPath($user, 30),
				($curVersion['_logDate'] ?: '-'),
				($modificationText != '' ? $modificationText : '') .
				$fromScheduler .
				$fromImport .
				$resetFromVersion,
				(we_base_permission::hasPerm([we_base_perm::ADMINISTRATOR]) ? we_html_forms::checkbox((string) $curVersion['ID'], false, 'deleteVersion', '', false, 'defaultfont', '') : ''),
				we_html_element::span(['class' => 'printShow'], we_html_button::create_button(we_html_button::RESET_VERSION, "javascript:resetVersion('" . $curVersion['ID'] . "','" . $curVersion['documentID'] . "','" . $curVersion['version'] . "','" . $curVersion['documentTable'] . "');", '', "", $disabledReset)),
				we_html_element::span(['class' => 'printShow'], we_html_button::create_button(we_html_button::PREVIEW, "javascript:previewVersion('" . $table . "'," . $id . ',' . $curVersion['version'] . ");")),
				we_html_element::span(['class' => 'printShow'],
													(($curVersion['ContentType'] == we_base_ContentTypes::WEDOCUMENT || $curVersion['ContentType'] == we_base_ContentTypes::HTML || $curVersion['ContentType'] === we_base_ContentTypes::OBJECT_FILE) ?
					we_html_forms::checkbox((string) $curVersion['ID'], false, 'publishVersion_' . $curVersion['ID'], g_l('versions', '[publishIfReset]'), false, 'middlefont', '') :
					'')),
			];
		}

		return $content;
	}

	/**
	 * generates headline-titles for columns
	 * @return string[] with headlines
	 */
	public function makeHeadLines(): array{
		return [
			'<span onclick="setOrder(\'version\');">' . g_l('versions', '[version]') . ' <span id="version" >' . $this->getSortImage('version') . '</span></span>',
			'<span onclick="setOrder(\'modifierID\');">' . g_l('versions', '[user]') . ' <span id="modifierID" >' . $this->getSortImage('modifierID') . '</span></span>',
			'<span onclick="setOrder(\'logDate\');">' . g_l('versions', '[modTime]') . '</a> <span id="logDate" >' . $this->getSortImage('logDate') . '</span></span>',
			g_l('versions', '[modifications]'),
			we_html_button::create_button(we_html_button::TOGGLE, "javascript:checkAll();", '', '', false, false, '', false, g_l('versions', '[mark]')),
			'',
			'',
			'',
			'',
		];
	}

	/**
	 * generate html list for modifications
	 * @return string
	 */
	private function getTextForMod(string $modString, string $status): string{
		$statusTxt = ($status === 'published' ? we_html_element::span(['style' => "color:#ff0000;"], g_l('versions', '[' . $status . ']')) : '');
		if($modString == ''){
			return we_html_element::div([], $statusTxt);
		}

		$out = '';
		$modifications = explode(',', $modString);
		foreach($modifications as $key){
			$out .= '<li><strong>' . g_l('versions', '[' . $key . ']') . '</strong></li>';
		}

		return we_html_element::div([], ($out ? '<ul>' . $out . '</ul>' : '') . $statusTxt);
	}

	/**
	 * generate html for SortImage
	 * @return string
	 */
	private function getSortImage(string $for): string{
		$order = $this->Model->getProperty('currentOrder');

		if(str_starts_with($order, $for)){
			if(str_contains($order, 'DESC')){
				return '<i class="fa fa-sort-desc fa-lg"></i>';
			}
			return '<i class="fa fa-sort-asc fa-lg"></i>';
		}

		return '<i class="fa fa-sort fa-lg"></i>';
	}

	/**
	 * @var array $content
	 * @var string[] $headline
	 * generate html for version list
	 * @return string
	 */
	public function tblList(): string{
		$headline = $this->makeHeadLines();
		//$anz = count($headline) - 1;
		return we_html_element::div(['id' => 'scrollContent'],
															'<table class="contentTable" cellpadding="5" data-bla="hoi">
	<thead>
		<tr id="contentTable_headTR" class="middlefont">
			<th style="min-width:70px;">' . $headline[0] . '</th>
			<th style="min-width:70px;">' . $headline[1] . '</th>
			<th style="min-width:110px;">' . $headline[2] . '</th>
			<th style="min-width:65px;">' . $headline[4] . '</th>
			<th>' . $headline[3] . '</th>
			<th></th>
		</tr>
	</thead><tbody style="background-color:#fff;width:100%" id="contentTbody">' .
				$this->tabListContent() .
				'</tbody></table>'
		);
	}

	public function tabListContent(): string{
		$content = $this->getVersionsOfDoc();
		$searchstart = $this->Model->getPropertyInt('currentSearchstart');
		$anzahl = $this->Model->getPropertyInt('currentAnzahl');

		$this->foundItems = count($content);
		$x = $searchstart + $anzahl;

		if($x > $this->foundItems){
			$x -= ($x - $this->foundItems);
		}
		$out = '';
		for($m = $searchstart; $m < $x; $m++){
			$out .= '<tr class="middlefont">' . self::tblListRow($content[$m]) . '</tr>';
		}

		return $out;
	}

	private static function tblListRow(array $content): string{
		return '<td style="min-width:70px;">' . $content[0] . '</td>
<td style="min-width:70px;">' . $content[1] . '</td>
<td style="min-width:110px;">' . $content[2] . '</td>
<td style="min-width:65px;">' . $content[4] . '</td>
<td>' . $content[3] . '</td>
<td>' . $content[6] . $content[5] . $content[7] . '</td>';
	}

	/**
	 *
	 * @param string[] $content
	 * @return string
	 */
	public function getHTMLforVersions(array $content): string{
		$uniqname = md5(uniqid(__FUNCTION__, true));
		$out = '';

		foreach($content as $i => $c){
			$out .= we_html_element::div(['id' => 'div_' . $uniqname . '_' . $i],
																we_html_element::div([], $c));
		}

		return $out;
	}

}
