<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: e4301c9e291b $
 * $Author: Marc Krämer <marc.kraemer@aldente-it.de> $
 * $Date: 2019-11-13 12:33 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */

/**
 * Baseclass for all UI relevant functions
 * all functions not found in extended classes will be redirected to their corresponding document which holds the data
 */
abstract class we_view_base{
	protected $doc;

	public function __call(string $name, array $params){
		t_e('call, use $this->doc->', $name);
		return $this->doc->$name(...$params);
	}

	public function __get(string $name){
		t_e('get, use $this->doc->', $name);
		return $this->doc->$name;
	}

	public function __set(string $name, /* mixed */ $value): void{
		t_e('set, use $this->doc->', $name);
		$this->doc->$name = $value;
	}

	public function __isset(string $name): bool{
		t_e('isset, use $this->doc->', $name);
		return isset($this->doc->$name);
	}

	public static function getView(we_base_mainmodel $doc): we_view_base{
		$cnt = 0;
		$class = get_class($doc);
		$nClass = str_replace(['_contents_', '_document_'], '_view_', $class, $cnt);
		if(!$cnt){
			$nClass = str_replace('we_', 'we_view_', $class, $cnt);
		}
		if($cnt && class_exists($nClass)){
			return new $nClass($doc);
		}
		throw new we_exception_base('no view for doc found ' . $class);
	}

	protected function __construct(we_base_mainmodel $doc){
		$this->doc = $doc;
	}

}
