<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 8fe07b552f5a $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-12-10 15:55 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_xml_browser extends we_xml_parser{
	/** @var string */
	private $cache;

	public function __construct(string $filename = '', string $mode = 'backup'){
		$this->mode = $mode;
		$this->xmlExt = false;
		parent::__construct($filename);
	}

	public function saveCache(string $cache = '', int $expire = 0): bool{
		if(!$cache){
			$cache = $this->cache;
		} else {
			$this->cache = $cache;
		}
		$expire = $expire ?: 1800;

		if(!is_dir(dirname($cache))){
			we_base_file::createLocalFolderByPath(dirname($cache));
		}
		try{
			we_base_file::save($cache, we_serialize([
				'encoding' => $this->mainXmlEncoding,
				'nodes' => $this->nodes
					], SERIALIZE_PHP, false, 6));
			we_base_file::insertIntoCleanUp($cache, $expire);
		} catch (we_exception_file $e){
			//save failed
			return false;
		}
		return true;
	}

	public function loadCache(string $cache = ''): void{
		if(empty($cache)){
			$cache = $this->cache;
		} else {
			$this->cache = $cache;
		}
		$data = we_unserialize(we_base_file::load($cache));
		$this->mainXmlEncoding = $data['encoding'];
		$this->nodes = $data['nodes'];
	}

	public function getFile(string $file, int $timeout = 0): bool{
		$this->fileName = $file;

		switch((we_base_file::hasURL($file) ? we_http_util::getHttpOption() : 'local')){
			case 'fopen':
			case 'curl':
				$m = [];
				$pattern = '/^(((ht|f)tp(s?):\/\/)|(www\.))+(([a-zA-Z0-9\._-]+\.[a-zA-Z]{2,6})|([0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}))(\/[a-zA-Z0-9\&amp;%_\.\/-~-]*)?/i';
				if(!preg_match($pattern, $file, $m)){
					return false;
				}

				try{
					$content = we_http_util::getHTTP(str_replace($m[9], '', $file), $m[9], 0, '', '', $timeout);
				} catch (we_exception_http $e){
					return false;
				}
				break;
			case 'local':
				$content = file_get_contents($file);
				break;
			default:
				// has to be implemented
				return false;
		}

		if($content){
			$encoding = $this->setEncoding('', $content);
			return $this->parseXML($content, $encoding);
		}
		return false;
	}

}
