/* global CodeMirror,define */

/**
 * webEdition CMS
 *
 * $Rev: 6329b796be62 $
 * $Author: Lukas Imhof $
 * $Date: 2020-11-10 16:09 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */

/*taken in parts from https://github.com/coderaiser/cm-show-invisibles*/
'use strict';

(function (mod) {

	if (typeof exports === 'object' && typeof module === 'object') { // CommonJS
		mod(require('../../we/libraries/codemirror'));
	} else if (typeof define === 'function' && define.amd) { // AMD
		define(['../../we/libraries/codemirror'], mod);
	} else {
		mod(CodeMirror);
	}
})(function (CodeMirror) {

	CodeMirror.defineOption('showInvisibles', false, function (cm, val, prev) {
		var Count = 0,
			Maximum = cm.getOption('maxInvisibles') || 16;

		if (prev === CodeMirror.Init) {
			prev = false;
		}
		if (prev && !val) {
			cm.removeOverlay('invisibles');
		} else if (!prev && val) {
			add(Maximum);

			cm.addOverlay({
				name: 'invisibles',
				token: function nextToken(stream) {
					var ret,
						spaces = 0,
						peek = stream.peek() === ' ';

					if (peek) {
						while (peek && spaces < Maximum) {
							++spaces;

							stream.next();
							peek = stream.peek() === ' ';
						}

						ret = 'whitespace whitespace-' + spaces;

						/*
						 * styles should be different
						 * could not be two same styles
						 * beside because of this check in runmode
						 * function in `codemirror.js`:
						 *
						 * 6624: if (!flattenSpans || curStyle != style) {}
						 */
						if (spaces === Maximum) {
							ret += ' whitespace-rand-' + Count++;
						}
					} else {
						while (!stream.eol() && !peek) {
							stream.next();

							peek = stream.peek() === ' ';
						}

						ret = 'cm-eol';
					}

					return ret;
				}
			});
		}
	});

	function add() {
		document.getElementsByClassName('CodeMirror')[0].classList.add("showInvisibles");
	}

});