/* global WE, errorHandler,treeData,drawTree, top */

/**
 * webEdition CMS
 *
 * $Rev: 1c20ce8e74f4 $
 * $Author: Marc Kraemer <marc.kraemer@webedition.org> $
 * $Date: 2020-12-21 12:11 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
'use strict';
WE().util.getTreeIcon = function (contentType, open, extension) {
	var simplepre = '<span class="fa-stack fa-lg fileicon">';
	var pre = simplepre + '<i class="fa fa-file fa-inverse fa-stack-2x fa-fw"></i>',
					post = '</span>',
					applicationExtensions = ['.pdf', '.zip', '.sit', '.hqx', '.bin', '.odg', '.otg',
						'.odt', '.ott', '.dot', '.doc', '.ods', '.ots', '.xlt', '.xls', '.odp', '.otp', '.ppt'];

	if (!contentType && extension && applicationExtensions.indexOf(extension) !== -1) {
		contentType = WE().consts.contentTypes.APPLICATION;
	}

	switch (contentType) {
		case 'cockpit':
			return simplepre + '<i class="fa fa-th-large fa-stack-2x"></i>' + post;
		case WE().consts.contentTypes.FOLDER_COMMON:
		case WE().consts.contentTypes.FOLDER_DOCUMENT:
		case WE().consts.contentTypes.FOLDER_OBJECTFILE:
		case WE().consts.contentTypes.FOLDER_TEMPLATE:
		case WE().consts.contentTypes.FOLDER_VFILE:
			return simplepre + '<i class="fa fa-folder' + (open ? '-open' : '') + ' fa-stack-2x"></i><i class="fa fa-folder' + (open ? '-open' : '') + '-o fa-stack-2x"></i>' + post;
		case WE().consts.contentTypes.FOLDER_PHPCLASS:
			return pre + '<i class="fa fa-folder' + (open ? '-open' : '') + ' fa-stack-2x"></i><span class="we-icon"><i class="fa fa-circle fa-stack-1x"></i><i class="fa fa-stack-1x fa-inverse">e</i></span><span class="we-classification"><i class="fa fa-stack-1x">P</i></span>' + post;
		case 'weThumbnail':
		case  WE().consts.contentTypes.IMAGE:
			return pre + '<i class="fa fa-file-image-o fa-stack-2x we-color"></i><i class="fa fa-file-o fa-stack-2x"></i>' + post;
		case 'text/js':
			return pre + '<i class="fa fa-file-o fa-stack-2x"></i><span class="we-otherfiles"><i class="fa fa-stack-1x">js</i></span>' + post;
		case 'text/css':
			return pre + '<i class="fa fa-file-o fa-stack-2x"></i><span class="we-otherfiles"><i class="fa fa-stack-1x">cs</i></span>' + post;
		case 'text/htaccess':
			return pre + '<i class="fa fa-file-o fa-stack-2x"></i><span class="we-otherfiles"><i class="fa fa-stack-1x">ht</i></span>' + post;
		case WE().consts.contentTypes.TEMPLATE:
			return pre + '<i class="fa fa-file-o fa-stack-2x"></i><span class="we-icon"><i class="fa fa-circle fa-stack-1x"></i><i class="fa fa-stack-1x fa-inverse">e</i></span><span class="we-classification"><i class="fa fa-stack-1x">T</i></span>' + post;
		case WE().consts.contentTypes.PHPCLASS:
			return pre + '<i class="fa fa-file-o fa-stack-2x"></i><span class="we-icon"><i class="fa fa-circle fa-stack-1x"></i><i class="fa fa-stack-1x fa-inverse">e</i></span><span class="we-classification"><i class="fa fa-stack-1x">P</i></span>' + post;
		case WE().consts.contentTypes.WEDOCUMENT:
			return pre + '<i class="fa fa-file-text-o fa-stack-2x"></i><span class="we-icon"><i class="fa fa-circle fa-stack-1x"></i><i class="fa fa-stack-1x fa-inverse">e</i></span>' + post;
		case WE().consts.contentTypes.XML:
		case WE().consts.contentTypes.HTML:
			return pre + '<i class="fa fa-file-code-o fa-stack-2x we-color"></i><i class="fa fa-file-o fa-stack-2x"></i>' + post;
		case 'video/*':
			return pre + '<i class="fa fa-file-video-o fa-stack-2x we-color"></i><i class="fa fa-file-o fa-stack-2x"></i>' + post;
		case 'audio/*':
			return pre + '<i class="fa fa-file-sound-o fa-stack-2x we-color"></i><i class="fa fa-file-o fa-stack-2x"></i>' + post;
		case 'text/plain':
			return pre + '<i class="fa fa-file-text-o fa-stack-2x we-color"></i><i class="fa fa-file-o fa-stack-2x"></i>' + post;
		case 'file':
		case 'application/pdf':
			return pre + '<i class="fa fa-file-pdf-o fa-stack-2x we-color"></i><i class="fa fa-file-o fa-stack-2x"></i>' + post;
		case WE().consts.contentTypes.APPLICATION:
			switch (extension) {
				case '.pdf':
					return pre + '<i class="fa fa-file-pdf-o fa-stack-2x we-color"></i><i class="fa fa-file-o fa-stack-2x"></i>' + post;
				case '.zip' :
				case '.sit' :
				case '.hqx' :
				case '.bin' :
					return pre + '<i class="fa fa-file-archive-o fa-stack-2x we-color"></i><i class="fa fa-file-o fa-stack-2x"></i>' + post;
				case '.odg':
				case '.otg':
				case '.odt':
				case '.ott':
				case '.dot' :
				case '.doc' :
					return pre + '<i class="fa fa-file-word-o fa-stack-2x we-color"></i><i class="fa fa-file-o fa-stack-2x"></i>' + post;
				case '.ods':
				case '.ots':
				case '.xlt' :
				case '.xls' :
					return pre + '<i class="fa fa-table fa-stack-1x we-color"></i><i class="fa fa-file-o fa-stack-2x"></i>' + post;
				case '.odp':
				case '.otp':
				case '.ppt' :
					return pre + '<i class="fa fa-line-chart fa-stack-1x we-color"></i><i class="fa fa-file-o fa-stack-2x"></i>' + post;
				default:
					return pre + '<i class="fa fa-file-o fa-stack-2x"></i>' + post;
			}
			return '';
		case WE().consts.contentTypes.OBJECT:
			return pre + '<i class="fa fa-file-o fa-stack-2x"></i><span class="we-icon"><i class="fa fa-circle fa-stack-1x"></i><i class="fa fa-stack-1x fa-inverse">e</i></span><span class="we-classification"><i class="fa fa-stack-1x">C</i></span>' + post;
		case WE().consts.contentTypes.OBJECT_FILE:
			return pre + '<i class="fa fa-file-o fa-stack-2x"></i><span class="we-icon"><i class="fa fa-circle fa-stack-1x"></i><i class="fa fa-stack-1x fa-inverse">e</i></span><span class="we-classification"><i class="fa fa-stack-1x">O</i></span>' + post;
		case 'text/weCollection':
			return simplepre + '<i class="fa fa-archive fa-stack-2x we-color"></i>' + post;
		case WE().consts.contentTypes.CUSTOMERGROUP:
		case WE().consts.contentTypes.USERGROUP:
			return simplepre + '<i class="fa fa-users fa-stack-2x we-color"></i>' + post;
		case WE().consts.contentTypes.USERALIAS:
			return simplepre + '<i class="fa fa-user fa-stack-2x" style="color:grey"></i>' + post;
		case WE().consts.contentTypes.CUSTOMER:
		case WE().consts.contentTypes.USER:
			return simplepre + '<i class="fa fa-user fa-stack-2x we-color"></i>' + post;
		case 'we/export':
			return pre + '<i class="fa fa-download fa-stack-2x we-color"></i><i class="fa fa-file-o fa-stack-2x"></i>' + post;
		case WE().consts.contentTypes.GLOSSAR:
			return simplepre + '<i class="fa fa-commenting fa-stack-2x we-color"></i>' + post;
		case WE().consts.contentTypes.NEWSLETTER:
			return simplepre + '<i class="fa fa-newspaper-o fa-stack-2x we-color"></i>' + post;
		case WE().consts.contentTypes.VOTING:
			return simplepre + '<i class="fa fa-thumbs-up fa-stack-2x we-color"></i>' + post;
		case WE().consts.contentTypes.NAVIGATION:
			return simplepre + '<i class="fa fa-compass fa-stack-2x we-color"></i>' + post;
		case 'we/search':
			return pre + '<i class="fa fa-search fa-stack-1x we-color"></i><i class="fa fa-file-o fa-stack-2x"></i>' + post;
		case 'we/shop':
			return simplepre + '<i class="fa fa-shopping-cart fa-stack-2x we-color"></i></i>' + post;
		case 'we/workflow':
			return simplepre + '<i class="fa fa-fa-gears fa-stack-2x we-color"></i></i>' + post;
		case WE().consts.contentTypes.CATEGORIES:
			return simplepre + '<i class="fa fa-tags fa-stack-2x we-color"></i>' + post;
		case WE().consts.contentTypes.CATEGORY:
			return simplepre + '<i class="fa fa-tag fa-stack-2x we-color"></i>' + post;
		case 'symlink':
			return pre + '<i class="fa fa-link fa-stack-2x we-color"></i>' + post;
		case 'settings':
			return simplepre + '<i class="fa fa-list fa-stack-2x we-color"></i>' + post;
		case 'we/doctype':
			return pre + '<i class="fa fa-flask fa-stack-2x ' + contentType + '"></i>' + post;
		default:
			return pre + '<i class="fa fa-file-o fa-stack-2x ' + contentType + '"></i>' + post;
	}
};