/* global tinymce */

/**
 * webEdition CMS
 *
 * $Rev: 364f7e19e0d0 $
 * $Author: Marc Kraemer <marc.kraemer@webedition.org> $
 * $Date: 2020-12-14 15:24 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
'use strict';
var weFocusedField; // set inside LinkDialog!

var LinkDialog = {
	tinymcePlugin: window.tinyMCEPopup,
	editableSelects: window.TinyMCE_EditableSelects,
	tinymce: window.tinymce,
	editor: window.tinyMCEPopup.editor,

	init: function () {
		//	tinyMCEPopup.resizeToInnerSize();

		var form = document.forms.we_form;
		var elm = this.editor.selection.getNode();
		var action = "insert";
		var html;

		if ((html = this.getAnchorListHTML('anchorlist', 'we_dialog_args[anchor]'))) {
			form['we_dialog_args[anchor]'].style.width = '190px';
			document.getElementById("anchorlistcontainer").innerHTML = html;
		}

		elm = this.editor.dom.getParent(elm, "A");
		if (elm !== null && elm.nodeName === "A") {
			action = "update";
		}
		document.getElementById('we_dialog_args[cssclass]').innerHTML = this.tinymcePlugin.getClassOptions('a');

		if (action === "update" && this.editor.isWeLinkInitialized === false && form) {
			this.editor.isWeLinkInitialized = true;
			for (var i of ['title', 'target', 'rel', 'lang', 'hreflang']) {
				if (form.elements['we_dialog_args[' + i + ']']) {
					form.elements['we_dialog_args[' + i + ']'].value = this.editor.dom.getAttrib(elm, i);
				}
			}
			if (form.elements['we_dialog_args[accesskey]']) {
				form.elements['we_dialog_args[accesskey]'].value = this.editor.dom.getAttrib(elm, 'accesskey', elm.accesskey !== undefined ? elm.accesskey : "");
			}
			if (form.elements['we_dialog_args[tabindex]']) {
				form.elements['we_dialog_args[tabindex]'].vaue = this.editor.dom.getAttrib(elm, 'tabindex', elm.tabindex !== undefined ? elm.tabindex : "");
			}
		}
	},

	writeBack: function (attribs) {
		var elem, elementArray, i;
		var editor = this.editor;

		elem = this.editor.selection.getNode();
		elem = this.editor.dom.getParent(elem, "A");

		// Remove element if there is no href
		if (!attribs.internalLink) {
			i = this.editor.selection.getBookmark();
			this.editor.dom.remove(elem, 1);
			this.editor.selection.moveToBookmark(i);
			this.editor.execCommand("mceEndUndoLevel");
			WE().util.dialogTop(window).doClose();
			return;
		}

		// Create new anchor elements
		if (elem === null) {
			this.editor.getDoc().execCommand("unlink", false, null);
			this.editor.execCommand("mceInsertLink", false, "#mce_temp_url#", {skip_undo: 1});
			elementArray = this.tinymce.grep(this.editor.dom.select("a"), function (n) {
				return editor.dom.getAttrib(n, 'href') === '#mce_temp_url#';
			});
			for (i = 0; i < elementArray.length; i++) {
				this.setAllAttribs(elem = elementArray[i], attribs);
			}
		} else {
			this.setAllAttribs(elem, attribs);
		}

		// Don't move caret if selection was image
		if (elem.childNodes.length !== 1 || elem.firstChild.nodeName !== 'IMG') {
			this.editor.focus();
			this.editor.selection.select(elem);
			this.editor.selection.collapse(0);
			this.tinymcePlugin.storeSelection();
		}

		this.editor.execCommand("mceEndUndoLevel");
		WE().util.dialogTop(window).doClose();
	},

	// FIXME: doesn't work
	getAnchorListHTML: function (selId, target) {
		var html = '', id;

		this.tinymce.each(this.editor.dom.select('a'), function (node) {
			if ((node.getAttribute('id') || node.getAttribute('name')) && !node.getAttribute('href')) {
				id = node.getAttribute('id') || node.getAttribute('name');
				html += '<option value="' + id + '">' + id + '</option>';
			}

			if (node.classList.contains('mce-item-anchor')) {
				if ((id = node.id ? node.id : (node.name ? node.name : ''))) {
					html += '<option value="' + id + '">' + id + '</option>';
				}
			}
		});

		return !html ? '' : '<select id="' + selId + '" name="' + selId + '" class="defaultfont" style="width:100px"' +
						' onchange="this.form.elements[\'' + target + '\'].value=this.options[this.selectedIndex].value;this.selectedIndex=0;"' + '>' +
						'<option value=""></option>' +
						html +
						'</select>';
	},

	setAllAttribs: function (elem, attribs) {
		this.setAttrib(elem, 'href', attribs.internalLink);
		for (var i of ['title', 'target', 'id', 'rel', 'lang', 'hreflang', 'tabindex', 'accesskey']) {
			if (attribs[i]) {
				this.setAttrib(elem, i, attribs[i]);
			}
		}
	},

	setAttrib: function (elem, attrib, value) {
		var dom = this.editor.dom;
		switch (attrib) {
			case 'style':
				value = dom.serializeStyle(dom.parseStyle(value), 'a');
				break;
			case 'href':
				value = value.replace(/ /g, '%20');
				break;
			case 'target':
				if (value === '_self') {
					value = '';
				}
				break;
		}

		dom.setAttrib(elem, attrib, value);
	},

	selectOptionByValue: function (form, selName, val) {
		if (form === undefined || form.elements[selName] === undefined && val === undefined) {
			return;
		}

		var i;
		if (val === '') {
			form.elements[selName].options[0].selected = true;
			for (i = 1; i < form.elements[selName].options.length; i++) {
				form.elements[selName].options[i].selected = false;
			}
		} else {
			var found = false;
			for (i = 1; i < form.elements[selName].options.length; i++) {
				if (form.elements[selName].options[i].value === val) {
					form.elements[selName].options[i].selected = true;
					found = true;
				} else {
					form.elements[selName].options[i].selected = false;
				}
			}
			if (!found) {
				form.elements[selName].options[i] = new Option('--------------------------------------', '');
				form.elements[selName].options[i + 1] = new Option(val, val);
				form.elements[selName].options[i + 1].selected = true;
			}
		}
	}
};

//window.tinyMCEPopup.onInit.add(LinkDialog.init);
$(function () {
	LinkDialog.init();
});
